<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\Registry\Registry;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\CMS\Table\Table;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Form\Form;
use Joomla\CMS\Cache\Cache;

class JFBConnectControllerAjax extends JFBConnectController
{
    function scsocialprovider()
    {
        $input = Factory::getApplication()->input;
        $modProvider = $input->get('provider');
        $widgets = JFBCFactory::getAllWidgets($modProvider);

        $options = array();
        $options[] = HTMLHelper::_('select.option', "widget", "--Select your widget--");
        foreach ($widgets as $widget)
        {
            $options[] = HTMLHelper::_('select.option', $widget->getSystemName(), $widget->getName());
        }
        $registry = $this->getModuleSettings($input->getInt('id'));

        echo HTMLHelper::_('select.genericlist', $options, 'jform[params][widget_type]', 'onchange="jfbcAdmin.scsocialwidget.fetchSettings(this.value);"', 'value', 'text', $registry->get('widget_type'));
        if ($registry->get('widget_type') != "widget")
            echo "<script>jfbcAdmin.scsocialwidget.fetchSettings('" . $registry->get('widget_type') . "');</script>";

        exit;
    }

    function scsocialwidget()
    {
        $input = Factory::getApplication()->input;
        $provider = $input->get('provider');
        $widget = $input->get('name');

        // load the name form.xml file here
        JFBConnectUtilities::loadLanguage('com_jfbconnect', JPATH_ADMINISTRATOR);
        JFBConnectUtilities::loadLanguage('plg_jfbconnect_provider_' . strtolower($provider), JPATH_ADMINISTRATOR);

        $xmlPath = JPATH_ROOT . '/plugins/jfbconnect/provider_'. $provider.'/provider/'. $provider . '/widget';
        Form::addFieldPath($xmlPath);
        $xmlFile = $xmlPath.'/'.$widget.'.xml';
        if(File::exists($xmlFile))
        {
            $options = array('control' => 'jform');
            $form = Form::getInstance('com_jfbconnect_' . $widget, $xmlFile, $options);

            $registry = $this->getModuleSettings($input->getInt('id'));

            $form->bind($registry);
            foreach ($form->getFieldsets() as $fiedsets => $fieldset)
            {
                foreach ($form->getFieldset($fieldset->name) as $field)
                    $this->formShowField($field);
            }
        }
        exit;
    }

    function getModuleSettings($id)
    {
        $table = Table::getInstance('Module', 'Joomla\\CMS\Table\\', array());
        // Attempt to load the row.
        $return = $table->load($id);
        $registry = new Registry;
        if ($return)
        {
            $table->getProperties(1);
            $registry->loadString($table->params);
            $registry->set('params.widget_settings', $registry->get('widget_settings'));
            $registry->set('widget_settings', null);
        }
        return $registry;
    }

    public function formShowField($field)
    {
        echo "  <div class=\"control-group\">\n";
        echo "    <div class=\"control-label\">\n";
        echo "   " . $field->label . "\n";
        echo "   </div>\n";
        echo "     <div class=\"controls\">\n";
        echo "       " . $field->input . "\n";
        echo "     </div>\n";
        echo "  </div>\n";
    }

    public function channelGetOutboundChannels()
    {
        $input = Factory::getApplication()->input;
        $p = $input->getString('provider');
        $options = array();
        $options[] = HTMLHelper::_('select.option', "--", "-- Select a Channel --");

        $provider = JFBCFactory::provider($p);
        $channels = $provider->getChannelsOutbound();
        foreach ($channels as $c)
        {
            $options[] = HTMLHelper::_('select.option', strtolower($c->name), $c->name);
        }
        echo HTMLHelper::_('select.genericlist', $options, 'jform[type]', 'class="custom-select" onchange="jfbcAdmin.channels.outbound.fetchChannelSettings(this.value);"', 'value', 'text', '--', 'jform_type');

        exit;
    }

    public function channelGetOutboundChannelSettings()
    {
        $input = Factory::getApplication()->input;
        $p = $input->getString('provider');
        $c = $input->getString('channel');
        $options = array();
        $options[] = HTMLHelper::_('select.option', "--", "-- Select a Channel --");

        $pluginPath = JPATH_SITE . '/plugins/jfbconnect/provider_'.$this->systemName.'/provider/'.$this->systemName.'/channel/';
        $formFile = null;
        if(Folder::exists($pluginPath))
        {
            $formFile = $pluginPath . $c . '_outbound.xml';
        }

        if (!$form = Form::getInstance('com_jfbconnect.' . $p . '.channel.' . $c, $formFile, $options))
            throw new Exception(Text::_('JERROR_LOADFILE_FAILED'));

        ob_start();
        foreach ($form->getFieldsets() as $fiedsets => $fieldset)
        {
            foreach ($form->getFieldset($fieldset->name) as $field)
                $this->formShowField($field);
        }
        $html = ob_get_clean();
        echo $html;
        exit;
    }

    public function channelShowAttributes()
    {
        $input = Factory::getApplication()->input;
        $app = Factory::getApplication();
        $providerData = new stdClass();
        $providerData->provider = $input->getString('provider');
        $providerData->type = $input->getString('type');
        $app->setUserState('com_jfbconnect.edit.channel.data', $providerData);

        JFBConnectUtilities::loadLanguage('plg_jfbconnect_provider_' . strtolower($providerData->provider), JPATH_ADMINISTRATOR);

        $this->displayAttributes();
        exit;
    }

    public function channelUpdateUser()
    {
        $app = Factory::getApplication();
        $providerData = new stdClass();
        $providerData->provider = $app->input->getString('provider');
        $providerData->type = $app->input->getString('type');
        $providerData->attribs = new stdClass();
        $providerData->attribs->user_id = $app->input->getString('userid');
        $app->setUserState('com_jfbconnect.edit.channel.data', $providerData);

        JFBConnectUtilities::loadLanguage('plg_jfbconnect_provider_' . strtolower($providerData->provider), JPATH_ADMINISTRATOR);

        $this->displayAttributes();
        exit;
    }

    private function displayAttributes()
    {
        require_once(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/views/channel/view.html.php');
        $model = BaseDatabaseModel::getInstance('Channel', 'JFBConnectModel');
        $view = new JFBConnectViewChannel();
        $view->setModel($model);
        echo $view->displayAttributes();
    }

    public function showAccessField()
    {
        $model = BaseDatabaseModel::getInstance('Channel', 'JFBConnectModel');
        $form = $model->getForm();

        require_once(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/views/channel/view.html.php');
        $view = new JFBConnectViewChannel();

        $input = Factory::getApplication()->input;
        $objects = $input->getString('objects',null);
        $plugins = $input->getString('plugins',null);

        if($objects)
            $objects = explode(',', $objects);
        if($plugins)
            $plugins = explode(',', $plugins);

        $show = $view->checkIfObjectsAreContent($objects);
        $show = $show || $view->checkIfPluginsAreContent($plugins);
        $field = $view->getAccessFields($form, $show);
        echo $field;
        exit;
    }

    public function enableAppPlugin()
    {
        $app = Factory::getApplication();
        $systemName = $app->input->get('name');

        $db = Factory::getDBO();
        $query = $db->getQuery(true);
        $query->update($db->qn('#__extensions'))
            ->set($db->qn('enabled') . '=1' )
            ->where($db->qn('type') . '=' . $db->q('plugin'))
            ->where($db->qn('element') . '=' . $db->q($systemName));
        $db->setQuery($query);
        $db->execute();

        $this->cleanAdminCache('com_plugins');

        exit;
    }

    public function cleanAdminCache($group)
    {
        $config = Factory::getConfig();
        $caching = $config->get('caching', 0);
        if($caching > 0)
        {
            $options = array(
                'defaultgroup' => $group,
                'cachebase' => JPATH_ADMINISTRATOR . '/cache',
                'result' => true,
            );

            try
            {
                $cache = Cache::getInstance('callback', $options);
                $cache->clean();
            }
            catch (CacheExceptionInterface $exception)
            {
                $options['result'] = false;
            }
        }
    }
}
