<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\Uri\Uri;

class JFBConnectControllerAutoTune extends JFBConnectController
{
    public $steps;

    public function __construct()
    {
        parent::__construct();
        $document = Factory::getApplication()->getDocument();
        $viewType = $document->getType();
        $viewName = 'autotune';
        $this->view = $this->getView($viewName, $viewType);

        $this->steps = [
            1 => [
                'name' => Text::_('COM_JFBCONNECT_AUTOTUNE_SIDEBAR_STEP_DEFAULT'),
                'task' => 'display'
            ],
            2 => [
                'name' =>  Text::_('COM_JFBCONNECT_AUTOTUNE_SIDEBAR_STEP_BASICINFO'),
                'task' => 'basicinfo'
            ],
            3 => [
                'name' => Text::_('COM_JFBCONNECT_AUTOTUNE_SIDEBAR_STEP_NETWORKSETTINGS'),
                'task' => 'networksettings'
            ],
            4 => [
                'name' => Text::_('COM_JFBCONNECT_AUTOTUNE_SIDEBAR_STEP_FBAPP'),
                'task' => 'fbapp'
            ],
            5 => [
                'name' => Text::_('COM_JFBCONNECT_AUTOTUNE_SIDEBAR_STEP_SITECONFIG'),
                'task' => 'siteconfig'
            ],
            6 => [
                'name' => Text::_('COM_JFBCONNECT_AUTOTUNE_SIDEBAR_STEP_ERROR'),
                'task' => 'errors'
            ],
            7 => [
                'name' => Text::_('COM_JFBCONNECT_AUTOTUNE_SIDEBAR_STEP_FINISH'),
                'task' => 'finish'
            ]
        ];
        $this->view->autotuneSteps = $this->steps;
    }

    function display($cachable = false, $urlparams = false)
    {
        $app = Factory::getApplication();
        $task = $app->input->getCmd('task', 'display');

        if ($this->getCurrentStep($task) > 2)
        {
            $downloadId = JFBCFactory::config()->get('sc_download_id');
            if (!$downloadId)
            {
                JFBCFactory::log(Text::_('COM_JFBCONNECT_MSG_ERROR_BASIC_INFO'), 'error');

                $this->setRedirect('index.php?option=com_jfbconnect&view=autotune&task=basicinfo');
                $this->redirect();
            }
        }

        $viewLayout = $task;
        $autotuneModel = $this->getModel('autotune');
        $this->view->setModel($autotuneModel, true);

        $configModel = $this->getModel('config');
        $this->view->setModel($configModel, false);

        switch ($task)
        {
            case 'basicinfo':
                $this->getBasicInfo();
                break;
            case 'fbapp':
                $viewLayout = 'fbapp';
                // First, check that a FB App ID and Key are set. If not, skip this step.
                $appId = JFBCFactory::config()->get('facebook_app_id');
                $secretKey = JFBCFactory::config()->get('facebook_secret_key');
                if (!$appId || !$secretKey)
                {
                    JFBCFactory::log(Text::_('COM_JFBCONNECT_MSG_ERROR_NOT_CONFIGURED'), 'error');
                    $this->setRedirect('index.php?option=com_jfbconnect&view=autotune&task=siteconfig');
                    $this->redirect();
                }

                // Next, check that the app is valid
                if (!$autotuneModel->validateApp(JFBCFactory::config()->get('facebook_app_id'), JFBCFactory::config()->get('facebook_secret_key')))
                {
                    JFBCFactory::log(Text::_('COM_JFBCONNECT_MSG_CHECK_KEYS'), 'error');
                    $this->setRedirect('index.php?option=com_jfbconnect&view=autotune&task=basicinfo');
                    $this->redirect();
                }

                // Finally, check if we should redirect to the new app page
                if ($autotuneModel->isNewApp())
                {
                    $this->setRedirect('index.php?option=com_jfbconnect&view=autotune&task=fbappnew');
                    $this->redirect();
                }

                $fields = $autotuneModel->getFieldDescriptors(false);
//                if ($fields == null) // Check if we should force load the data
//                    $autotuneModel->getFieldDescriptors(true);
                $appConfig = $autotuneModel->getAppConfig(false);
//                if ($appConfig == null)
//                    $autotuneModel->getFieldDescriptors(true);

                ToolBarHelper::custom('fbappRefresh', 'refresh', 'refresh', 'Refresh', false, false);
                ToolBarHelper::divider();
                break;
            case 'fbappnew':
                break;
            case 'siteconfig':
                $this->setupSiteConfig();
                break;
            case 'errors':
                $this->setupCheckErrors();
                break;
            case 'finish':
                break;
            case 'default':
            default:
                $this->setupIntroPage();
                break;
        }

        $this->setupToolbarButtons($this->getCurrentStep($task));

        ToolBarHelper::divider();
        ToolBarHelper::cancel('exitAutoTune', 'Exit AutoTune');
        $this->view->setLayout($viewLayout);
        $this->view->display();
    }

    public function setupToolbarButtons($currentStep)
    {
        $backIcon = 'arrow-left';
        $forwardIcon = 'arrow-right';

        if ($currentStep > 1)
            ToolBarHelper::custom($this->steps[$currentStep-1]['task'], $backIcon, $backIcon, $this->steps[$currentStep-1]['name'], false);
        if ($currentStep < count($this->steps))
            ToolBarHelper::custom($this->steps[$currentStep+1]['task'], $forwardIcon, $forwardIcon, $this->steps[$currentStep+1]['name'], false);
    }

    private function getCurrentStep($task)
    {
        // find our current step
        for ($i = 1; $i <= count($this->steps); $i++)
        {
            if ($task == $this->steps[$i]['task'])
                return $i;
        }
        return 1;
    }

    public function exitAutoTune()
    {
        $this->setRedirect('index.php?option=com_jfbconnect');
    }

    private function refresh()
    {
        $autotuneModel = $this->getModel('autotune');
        $autotuneModel->getFieldDescriptors(true);
        $autotuneModel->getAppConfig(true);
    }

    public function appRefresh()
    {
        //Refresh and redirect back to current page
        $this->refresh();

        $app = Factory::getApplication();
        $return = $app->input->getString('return', '');
        if(!empty($return))
        {
            $url = base64_decode($return);
            $this->setRedirect($url);
        }
    }

    public function fbappRefresh()
    {
        $this->refresh();
        $this->setRedirect('index.php?option=com_jfbconnect&view=autotune&task=fbapp');
    }

    private function setupSiteConfig()
    {
        $autotuneModel = $this->getModel('autotune');
        $JFBCSystemEnabled = $autotuneModel->isPluginEnabled('jfbcsystem');
        $JFBCAuthenticationEnabled = $autotuneModel->isPluginEnabled('jfbconnectauth');
        $JFBCContentEnabled = $autotuneModel->isPluginEnabled('jfbccontent');
        $JFBCUserEnabled = $autotuneModel->isPluginEnabled('jfbconnectuser');

        $errors = $autotuneModel->getJoomlaErrors();

        $this->view->JFBCSystemEnabled = $JFBCSystemEnabled;
        $this->view->JFBCAuthenticationEnabled = $JFBCAuthenticationEnabled;
        $this->view->JFBCContentEnabled = $JFBCContentEnabled;
        $this->view->JFBCUserEnabled = $JFBCUserEnabled;
        $this->view->joomlaErrors = $errors;
    }

    private function setupIntroPage()
    {
        $phpVersion = phpversion();
        $errorsFound = false;
        if (version_compare($phpVersion, '5.0.0') >= 0)
            $phpVersion .= '<td><img src="components/com_jfbconnect/assets/images/icon-16-allow.png" /></td>';
        else
        {
            $phpVersion .= '<td><img src="components/com_jfbconnect/assets/images/icon-16-deny.png" /></td>';
            $errorsFound = true;
        }
        $this->view->phpVersion = $phpVersion;

        // cURL check
        $disableFunctions = ini_get('disable_functions');
        if (in_array('curl', get_loaded_extensions()) && strpos($disableFunctions, 'curl_exec') === false)
            $curlCheck = 'Enabled <td><img src="components/com_jfbconnect/assets/images/icon-16-allow.png" /></td>';
        else
        {
            $curlCheck = '<strong>Disabled</strong> <td><img src="components/com_jfbconnect/assets/images/icon-16-deny.png" /></td>';
            $errorsFound = true;
        }

        if ($errorsFound)
        {
            JFBCFactory::log(Text::_('COM_JFBCONNECT_MSG_ERROR_SERVER_CONFIG'), 'error');
        }
        $this->view->curlCheck = $curlCheck;
        $this->view->errorsFound = $errorsFound;
    }

    public function getBasicInfo()
    {
        $configModel = $this->getModel('config');
        $subscriberId = $configModel->get('sc_download_id');

        $this->view->subscriberId = $subscriberId;
    }

    public function saveBasicInfo()
    {
        $app = Factory::getApplication();
        $subscriberId = $app->input->getString('subscriberId');
        foreach (JFBCFactory::getAllProviders() as $provider)
        {
            $name = $provider->systemName;
            $appId = $app->input->getString($name . '_app_id');
            $secretKey = $app->input->getString($name . '_secret_key');
            JFBCFactory::config()->update($name . '_app_id', $appId);
            JFBCFactory::config()->update($name . '_secret_key', $secretKey);
        }

        JFBCFactory::config()->update('sc_download_id', $subscriberId);

        $this->setRedirect('index.php?option=com_jfbconnect&view=autotune&task=networksettings');
    }

    private function setupCheckErrors()
    {
        if (defined('JFBCDEV'))
        {
            $domain = 'http://localhost/autotune/start.php';
            $baseUrl = 'http://www.sourcecoast.com';
        }
        else
        {
            $domain = 'https://autotune.sourcecoast.com/start.php';
            $baseUrl = Uri::root();
        }
        $baseUrl = base64_encode(urlencode($baseUrl));

        $autotuneModel = $this->getModel('autotune');
        $configModel = $this->getModel('config');
        $subscriptionId = $configModel->get('sc_download_id');
        $query = '?baseUrl=' . $baseUrl . '&subscriptionId=' . $subscriptionId . '&task=jfbconnect.errorStart&format=html&' . $autotuneModel->getVersionURLQuery();

        $iframeUrl = $domain . $query;
        $this->view->iframeUrl = $iframeUrl;
    }

    public function saveAppConfig()
    {
        $autotuneModel = $this->getModel('autotune');
        $settings = $autotuneModel->getAppValuesToSave(false);
        $autotuneModel->updateFBApplication($settings);

        // Always set the migrations
        $migrations = $autotuneModel->getAppMigrationsToSave();
        $autotuneModel->updateFBApplication($migrations);

        // Update the database with the new app info
        $autotuneModel->getAppConfig(true);
        $this->setRedirect('index.php?option=com_jfbconnect&view=autotune&task=fbapp');
    }

    public function saveAppRecommendations()
    {
        $autotuneModel = $this->getModel('autotune');
        $settings = $autotuneModel->getAppValuesToSave(true);
        $autotuneModel->updateFBApplication($settings);

        // Always set the migrations
        $migrations = $autotuneModel->getAppMigrationsToSave();
        $autotuneModel->updateFBApplication($migrations);

        // Update the database with the new app info
        $autotuneModel->getAppConfig(true);
        $this->setRedirect('index.php?option=com_jfbconnect&view=autotune&task=fbapp');
    }

    public function publishPlugin()
    {
        $app = Factory::getApplication();
        $name = $app->input->getString('pluginName');
        $status = $app->input->getInt('pluginStatus');
        $autotuneModel = $this->getModel('autotune');
        $autotuneModel->publishPlugin($name, $status);
        $this->setRedirect('index.php?option=com_jfbconnect&view=autotune&task=siteconfig');
    }

    /*
     * Ajax call to update each social network's constants for use as necessary
     */
    public function fetchNetworkSettings()
    {
        $autotuneModel = $this->getModel('autotune');
        $response = $autotuneModel->fetchNetworkSettings();
        echo json_encode($response);
        exit;
    }
}