<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */
// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\Folder;

class JFBConnectControllerProviders extends JFBConnectController
{
    var $availableProviders;
    var $allProviders;
    var $installedProviders;
    var $enabledProviders;

    function apply()
    {
        $input = Factory::getApplication()->input;
        $configs = $input->post->getArray();
        $model = $this->getModel('config');
        $model->saveSettings($configs);
        JFBCFactory::log(Text::_('COM_JFBCONNECT_MSG_SETTINGS_UPDATED'));
        $this->setRedirect('index.php?option=com_jfbconnect&view=providers');
    }

    public static function providers()
    {
// Disabling caching as newly enabled plugins aren't properly shown
//        $providers = JFBCFactory::cache()->get('providers.list');
//        if ($providers === false)
//        {
        $providers = self::getAllAvailableProviders();
        JFBCFactory::cache()->store($providers, 'providers.list');
//        }

        $allProviders = array();
        self::updateInstalled($allProviders);
        self::updateEnabled($allProviders);

        return $providers;
    }

    private static function getAllAvailableProviders()
    {
        $jsonFile = "https://sourcecoast.com/provider-repo/availableProviders.json";
        $fileContents = @file_get_contents($jsonFile);
        $data = json_decode($fileContents, true);

        if($data == null)
        {
            $jsonFile = JPATH_ADMINISTRATOR.'/components/com_jfbconnect/includes/availableProviders.json';
            $data = json_decode(file_get_contents($jsonFile), true);
        }

        $allProviders = array();
        foreach($data["repo"][0]["providers"] as $p)
        {
            $allProviders[$p['systemName']] = array(
                'name' => $p['name'],
                'description' => $p['description'],
                'link' => $p['link'],
                'installed' => false,
                'enabled' => false
            );
        }

        self::updateInstalled($allProviders);
        self::updateEnabled($allProviders);

        return $allProviders;
    }

    private static function updateInstalled(&$allProviders)
    {
        if($allProviders)
        {
            foreach($allProviders as $systemName => $p)
            {
                $directory = JPATH_SITE . '/plugins/jfbconnect/provider_'.$systemName.'/provider/';
                $allProviders[$systemName]['installed'] = Folder::exists($directory);
            }
        }
    }

    private static function updateEnabled(&$allProviders)
    {
        $usermapModel = JFBCFactory::usermap();

        $all = JFBCFactory::getAllProviders();
        foreach($all as $p)
        {
            $allProviders[$p->systemName]['enabled'] = true;
            $allProviders[$p->systemName]['userCount'] = $usermapModel->getTotalMappings($p->systemName);
        }
    }
}