<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */
 // Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Toolbar\Toolbar;
use Joomla\CMS\Toolbar\ToolbarHelper;

class JFBConnectControllerUserMap extends JFBConnectController
{
    function __construct()
    {
        parent::__construct();
        $document = Factory::getApplication()->getDocument();
        $viewType = $document->getType();
        $viewName = 'usermap';
        $this->view = $this->getView($viewName, $viewType);
    }

    function display($cachable = false, $urlparams = false)
    {
        $usermapModel = $this->getModel('usermap');
        $this->view->setModel($usermapModel, false);

        $app = Factory::getApplication();
        $viewLayout = $app->input->getCmd('layout', 'list');
        $this->view->setLayout($viewLayout);

        // Add Toolbar icons
        $app = Factory::getApplication();
        PluginHelper::importPlugin('socialprofiles');
        $profilePlugins = $app->triggerEvent('onSocialProfilesGetPlugins');
        $pluginNames = array();
        foreach ($profilePlugins as $plugin)
        {
            if ($plugin->canImportConnections())
                $pluginNames[] = $plugin->getName();
        }
        if (count($pluginNames) != 0)
        {
            $doc = Factory::getApplication()->getDocument();
            $doc->addCustomTag('<script>var jfbcImportMsg = "' . Text::_('COM_JFBCONNECT_MSG_IMPORT_DESC') . '\n' . implode('\n', $pluginNames) . '\n\n' . Text::_('COM_JFBCONNECT_MSG_IMPORT_CONFIRMATION') . '";</script>');
            $html = "onclick=\"javascript:if(confirm(jfbcImportMsg)){submitbutton('importConnections');}\" class=\"toolbar\">\n";
            $html .= "<span class=\"icon-32-upload\" title=\"" . Text::_('COM_JFBCONNECT_MSG_IMPORT_TITLE') . "\">\n";
            $html .= "</span>\n";
            $html .= Text::_('COM_JFBCONNECT_MSG_IMPORT_TITLE') . "\n";

            $html = '<button ' . $html . "</button>\n";
            $bar = Toolbar::getInstance('toolbar');
            //$bar->appendButton('Confirm', "Import connections from the enabled profile plugins? Ensure you've enabled only the plugins for 3rd party components that you'd want to import previous connections from.", 'upload', 'Import Connections', 'importConnections', false, false);
            $bar->appendButton('Custom', $html);
        }
        ToolBarHelper::deleteList(Text::_('COM_JFBCONNECT_MSG_USERMAP_DELETE_CONFIRMATION'));

        $this->view->display();
    }

    function remove()
    {
        $model = $this->getModel('UserMap');

        if (!$model->delete())
        {
            $msg = Text::_('COM_JFBCONNECT_MSG_USERMAP_DELETE_FAIL');
        }
        else
        {
            $msg = Text::_('COM_JFBCONNECT_MSG_USERMAP_DELETE_SUCCESS');
        }

        $this->display();
    }

    function importConnections()
    {
        $app = Factory::getApplication();
        PluginHelper::importPlugin('socialprofiles');
        $profilePlugins = $app->triggerEvent('onJfbcImportConnections');
        $msg = Text::_('COM_JFBCONNECT_MSG_IMPORT_SUCCESS');
        JFBCFactory::log($msg);
        $this->display();
    }
}