<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\MVC\View\HtmlView;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Form\Form;
use Joomla\CMS\Router\Route;

class JFBConnectAdminView extends HtmlView
{
    protected $forms = array();

    var $sidebar;
    var $displaySidebar = true;
    var $prefooter = "";

    public function display($tpl = null)
    {
        $app = Factory::getApplication();

        $this->addToolbar();

        echo '<div class="row-fluid">';
        $hideSidebar = $app->input->get('displaySidebar', '') == 'false' || (JVERSION >= 4.0 ) ;
        if($this->displaySidebar && !$hideSidebar)
        {
            $this->sidebar = $this->getSidebar();
            $this->toggleSidebar();
            echo '<div id="j-sidebar-container" class="col-md-2">';
            echo $this->sidebar;
            echo '</div>';
        }
        echo '<div id="j-main-container"'.(!empty($this->sidebar) ? ' class="col-md-10"' : '') . '>';

        parent::display($tpl);

        echo $this->prefooter;

        if ($app->input->getCmd('tmpl') != 'component')
        {
            include_once(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/assets/footer/footer.php');
        }
        echo '</div></div>';
    }

    private function toggleSidebar()
    {
        Factory::getDocument()->addScriptDeclaration('
            jQuery(document).ready(function($)
            {
                if (window.toggleSidebar)
                {
                    toggleSidebar(true);
                }
                else
                {
                    $("#j-toggle-sidebar-header").css("display", "none");
                    $("#j-toggle-button-wrapper").css("display", "none");
                }
            });
        ');
    }

    private function getSidebar()
    {
        $html = '<div id="j-toggle-sidebar-wrapper">
	<div id="j-toggle-button-wrapper" class="j-toggle-button-wrapper">
		<div
	id="j-toggle-sidebar-button"
	class="j-toggle-sidebar-button hidden-phone hasTooltip"
	onclick="toggleSidebar(false); return false;"
	>
		<span id="j-toggle-sidebar-icon" class="icon-arrow-left-2" aria-hidden="true"></span>
        </div>
	</div>
	<div class="sidebar" id="sidebar">'
            . '<div class="sidebar-nav">'
            . $this->getSubMenu()
            . '</div></div>'
            . '<div id="j-toggle-sidebar"></div>'
            . '</div>';
        return $html;
    }

    protected function getSubMenu()
    {
        $input = Factory::getApplication()->input;
        $view = $input->get('view');
        $provider = $input->get('provider', '');
        $provider = (!empty($provider)) ? '&provider=' . $provider : '';

        $vName = strtolower($view);
        if ($vName == "autotune" || $vName == "overview")
            return; // Don't show the sub-menus when inside AutoTune or the main overview page

        $menus = array(
            array(
                'title' => Text::_('COM_JFBCONNECT_MENU_OVERVIEW'),
                'url' => 'index.php?option=com_jfbconnect',
                'active' => $vName == 'overview',
                'children' => array(),
            ),
            array(
                'title' => Text::_('COM_JFBCONNECT_MENU_CONFIGURATION'),
                'url' => 'index.php?option=com_jfbconnect&view=config',
                'active' => $vName == 'config',
                'children' => array(),
            ),
            array(
                'title' => Text::_('COM_JFBCONNECT_MENU_PROVIDERS'),
                'url' => 'index.php?option=com_jfbconnect&view=providers',
                'active' => $vName == 'providers',
                'children' => $this->getProviders(),
            ),
            array(
                'title' => Text::_('COM_JFBCONNECT_MENU_OPENGRAPH'),
                'url' => 'index.php?option=com_jfbconnect&view=opengraph',
                'active' => $vName == 'opengraph',
                'children' => array(),
            ),
            array(
                'title' => Text::_('COM_JFBCONNECT_MENU_SOCIAL'),
                'url' => 'index.php?option=com_jfbconnect&view=social',
                'active' => $vName == 'social',
                'children' => array(),
            ),
            array(
                'title' => Text::_('COM_JFBCONNECT_MENU_PROFILES'),
                'url' => 'index.php?option=com_jfbconnect&view=profiles',
                'active' => $vName == 'profiles',
                'children' => array(),
            ),
            array(
                'title' => Text::_('COM_JFBCONNECT_MENU_USER_MAP'),
                'url' => 'index.php?option=com_jfbconnect&view=usermap',
                'active' => $vName == 'usermap',
                'children' => array(),
            ),
            array(
                'title' => Text::_('COM_JFBCONNECT_MENU_CHANNELS'),
                'url' => 'index.php?option=com_jfbconnect&view=channels',
                'active' => $vName == 'channels',
                'children' => array(),
            ),
            array(
                'title' => Text::_('COM_JFBCONNECT_MENU_CHANNELHISTORY'),
                'url' => 'index.php?option=com_jfbconnect&view=channelhistory',
                'active' => $vName == 'channelhistory',
                'children' => array(),
            ),
        );

        $html = '<ul id="submenu" class="nav nav-list">';

        foreach ($menus as $menu)
        {
            $hasChildren = !empty($menu['children']);
            $dropdownToggleClass = ($hasChildren) ? 'dropdown-toggle' : '';
            $isOpen = false;
            $current = '';

            if ($hasChildren) {
                foreach ($menu['children'] as $child) {
                    if ($child['url'] == 'index.php?option=com_jfbconnect&view=' . $view . $provider) {
                        $isOpen = true;
                        $current = $child['url'];
                        break;
                    }
                }
            } else {
                if ($menu['url'] == 'index.php?option=com_jfbconnect&view=' . $view) {
                    $isOpen = true;
                }
            }

            $openClass = ($isOpen) ? 'open' : '';
            $openSubStyle = ($isOpen) ? 'display: block;' : '';

            $html .= '<li class="' . $openClass . '"><a href="' . Route::_($menu['url']) . '" class="' . $dropdownToggleClass . '">';
            $html .= $menu['title'];

            if ($hasChildren) {
                $html .= '<b class="arrow fa fa-angle-down"></b>';
            }

            $html .= '</a>';

            if ($hasChildren) {
                $html .= '<ul class="submenu nav nav-list" style="' . $openSubStyle . '">';
                foreach ($menu['children'] as $child) {
                    $submenuStyle = ($current == $child['url']) ? 'class="active nav-item"' : 'nav-item';
                    $html .= '<li ' . $submenuStyle . '>';
                    $html .= '<a class="'.$child['class'].'" href="' . Route::_($child['url']) . '">';
                    $html .=  $child['title'];
                    $html .= '</a></li>';
                }
                $html .= '</ul>';
            }
            $html .= '</li>';
        }
        $html .= '</ul>';

        return $html;
    }

    public function getProviders()
    {
        $providers = array();

        require_once(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/controllers/providers.php');
        $all = JFBConnectControllerProviders::providers();
        foreach($all as $systemName => $p)
        {
            $name = $p['name'];

            if(!$p['installed'] || !$p['enabled'])
                continue;

            $providers[] = array(
                'title' => $name,
                'url' => 'index.php?option=com_jfbconnect&view=providers&layout=provider&provider='.$systemName,
                'class' => 'nav-link'
            );
        }

        $providers[] = array(
            'title' => Text::_('COM_JFBCONNECT_PROVIDERS_ADDMORE'),
            'url' => 'index.php?option=com_jfbconnect&view=providers',
            'class' => 'nav-link'
        );

        return $providers;
    }

    public function addToolbar()
    {
    }

    private function getTabPrefix()
    {
        if(JVERSION < 4.0)
            return 'bootstrap';
        else
            return 'uitab';
    }

    public function tabsStart($tabGroup, $active)
    {
        echo HTMLHelper::_($this->getTabPrefix().'.startTabSet', $tabGroup, array('active' => $active));
    }

    public function tabsEnd()
    {
        echo HTMLHelper::_($this->getTabPrefix().'.endTabSet');
    }

    public function tabStart($tabGroup, $tabName, $label)
    {
        echo HTMLHelper::_($this->getTabPrefix().'.addTab', $tabGroup, $tabName, $label, true);
    }

    public function tabEnd()
    {
        echo HTMLHelper::_($this->getTabPrefix().'.endTab');
    }

    // This form stuff should be broken into it's own class
    public function formLoad($name, $formPath, $options = array())
    {
        $form = Form::getInstance('com_jfbconnect_' . $name, $formPath, $options);
        $this->forms[$name] = $form;
    }

    public function formBind($name, $data = null)
    {
        if (!isset($this->forms[$name]))
            return false;

        if (!$data)
            $data = JFBCFactory::config()->getSettings();
        $this->forms[$name]->bind($data);
    }

    public function formShowField($field)
    {
        echo $this->formGetField($field);
    }

    public function formGetField($field)
    {
        $fieldValue = $field->renderField();
        return $fieldValue;
    }

    public function formDisplay($name, $columns = null)
    {
        $form = $this->forms[$name];
        // use a 2 column layout (by default) for more than one fieldset, if not passed in
        $blocks = count($form->getFieldsets());
        if (!$columns)
            $columns = $blocks > 1 ? 2 : 1;
        $span = $columns == 1 ? 12 : 6;
        $split = ceil($blocks / 2);
        $column = 0;

        echo "\n<div class=\"row\">\n";
        foreach ($form->getFieldsets() as $fiedsets => $fieldset)
        {
            if ($column == 0 || $column == $split)
                echo '<div class="col-md-' . $span . '">' . "\n";
            echo "<fieldset class=\"options-form card\">\n";
            if ($fieldset->label)
                $label = $fieldset->label;
            else
                $label = Text::_(strtoupper($form->getName()) . '_MENU_' . strtoupper($fieldset->name));
            echo '<legend>' . $label . "</legend>\n";
            foreach ($form->getFieldset($fieldset->name) as $field)
                $this->formShowField($field);
            echo "</fieldset>\n";
            $column++;
            if ($column == $split)
                echo "</div>\n";
        }
        echo ($column > $split) ? "</div>\n" : '';
        echo "</div>\n";
    }
}