<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\CMS\MVC\Model\AdminModel;
use Joomla\CMS\Log\Log;
use Joomla\CMS\Form\Form;
use Joomla\Event\Event;
use Joomla\CMS\Filesystem\Folder;

jimport('joomla.application.component.modeladmin');
class JFBConnectModelChannel extends AdminModel
{
    function __construct($channelId)
    {
        $apModel = BaseDatabaseModel::getInstance('Autopost', 'JFBConnectModel');

        if(JVERSION >= 4.0)
        {
            $dispatcher = Factory::getContainer()->get('dispatcher');
            $dispatcher->addListener('onContentAfterSave', [$this, 'onContentAfterSave']);
        }

        parent::__construct();
    }

    function getForm($data = array(), $loadData = true)
    {
        $form = $this->loadForm('com_jfbconnect_channel_edit', 'channeledit', array('control' => 'jform', 'load_data' => $loadData));
        if (empty($form))
            return false;

        return $form;
    }

    public function onContentAfterSave(Event $event)
    {
        $table = $event->offsetGet(1);
        //TODO: Joomla 4 If you don't fetch state first, doesn't retrieve correctly later
        $state = $this->getState('channel.id', 0);
        $this->setState('channel.id', $table->id);
    }

    function save($data)
    {
        $oldData = array();

        //TODO Joomla 4 exception for "false" instead of "0"
        $data['published'] = (int) $data['published'];

        if ($data['id'] > 0)
        {
            $table = $this->getTable();
            $table->load($data['id']);
            $oldData['id'] = $table->id;
            $oldData['provider'] = $table->provider;
            $oldData['type'] = $table->type;
            $oldData['attribs'] = (array)$table->attribs;
        }

        $data['attribs']['autopost_access_type'] = isset($data['autopost_access_type']) ? $data['autopost_access_type'] : array();

        // Set the dates
        $date = Factory::getDate();
        if ($data['id'] == 0)
            $data['created'] = $date->toSql();
        $data['modified'] = $date->toSql();

        if ($data['provider'] != '--' && $data['type'] != '--')
        {
            $channel = JFBCFactory::provider($data['provider'])->channel($data['type']);
            $data = $channel->onBeforeSave($data); // Manipulate the data however may be necessary
        }
        $return = parent::save($data);
        $data['id'] = $this->getState('channel.id', $data['id']);

        // Used to removed permissions from users that aren't associated with this channel (or anything else needed for cleanup)
        if ($return && $data['provider'] != '--' && $data['type'] != '--')
        {
            $channel->onAfterSave($data, $oldData);
        }

        //Add autosave option. Need to wait for ID to be populated here.
        $autopost = new JFBConnectModelAutopost($data['id']);
        $autopostObjects = (isset($data['autopost_objects'])?$data['autopost_objects']:array());
        $autopostPlugins = (isset($data['autopost_plugins'])?$data['autopost_plugins']:array());
        $autopostData = array_merge((array)$autopostObjects, (array)$autopostPlugins);
        $autopost->setData($autopostData);

        return $return;
    }

    /**
     * Method to delete one or more records.
     *
     * @param   array &$pks  An array of record primary keys.
     *
     * @return  boolean  True if successful, false if an error occurs.
     *
     * Copied from the Joomla legacy folder. Removed content triggers and added deletion of scope for associated user
     */
    public function delete(&$pks)
    {
        $pks = (array)$pks;
        $table = $this->getTable();

        // Iterate the items to delete each one.
        foreach ($pks as $i => $pk)
        {
            if ($table->load($pk))
            {
                if ($this->canDelete($table))
                {
                    //Delete any autopost associated with this channel
                    $autopost = new JFBConnectModelAutopost($table->id);
                    $autopost->delete();

                    // Delete any scope for the user associated with this channel
                    $attribs = $table->attribs;
                    if (isset($attribs->user_id))
                    {
                        $userModel = JFBConnectModelUserMap::getUser($attribs->user_id, $table->provider);
                        $userModel->removeAllScope('channel', $table->id);
                    }

                    if (!$table->delete($pk))
                    {
                        $this->setError($table->getError());
                        return false;
                    }
                }
                else
                {
                    // Prune items that you can't change.
                    unset($pks[$i]);
                    $error = $this->getError();
                    if ($error)
                    {
                        Log::add($error, Log::WARNING, 'jerror');
                        return false;
                    }
                    else
                    {
                        Log::add(Text::_('JLIB_APPLICATION_ERROR_DELETE_NOT_PERMITTED'), Log::WARNING, 'jerror');
                        return false;
                    }
                }
            }
            else
            {
                $this->setError($table->getError());
                return false;
            }
        }
        // Clear the component's cache
        $this->cleanCache();

        return true;
    }

    protected function preprocessForm(Form $form, $data, $group = 'plugin')
    {
        // In here, we're also going to override the form's saved value if type= is in the query string.
        $input = Factory::getApplication()->input;
        $type = $input->get('type', null);
        $provider = $input->get('provider', null);

        // Figure out the provider/channel selected.. either from the request or from the saved table data
        if (!$type && !$provider) {
            if (empty($data)) {
                $jform = Factory::getApplication()->input->post->get('jform', array(), 'array');
                $provider = isset($jform['provider']) ? $jform['provider'] : null;
                $type = isset($jform['type']) ? $jform['type'] : null;
            } else {
                $provider = isset($data->provider) ? $data->provider : null;
                $type = isset($data->type) ? $data->type : null;
            }
        }

        if ($provider && $type) {
            $pluginPath = JPATH_SITE . '/plugins/jfbconnect/provider_' . $provider . '/provider/' . $provider . '/channel/';

            $outboundFile = null;
            if (Folder::exists($pluginPath . 'fields')) {
                Form::addFieldPath($pluginPath . 'fields');
                $outboundFile = $pluginPath . $type . '_outbound.xml';
            }

            // Prepend the User selection to all the attributes for every channel type
            $formFile = JPATH_ADMINISTRATOR . '/components/com_jfbconnect/models/fields/channelusers.xml';
            if (file_exists($formFile))
            {
                if (!$form->loadFile($formFile, false))
                {
                    throw new Exception(Text::_('JERROR_LOADFILE_FAILED'));
                }
            }

            if ($outboundFile && file_exists($outboundFile))
            {
                if (!$form->loadFile($outboundFile, false))
                {
                    throw new Exception(Text::_('JERROR_LOADFILE_FAILED'));
                }
            }
        }

        // Trigger the default form events.
        parent::preprocessForm($form, $data, $group);
    }

    protected function loadFormData()
    {
        // Check the session for previously entered form data.
        $app = Factory::getApplication();
        $data = $app->getUserState('com_jfbconnect.edit.channel.data', array());

        if (empty($data))
        {
            $data = $this->getItem();

            if (empty($data->provider))
            {
                $input = Factory::getApplication()->input->get('provider', null);
                $data->set('provider', $input);
            }
            if (empty($data->type))
            {
                $input = Factory::getApplication()->input->get('type', null);
                $data->set('type', $input);
            }

            $autopost = new JFBConnectModelAutopost($data->id);
            $autopostData = $autopost->getData();

            $apObjectsData = array();
            $apPluginsData = array();

            foreach($autopostData as $object)
            {
                if(is_numeric($object->opengraph_type))
                    $apObjectsData[] = $object->opengraph_type;
                else
                    $apPluginsData[]=$object->opengraph_type;
            }

            $data->set('autopost_objects', $apObjectsData);
            $data->set('autopost_plugins', $apPluginsData);
            $data->set('autopost_access_type', isset($data->attribs->autopost_access_type) ? $data->attribs->autopost_access_type : "1");

            // Prime some default values.
            if ($this->getState('channel.id') == 0)
            {
                $data->set('published', '0');
            }
        }

        return $data;
    }

    function getUserList($provider)
    {
        $users = array();

        if(!empty($provider))
        {
            $search = '';
            $db = Factory::getDbo();
            $query = $db->getQuery(true);

            //$filter_search = trim($this->getState('filter.search'));
            $input = Factory::getApplication()->input;
            $filter_search = $input->get('filter_search');
            if($filter_search)
            {
                $search = $db->quote('%' . str_replace(' ', '%', $db->escape($filter_search, true) . '%'));

                // Compile the different search clauses.
                $searches   = array();
                $searches[] = 'u.name LIKE ' . $search;
                $searches[] = 'u.username LIKE ' . $search;
                $searches[] = 'u.email LIKE ' . $search;
            }

            $query->select('u.id,u.name,u.username')
                ->from('#__users u')
                ->join('inner','#__jfbconnect_user_map AS m ON u.id=m.j_user_id')
                ->where('m.provider='.$db->q(strtolower($provider)));
            if(!empty($search))
                $query->where('(' . implode(' OR ', $searches) . ')'); // Add the clauses to the query.
            $db->setQuery($query);
            $users = $db->loadObjectList();
        }
        return $users;
    }
}