<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Form\FormHelper;

jimport('joomla.form.helper');
FormHelper::loadFieldClass('list');

class JFormFieldChannelAutopostObjects extends JFormFieldList
{
    public $type = 'ChannelAutopostObjects';

    protected function getOptions()
    {
        $options = array();

        //Get enabled plugins
        $app = Factory::getApplication();
        PluginHelper::importPlugin('opengraph');
        $ogPlugins = $app->triggerEvent('onOpenGraphProfilesGetPlugins');

        //Build list of all objects that support autoposting
        $allSupportedObjects = array();
        foreach ($ogPlugins as $plugin)
        {
            if($plugin->supportedAutopostTypes)
                $allSupportedObjects = array_merge($allSupportedObjects, $plugin->supportedAutopostTypes);
        }


        //Get published open graph objects
        BaseDatabaseModel::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/models', 'JFBConnectAdminModel');
        $this->ogObjectModel = BaseDatabaseModel::getInstance('OpenGraphObject', 'JFBConnectAdminModel');
        $ogObjects = $this->ogObjectModel->getObjects(true);

        $tmp = HTMLHelper::_('select.option', 0, Text::_('COM_JFBCONNECT_CHANNEL_AUTOPOST_SELECT_OBJECT_DESC'), 'value', 'text');
        $tmp->checked = false;
        $options[] = $tmp;

        //Add open graph objects that are supported
        foreach ($ogObjects as $ogObject)
        {
            if(in_array($ogObject->system_name, $allSupportedObjects))
            {
                $tmp = HTMLHelper::_('select.option', $ogObject->id, Text::sprintf('COM_JFBCONNECT_CHANNEL_AUTOPOST_OG_CONTENT_DESC',$ogObject->display_name), 'value', 'text');
                $tmp->checked = false;
                $options[] = $tmp;
            }
        }

        return $options;
    }
}