<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Table\Table;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Form\FormField;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;

/**
 * Field to select a user ID from a modal list.
 *
 * @package     Joomla.Libraries
 * @subpackage  Form
 * @since       1.6
 */
class JFormFieldChannelUserList extends FormField
{
    /**
     * The form field type.
     *
     * @var    string
     * @since  1.6
     */
    public $type = 'ChannelUserList';
    public $provider = '';

    /**
     * Method to get the user field input markup.
     *
     * @return  string  The field input markup.
     *
     * @since   1.6
     */
    protected function getInput()
    {
        $input = Factory::getApplication()->input;
        // Get the provider from the URL if set (first time setup). If not, it should be in the form (previously saved)
        $this->provider = $input->get('provider', $this->form->getData()->get('provider'));

        $html = array();
        $groups = $this->getGroups();
        $excluded = $this->getExcluded();
        $link = 'index.php?option=com_jfbconnect&amp;view=channel&amp;layout=user&displaySidebar=false&amp;provider='.$this->provider.'&amp;tmpl=component&amp;field=' . $this->id
            . (isset($groups) ? ('&amp;groups=' . base64_encode(json_encode($groups))) : '')
            . (isset($excluded) ? ('&amp;excluded=' . base64_encode(json_encode($excluded))) : '');

        // Initialize some field attributes.
        $attr = !empty($this->class) ? ' class="' . $this->class . '"' : '';
        $attr .= !empty($this->size) ? ' size="' . $this->size . '"' : '';
        $attr .= $this->required ? ' required' : '';

        // Load the current username if available.
        $table = Table::getInstance('user');

        if (is_numeric($this->value))
        {
            $table->load($this->value);
        }
        // Handle the special case for "current".
        elseif (strtoupper($this->value) == 'CURRENT')
        {
            // 'CURRENT' is not a reasonable value to be placed in the html
            $this->value = Factory::getUser()->id;
            $table->load($this->value);
        }
        else
        {
            $table->name = Text::_('JLIB_FORM_SELECT_USER');
        }

        // Create a dummy text field with the user name.
        $html[] = '<div class="input-append">';
        $html[] = '	<input type="text" id="' . $this->id . '" value="' . htmlspecialchars($table->name, ENT_COMPAT, 'UTF-8') . '"'
            . ' readonly' . $attr . ' />';

        $html[] = HTMLHelper::_(
            'bootstrap.renderModal',
            'usersModal_' . $this->id,
            array(
                'url' => Route::_($link),
                'title' => Text::_('JLIB_FORM_CHANGE_USER'),
                'animation' => false,
                'height' => '500px',
                'width' => '800px',
                'modalWidth' => '800px',
                'bodyHeight' => '500px',
                'closeButton' => true,
                'backdrop' => false,
            )
        );

        // Create the user select button.
        if ($this->readonly === false)
        {
            $toggle = (JVERSION < 4.0) ? ' data-toggle' : ' data-bs-toggle';
            $target = (JVERSION < 4.0) ? ' data-target' : ' data-bs-target';

            $button = '<button type="button"'
                . ' class="btn btn-primary button-select"'
                . ' title="' . Text::_('JLIB_FORM_CHANGE_USER') . '"'
                . $toggle.'="modal"'
                . $target.'="#usersModal_' . $this->id . '"'
                . ' aria-label="' . Text::_('JLIB_FORM_CHANGE_USER') . '">'
                . ' <span class="icon-user" aria-hidden="true"></span>' . Text::_('JLIB_FORM_CHANGE_USER')
                . ' </button>';

            $html[] = $button;
        }
        $html[] = '</div>';

        // Create the real field, hidden, that stored the user id.
        $html[] = '<input type="hidden" id="' . $this->id . '_id" name="' . $this->name . '" value="' . $this->value . '" />';

        $niceName = JFBCFactory::provider($this->provider)->name;
        $loginLink = Uri::root() . 'index.php?option=com_jfbconnect&provider=' . $this->provider . '&task=authenticate.login&force_login=1';
        $html[] = "<div><strong>Don't see your user or need additional permissions?</strong> <a href='" . $loginLink . "' target='_blank'>Log in to " . $niceName . "</a></div>";

        return implode("\n", $html);
    }

    /**
     * Method to get the filtering groups (null means no filtering)
     *
     * @return  mixed  array of filtering groups or null.
     *
     * @since   1.6
     */
    protected function getGroups()
    {
        return null;
    }

    /**
     * Method to get the users to exclude from the list of users
     *
     * @return  mixed  Array of users to exclude or null to to not exclude them
     *
     * @since   1.6
     */
    protected function getExcluded()
    {
        return null;
    }
}
