<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Uri\Uri;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Form\FormField;

jimport('joomla.form.helper');

class JFormFieldProviderloginbutton extends FormField
{
    protected function getInput()
    {
        $html = array();
        $provider = $this->element['provider'] ? (string)$this->element['provider'] : null;
        $style = $this->element['style'] ? (string)$this->element['style'] . '"' : '';
        // Initialize some field attributes.

        $class = !empty($this->class) ? ' class="radio ' . $this->class . '"' : ' class="radio"';
        $required = $this->required ? ' required aria-required="true"' : '';
        $autofocus = $this->autofocus ? ' autofocus' : '';
        $disabled = $this->disabled ? ' disabled' : '';
        $readonly = $this->readonly;

        $style = 'style="float:left;' . $style . '"';

        $html[] = '<div style="clear: both"> </div>';

        // Start the radio field output.
        $html[] = '<fieldset id="' . $this->id . '"' . $class . $required . $autofocus . $disabled . $style . ' >';

        // Get the field options.
        $options = $this->getOptions();

        $p = JFBCFactory::provider($provider);
        // Build the radio field output.
        $html[] = '<label class="providername">' . $p->name . '</label>';

        foreach ($options as $i => $option)
        {
            // Initialize some option attributes.
            $checked = ((string)$option->value == (string)$this->value) ? ' checked="checked"' : '';
            $class = !empty($option->class) ? ' class="' . $option->class . '"' : '';

            $disabled = !empty($option->disable) || ($readonly && !$checked);

            $disabled = $disabled ? ' disabled' : '';

            $html[] = '<div class="option" style="float:left">';
            $html[] = '<input class="my-2" style="float:left" type="radio" id="' . $this->id . $i . '" name="' . $this->name . '" value="'
                . htmlspecialchars($option->value, ENT_COMPAT, 'UTF-8') . '"' . $checked . $class . $required . $disabled . ' />';

            $html[] = '<label class="my-2" for="' . $this->id . $i . '"' . $class . ' >' .
                '<img src="' . Uri::root() . 'media/sourcecoast/images/provider/' . $provider . '/' . $option->value . '" />' .
                #. Text::alt($option->text, preg_replace('/[^a-zA-Z0-9_\-]/', '_', $this->fieldname)) . '</label>'
                '</label>' .

                $required = '';
            $html[] = '</div>';
        }

        // End the radio field output.
        $html[] = '</fieldset>';
        $html[] = '<div style="clear: both"> </div>';

        return implode($html);
    }

    protected function getOptions()
    {
        // Scan the /media/sourcecoast/images/provider directory for this provider's buttons
        // Merge in any custom images from ??
        $provider = $this->element['provider'] ? (string)$this->element['provider'] : null;

        $options = array();
        $buttons = $this->getButtons('/media/sourcecoast/images/provider/' . $provider);
        if ($buttons)
        {
            foreach ($buttons as $button)
            {
                $options[] = HTMLHelper::_('select.option', $button, $button, 'value', 'text', false);
            }
        }

        reset($options);
        return $options;
    }

    private function getButtons($folder)
    {
        $folder = JPATH_SITE . $folder;
        $buttons = array();

        if (Folder::exists($folder))
        {
            $buttons = Folder::files($folder, '^' . '.*(\.png|\.jpg|\.gif|\.svg)$');
        }
        return $buttons;
    }
}
