<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Router\Route;
use Joomla\CMS\HTML\HTMLHelper;

HTMLHelper::addIncludePath(JPATH_COMPONENT . '/helpers/html');

HTMLHelper::_('bootstrap.tooltip');
HTMLHelper::_('formbehavior.chosen', 'select');
HTMLHelper::_('behavior.multiselect');

$userTableClass = "table table-striped table-condensed";

$input     = Factory::getApplication()->input;
$field     = $input->getCmd('field');
$function  = 'jSelectUser_' . $field;
$listOrder = $this->escape($this->state->get('list.ordering'));
$listDirn  = $this->escape($this->state->get('list.direction'));

$providerName  = $input->getString('provider');
$provider = JFBCFactory::provider($providerName);
$userList = $this->channelModel->getUserList($provider->systemName);

?>
<div class="sourcecoast ns-bs5">
    <form action="<?php echo Route::_('index.php?option=com_jfbconnect&view=channel&layout=user&provider='.$provider->systemName.'&tmpl=component');?>" method="post" name="adminForm" id="adminForm">
        <fieldset class="filter">
            <div id="filter-bar" class="btn-toolbar">
                <div class="filter-search btn-group float-left">
                    <label for="filter_search" class="element-invisible"><?php echo Text::_('JSEARCH_FILTER'); ?></label>
                    <input type="text" name="filter_search" id="filter_search" placeholder="<?php echo Text::_('JSEARCH_FILTER'); ?>" value="<?php echo $this->escape($this->state->get('filter.search')); ?>" class="hasTooltip" title="<?php echo HTMLHelper::tooltipText('COM_JFBCONNECT_FILTER'); ?>" data-placement="bottom"/>
                </div>
                <div class="btn-group float-left">
                    <button type="submit" class="btn hasTooltip" title="<?php echo HTMLHelper::tooltipText('JSEARCH_FILTER_SUBMIT'); ?>" data-placement="bottom"><span class="icon-search"></span></button>
                    <button type="button" class="btn hasTooltip" title="<?php echo HTMLHelper::tooltipText('JSEARCH_FILTER_CLEAR'); ?>" data-placement="bottom" onclick="document.id('filter_search').value='';this.form.submit();"><span class="icon-remove"></span></button>
                    <button type="button" class="btn" onclick="if (window.parent) window.parent.<?php echo $this->escape($function); ?>('', '<?php echo Text::_('JLIB_FORM_SELECT_USER'); ?>');"><?php echo Text::_('JOPTION_NO_USER'); ?></button>
                </div>
            </div>
        </fieldset>
        <h4><?php echo Text::sprintf('COM_JFBCONNECT_CHANNEL_USERS_AUTHENTICATED_LABEL', $provider->name);?></h4>
        <table class="<?php echo $userTableClass;?>">
            <thead>
            <tr>
                <th class="left">
                    <?php echo HTMLHelper::_('grid.sort', 'COM_JFBCONNECT_USERMAP_TITLE_JOOMLA_USER', 'a.name', $listDirn, $listOrder); ?>
                </th>
                <th class="nowrap" width="25%">
                    <?php echo HTMLHelper::_('grid.sort', 'JGLOBAL_USERNAME', 'a.username', $listDirn, $listOrder); ?>
                </th>
            </tr>
            </thead>
            <!--<tfoot>
            <tr>
                <td colspan="15">
                    <?php /*echo $this->pagination->getListFooter(); */?>
                </td>
            </tr>
            </tfoot>-->
            <tbody>
            <?php
            $i = 0;

            foreach ($userList as $item) : ?>
                <tr class="row<?php echo $i % 2; ?>">
                    <td>
                        <a class="pointer" onclick="if (window.parent) window.parent.<?php echo $this->escape($function);?>('<?php echo $item->id; ?>', '<?php echo $this->escape(addslashes($item->name)); ?>');">
                            <?php echo $item->name; ?></a>
                    </td>
                    <td align="center">
                        <?php echo $item->username; ?>
                    </td>
                    <!--<td align="left">
                        <?php /*echo nl2br($item->group_names);*/ ?>
                    </td>-->
                </tr>
            <?php endforeach; ?>
            </tbody>
        </table>
        <div>
            <input type="hidden" name="task" value="" />
            <input type="hidden" name="field" value="<?php echo $this->escape($field); ?>" />
            <input type="hidden" name="boxchecked" value="0" />
            <input type="hidden" name="filter_order" value="<?php echo $listOrder; ?>" />
            <input type="hidden" name="filter_order_Dir" value="<?php echo $listDirn; ?>" />
            <?php echo HTMLHelper::_('form.token'); ?>
        </div>
    </form>
</div>