<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\Registry\Registry;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\Table\Table;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Form\Form;

jimport('joomla.application.component.view');
jimport('sourcecoast.adminHelper');

require_once(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/includes/views.php');

include_once(JPATH_ADMINISTRATOR.'/components/com_jfbconnect/models/fields/channelusers.php');

class JFBConnectViewChannel extends JFBConnectAdminView
{
    function display($tpl = null)
    {
        $this->form = $this->get('Form');

        // Get the provider name either from the form data or the input (if the form hasn't been saved yet)
        $provider = $this->form->getData()->get('provider');
        if (!$provider) {
            $input = Factory::getApplication()->input;
            $provider = $input->get('provider');
        }
        Form::addFieldPath(JPATH_PLUGINS . '/jfbconnect/provider_' . $provider . '/provider/' . $provider . '/channel/fields');

        $this->state = $this->get('State');
        $this->item = $this->get('Item');
        $this->channelModel = $this->getModel('channel');
        $this->pagination = $this->get('Pagination');

        // In here, we're also going to override the form's saved value if type= is in the query string.
        // In the model, we also do a similar override in preprocessform
        $input = Factory::getApplication()->input;
        $qsType = $input->get('type', null);
        $qsProvider = $input->get('provider', null);
        if ($qsType)
            $this->form->setValue('type', null, $qsType);
        if ($qsProvider)
            $this->form->setValue('provider', null, $qsProvider);


        $input = Factory::getApplication()->input;
        if ($input->get('title', null))
            $this->form->getData()->set('title', $input->get('title', null, 'string'));
        if ($input->get('description', null))
            $this->form->getData()->set('description', $input->get('description', null, 'string'));

        Factory::getApplication()->getDocument()->addScriptDeclaration('var jfbc_language_click_save="' . Text::_('COM_JFBCONNECT_CHANNEL_CLICK_SAVE_LOAD_SETTINGS_LABEL') . '";');
        Factory::getApplication()->getDocument()->addScriptDeclaration('var jfbc_language_select_provider="' . Text::_('COM_JFBCONNECT_CHANNEL_SELECT_PROVIDER_CHANNEL_LABEL') . '";');

        /*
        if(JVERSION < 4.0)
        {
            Factory::getApplication()->getDocument()->addStyleSheet(Uri::root(true) . "/media/system/css/modal.css");
            Factory::getApplication()->getDocument()->addScript(Uri::root(true)."/media/system/js/mootools-core.js");
            Factory::getApplication()->getDocument()->addScript(Uri::root(true)."/media/system/js/mootools-more.js");
            Factory::getApplication()->getDocument()->addScript(Uri::root(true)."/media/system/js/core.js");
            Factory::getApplication()->getDocument()->addScript(Uri::root(true)."/media/system/js/modal.js");
        }*/

        $script = 'function jSelectUser_jform_attribs_user_id(id, title) {
         var old_id = document.getElementById("jform_attribs_user_id_id").value;
         if (old_id != id) 
         {
            document.getElementById("jform_attribs_user_id_id").value = id;
            document.getElementById("jform_attribs_user_id").value = title;
            document.getElementById("jform_attribs_user_id").className = document.getElementById("jform_attribs_user_id").className.replace(" invalid" , "");
            jfbcAdmin.channels.outbound.onuserchange(this.value)  
            jfbcJQuery("#usersModal_jform_attribs_user_id").modal("hide"); 
        }}';
        Factory::getApplication()->getDocument()->addScriptDeclaration($script);

        // Check for errors.
        if (count($errors = $this->get('Errors')))
        {
            throw new RuntimeException(implode("\n", $errors), 500);
        }

        parent::display($tpl);
    }

    public function addToolbar()
    {
        $title = Text::_('COM_JFBCONNECT_CHANNEL_TITLE');
        ToolBarHelper::title($title, 'jfbconnect.png');

        ToolBarHelper::apply('channel.apply', Text::_('COM_JFBCONNECT_BUTTON_SAVE'));
        ToolBarHelper::save('channel.save', Text::_('COM_JFBCONNECT_BUTTON_SAVEANDCLOSE'));
        ToolBarHelper::cancel('channel.cancel', Text::_('COM_JFBCONNECT_BUTTON_CANCEL'));

        SCAdminHelper::addAutotuneToolbarItem();
    }

    function displayUserSelection()
    {
        $this->_defaultModel = 'channel';
        $this->form = $this->get('Form');
        $this->state = $this->get('State');
        $this->item = $this->get('Item');

        // Check for errors.
        if (count($errors = $this->get('Errors')))
        {
            throw new RuntimeException(implode("\n", $errors), 500);
        }

        $attribs = '';
        // Only pull out the userlist field
        foreach ($this->form->getFieldset("attribs") as $field)
        {
            if ($field->type == "ChannelUserList")
                $attribs .= $this->formGetField($field);
        }

        return $attribs;
    }

    function displayAttributes()
    {
        $this->_defaultModel = 'channel';
        $this->form = $this->get('Form');
        $this->state = $this->get('State');
        $this->item = $this->get('Item');

        // Check for errors.
        if (count($errors = $this->get('Errors')))
        {
            throw new RuntimeException(implode("\n", $errors), 500);
        }

        $attribs = '';
        foreach ($this->form->getFieldset("attribs") as $field)
        {
            if ($field->type != "ChannelUserList")
                $attribs .= $this->formGetField($field);
        }

        $attribs .= $this->displayAutopostAttributes();

        return $attribs;
    }

    function displayAutopostAttributes()
    {
        $attribs = '';
        $provider = JFBCFactory::provider($this->form->getValue('provider'));
        $channel = $provider->channel($this->form->getValue('type'));
        if($channel->canPostToNetwork())
        {
            $checkType = false;

            foreach ($this->form->getFieldset("autopost") as $field)
            {
                $fieldName = $field->getAttribute('name');
                if ($fieldName == 'autopost_plugins')
                {
                    $val = $field->__get('value');
                    $checkType = $checkType || $this->checkIfPluginsAreContent($val);
                    $attribs .= $this->formGetField($field);
                }
                else if ($fieldName == 'autopost_objects')
                {
                    $vals = $field->__get('value');

                    $checkType = $checkType || $this->checkIfObjectsAreContent($vals);
                    $attribs .= $this->formGetField($field);
                }
            }

            $attribs .= $this->getAccessFields($this->form, $checkType);

        }
        return $attribs;
    }

    function checkIfPluginsAreContent($val)
    {
        if (is_array($val))
            return in_array('content', $val) || in_array('k2', $val);
        return false;
    }

    function checkIfObjectsAreContent($vals)
    {
        $obj = Table::getInstance('JFBConnectOpenGraphObject', 'Table');
        $check = false;
        if (is_array($vals))
        {
            foreach ($vals as $object)
            {
                $obj->load($object);
                $check = $obj->plugin == 'content' || $obj->plugin == 'k2';

                if ($check)
                    return true;
            }
        }
        return $check;
    }

    function getAccessFields($form, $display)
    {
        if (!$display)
            return '';

        foreach ($form->getFieldset("access") as $field)
        {

            $fieldName = $field->getAttribute('name');
            if ($fieldName == 'autopost_access_type')
                return $this->formGetField($field);
        }
    }

    function displayTest()
    {
        $app = Factory::getApplication();
        $id = $app->input->getInt('id', 0);
        $channels[] = strval($id);
        $options = new Registry();
        $options->set('show_provider', '1');
        $options->set('show_date', '1');
        $options->set('show_link', '1');
        $options->set('post_limit', '5');
        $options->set('datetime_format', Text::_('DATE_FORMAT_LC2'));
        $options->set('show_unpublished_channels', true);
        $stream = new JFBConnectStream($options, $channels);

        $app->getDocument()->addStyleSheet(Uri::root() . "media/sourcecoast/themes/scsocialstream/default/styles.css");
        $streamHtml = '<div class="sourcecoast socialstream ns-bs5">';
        $streamTest = $stream->getStreamHtml();
        if(empty($streamTest))
            $streamTest = '<div class="jfbc-error">'.Text::_('COM_JFBCONNECT_CHANNEL_EDIT_STREAM_EMPTY').'</div>';
        $streamHtml .= $streamTest;
        $streamHtml .= '</div>';
        echo $streamHtml;
    }

    public function testStream($id)
    {

    }
}