<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;

jimport('joomla.application.component.view');
jimport('sourcecoast.adminHelper');

require_once(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/includes/views.php');

class JFBConnectViewJfbconnect extends JFBConnectAdminView
{
    function display($tpl = null)
    {
        $configModel = JFBCFactory::config();
        $usermapModel = JFBCFactory::usermap();
        $autotuneModel = BaseDatabaseModel::getInstance('AutoTune', 'JFBConnectModel');

        $fbProvider = JFBCFactory::provider('facebook');
        if ($fbProvider && $fbProvider->appId)
        {
            $appConfig = JFBCFactory::config()->get('autotune_app_config', null);
            if (!$appConfig || count($appConfig) == 0)
            {
                JFBCFactory::log(Text::sprintf('COM_JFBCONNECT_MSG_RUN_AUTOTUNE', '<a href="index.php?option=com_jfbconnect&view=autotune">AutoTune</a>'), 'error');
            }
        }

        $userCounts = array();
        foreach (JFBCFactory::getAllProviders() as $p)
        {
            if ($p->appId)
                $userCounts[$p->systemName] = $usermapModel->getTotalMappings($p->systemName);
        }

        $this->configModel = $configModel;
        $this->autotuneModel = $autotuneModel;
        $this->usermapModel = $usermapModel;
        $this->userCounts = $userCounts;

        parent::display($tpl);
    }

    function addToolbar()
    {
        ToolBarHelper::title(Text::_('COM_JFBCONNECT'), 'jfbconnect.png');
        SCAdminHelper::addAutotuneToolbarItem();
    }

    function getFeed()
    {
        $feedHtml = JFBCFactory::cache()->get('sourcecoast.rss');
        if ($feedHtml === false)
        {
            $curl = curl_init();

            curl_setopt_array($curl, Array(
                CURLOPT_URL            => 'http://feeds.sourcecoast.com/sourcecoast-blog',
                CURLOPT_USERAGENT      => 'spider',
                CURLOPT_TIMEOUT        => 120,
                CURLOPT_CONNECTTIMEOUT => 30,
                CURLOPT_RETURNTRANSFER => TRUE,
                CURLOPT_ENCODING       => 'UTF-8'
            ));

            $data = curl_exec($curl);
            $errorCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            curl_close($curl);

            if($data && !empty($data) && $errorCode == 200)
            {
                $xml = simplexml_load_string($data, 'SimpleXMLElement', LIBXML_NOCDATA);
                if($xml)
                {
                    $feedHtml = '<h4>Recent News from SourceCoast</h4>';
                    for($i=0; $i<4; $i++) // Only show four items
                    {
                        if(isset($xml->channel->item[$i]))
                        {
                            $item = $xml->channel->item[$i];
                            $date = Factory::getDate($item->pubDate);
                            $dateStr = $date->format(Text::_('DATE_FORMAT_LC4'));
                            $feedHtml .= '<p><a href="'.$item->link.'">' . $item->title . '</a> <span><em>'.$dateStr.'</em></span></p>';
                        }
                    }
                }
            }
            JFBCFactory::cache()->store($feedHtml, 'sourcecoast.rss');
        }

        return $feedHtml;
    }

    function getDashIcon($link, $iconClass, $text, $target = "", $id = '')
    {
        $idClass = $id == '' ? '' : ' id="'.$id . '"';
        $targetLink = $target == "" ? "" : ' target="' . $target . '"';
        if(JVERSION < 4.0)
        {
            $html = '<div class="icon-wrapper"'.$idClass.'>
                        <div class="icon">
                            <a href="'.$link.'"'.$targetLink .'>
                                <i class="'.$iconClass.'" aria-hidden="true">&nbsp;</i>
                                <span>'.$text . '</span>
                            </a>
                        </div>
                    </div>';
        }
        else
        {
            $html = '<li class="quickicon quickicon-single"'.$idClass.'>
                        <ul class="list-unstyled d-flex w-100">
                            <li class="quickicon">
                                <a href="'.$link.'"'.$targetLink.'>
                                    <div class="quickicon-info">
                                        <div class="quickicon-icon">
                                            <div class="'.$iconClass.'" aria-hidden="true"></div>
                                        </div>
                                    </div>
                                    <div class="quickicon-name d-flex align-items-end">
                                        <span>'.$text.'</span>
                                    </div>
                                </a>
                            </li>
                        </ul>
                    </li>';
        }

        return $html;
    }
}