<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// no direct access
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Plugin\PluginHelper;

jimport('joomla.version');

?>
<div class="sourcecoast ns-bs5">
    <form method="post" id="adminForm" name="adminForm">
        <div class="row column-flex">
            <div class="col-md-9">
                <div class="card mb-3">
                    <div class="card-header">
                        <div class="module-actions dropdown">
                            <button type="button" data-bs-toggle="dropdown" aria-haspopup="true" aria-expanded="false" class="btn" id="dropdownMenuButton-SocialMeta">
                                <span class="icon-cog" aria-hidden="true"></span>
                                <span class="visually-hidden"><?php echo Text::_('COM_JFBCONNECT_OPENGRAPH_CONFIGURATION_EDIT_TEXT');?></span>
                            </button>
                            <div class="dropdown-menu dropdown-menu-end" aria-labelledby="dropdownMenuButton-SocialMeta">
                                <a class="dropdown-item" href="index.php?option=com_jfbconnect&view=opengraph&task=settings"><?php echo Text::_('COM_JFBCONNECT_OPENGRAPH_CONFIGURATION_EDIT_MENU');?></a>
                            </div>
                        </div>
                        <div>
                            <h2><?php echo Text::_('COM_JFBCONNECT_OPENGRAPH_CONFIGURATION_ADDOBJECT_LABEL');?></h2>
                        </div>
                    </div>
                    <div class="card-body">
                        <p><?php echo Text::_('COM_JFBCONNECT_OPENGRAPH_OVERVIEW_DESCRIPTION'); ?></p>
                        <nav class="quick-icons jfbcControlIcons px-3 pb-3" aria-label="Open Graph Objects">
                            <ul class="nav flex-wrap">
                                <?php if (count($this->plugins) > 0) {
                                    foreach ($this->plugins as $plugin) {
                                        foreach ($plugin->supportedObjects as $key => $value) {
                                            $title = $plugin->extensionName . ' - ' . $key;
                                            ?>
                                            <li class="quickicon-group">
                                                <ul class="list-unstyled d-flex w-100">
                                                    <?php echo $this->getDashIcon(Route::_('index.php?option=com_jfbconnect&controller=opengraph&task=objectedit&plugin=' . $plugin->pluginName . '&name=' . $value), $title); ?>
                                                </ul>
                                            </li>
                                        <?php } }
                                } else {
                                    echo "<li>".Text::_('COM_JFBCONNECT_OPENGRAPH_OBJECTCREATE_WARNING')."</li>";
                                }?>

                            </ul>
                        </nav>
                    </div>
                </div>
            </div>
            <div class="col-md-3">
                <div class="card mb-3">
                    <div class="card-header">
                        <h2><?php echo Text::_('COM_JFBCONNECT_OPENGRAPH_CONFIGURATION_GUIDE_LABEL');?></h2>
                    </div>
                    <div class="card-body">
                        <?php echo Text::sprintf('COM_JFBCONNECT_OPENGRAPH_CONFIGURATION_GUIDE_LINK', '<a
                    href="http://www.sourcecoast.com/jfbconnect/docs/facebook-open-graph-actions-for-joomla" target="_blank">', '</a>'); ?>
                    </div>
                </div>
            </div>
            <div class="col-md-12">
                <div class="card mb-3">
                    <div class="card-header">
                        <h2><?php echo Text::_('COM_JFBCONNECT_OPENGRAPH_CONFIGURATION_OBJECTS_LABEL');?></h2>
                    </div>
                    <div class="card-body">
                        <?php if ($this->objects) { ?>
                            <table class="adminlist table table-striped">
                                <thead>
                                <tr>
                                    <th class="title"><?php echo Text::_('COM_JFBCONNECT_OPENGRAPH_OBJECTS_NAME_LABEL');?></th>
                                    <th><?php echo Text::_('COM_JFBCONNECT_OPENGRAPH_OBJECTS_EXTENSION_LABEL');?></th>
                                    <th><?php echo Text::_('COM_JFBCONNECT_OPENGRAPH_OBJECTS_OBJECTTYPE_LABEL');?></th>
                                    <th><?php echo Text::_('JPUBLISHED');?></th>
                                </tr>
                                </thead>
                                <tbody>
                                <?php
                                $key = 0;
                                foreach ($this->objects as $object) :
                                    $pluginState = (PluginHelper::isEnabled('opengraph', $object->plugin))? "" : Text::_('COM_JFBCONNECT_OPENGRAPH_PLUGINS_DISABLED_WARNING');
                                    ?>
                                    <tr class="row<?php echo ($key % 2); ?>">
                                        <?php $key++; ?>
                                        <td><?php echo $object->display_name; ?></td>
                                        <td class="center"><?php echo ucwords($object->plugin . " - " . $object->system_name).$pluginState; ?></td>
                                        <td class="center"><?php echo $object->type ?></td>
                                        <td class="center"><?php echo HTMLHelper::_('jgrid.published', $object->published, 0, 'ogobjects.', false); ?></td>
                                    </tr>
                                <?php
                                endforeach;
                                ?>
                                </tbody>
                            </table>
                            <a href="index.php?option=com_jfbconnect&view=opengraph&task=objects" class="btn btn-secondary btn-sm float-end m-3">
                                <span class="fa fa-list" aria-hidden="true"></span><?php echo Text::_('COM_JFBCONNECT_OPENGRAPH_MANAGE_LABEL');?></a>
                        <?php } else {
                            echo Text::_('COM_JFBCONNECT_OPENGRAPH_CONFIGURATION_OBJECTS_NOOBJECTS_WARNING');
                        }?>
                    </div>
                </div>
            </div>
        </div>
        <input type="hidden" name="option" value="com_jfbconnect"/>
        <input type="hidden" name="controller" value="opengraph"/>
        <input type="hidden" name="task" value=""/>
        <?php echo HTMLHelper::_('form.token'); ?>
    </form>
</div>