<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Form\Form;
use Joomla\CMS\Form\FormHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Toolbar\Toolbar;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\Registry\Registry;

jimport('joomla.application.component.view');
jimport('sourcecoast.adminHelper');

require_once(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/includes/views.php');

class JFBConnectViewOpengraph extends JFBConnectAdminView
{
    var $objects;
    var $plugins;
    var $object;
    var $object_types;
    var $params;

    function display($tpl = null)
    {
        $title = Text::_('COM_JFBCONNECT_OPENGRAPH_TITLE');
        $app = Factory::getApplication();
        $bar = Toolbar::getInstance('toolbar');

        $layout = $this->getLayout();
        if ($layout != 'display' && $layout != 'default')
        {
            ToolBarHelper::custom('display', 'home', 'index.php?option=com_jfbconnect&view=opengraph', Text::_('COM_JFBCONNECT_OPENGRAPH_HOME'), false);
            ToolBarHelper::divider();
        }

        //Need to display list of available object types in multiple places
        PluginHelper::importPlugin('opengraph');
        $plugins = $app->triggerEvent('onOpenGraphGetPlugins');
        $this->plugins = $plugins;

        //Need to display list of created objects in multiple places
        $objectModel = $this->getModel('OpenGraphObject', 'JFBConnectAdminModel');
        $objects = $objectModel->getObjects();
        $this->objects = $objects;

        switch ($this->getLayout())
        {
            case 'objects':
                $title .= Text::_('COM_JFBCONNECT_OPENGRAPH_OBJECTS_TITLE');
                $bar->appendButton('Popup', 'new', Text::_('COM_JFBCONNECT_BUTTON_NEW'), 'index.php?option=com_jfbconnect&view=opengraph&task=objectcreate&tmpl=component&displaySidebar=false', 550, 400, '', '', '', Text::_('COM_JFBCONNECT_OPENGRAPH_OBJECTCREATE_LABEL'));

                ToolBarHelper::publishList();
                ToolBarHelper::unpublishList();
                ToolBarHelper::deleteList(Text::_('COM_JFBCONNECT_OPENGRAPH_OBJECTS_DELETE_CONFIRMATION'));
                break;

            // Modal popups for selecting the action/object to create
            case 'objectedit':
                $title .= Text::_('COM_JFBCONNECT_OPENGRAPH_OBJECTEDIT_TITLE');

                $model = $this->getModel('OpenGraphObject', 'JFBConnectAdminModel');
                $id = $app->input->getInt('id', 0);
                if ($id != 0)
                    $object = $model->getObject($id);
                else
                {
                    $plugin = $app->input->getString('plugin');
                    $name = $app->input->getString('name');

                    $object = new ogObject();
                    $object->loadDefaultObject($plugin, $name);
                }
                $this->object = $object;

                /* Get the current list of objects fetched from Autotune */
                $ogSettings = JFBCFactory::config()->get('general_autotune_network_settings');
                if($ogSettings)
                {
                    $registry = new Registry();
                    $registry->loadObject($ogSettings);
                    $objectTypes = $registry->get('open_graph.objects');
                }
                else
                    $objectTypes = array();
                if (empty($objectTypes))
                {
                    $none = new stdClass;
                    $none->name = "none";
                    $none->title = Text::_('COM_JFBCONNECT_OPENGRAPH_RUN_AUTOTUNE_WARNING');
                    $objectTypes = [$none];
                    Factory::getApplication()->enqueueMessage(Text::_('COM_JFBCONNECT_AUTOTUNE_NETWORKSETTINGS_NOT_LOADED'), 'error');
                }
                else
                {
                    ToolBarHelper::apply('apply', Text::_('COM_JFBCONNECT_BUTTON_SAVE'));
                    ToolBarHelper::save('save', Text::_('COM_JFBCONNECT_BUTTON_SAVEANDCLOSE'));
                }
                ToolBarHelper::cancel('cancel', Text::_('COM_JFBCONNECT_BUTTON_CANCEL'));

                $this->object_types = $objectTypes;

                // Load the params for this specific object
                jimport('joomla.filesystem.file');
                FormHelper::addFieldPath(JPATH_SITE . '/plugins/opengraph/' . $object->plugin . '/objects');
                $xml = JPATH_SITE . '/plugins/opengraph/' . $object->plugin . '/objects/' . $object->system_name . '.xml';
                if (File::exists($xml))
                {
                    $form = Form::getInstance('opengraph.' . $object->plugin . '.' . $object->system_name, $xml);
                    $form->bind(array('params' => $object->params->toArray()));
                }
                else
                    $form = null;
                $this->params = $form;
                break;

            case 'settings':
                $title .= Text::_('COM_JFBCONNECT_OPENGRAPH_SETTINGS_TITLE');
                ToolBarHelper::apply('apply', Text::_('COM_JFBCONNECT_BUTTON_SAVE'));
                ToolBarHelper::save('save', Text::_('COM_JFBCONNECT_BUTTON_SAVEANDCLOSE'));
                ToolBarHelper::cancel('cancel', Text::_('COM_JFBCONNECT_BUTTON_CANCEL'));

                Form::addFieldPath(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/models/fields');
                $this->formLoad('meta', JPATH_ADMINISTRATOR . '/components/com_jfbconnect/models/forms/opengraph_settings.xml');
                $this->formBind('meta', JFBCFactory::config()->getSettingsByPrefix('social_graph'));

                break;
            case 'default':
            case 'display':
                $bar->appendButton('Link', 'cog', 'COM_JFBCONNECT_OPENGRAPH_CONFIGURATION', 'index.php?option=com_jfbconnect&view=opengraph&task=settings');
                break;
        }
        ToolBarHelper::title($title, 'jfbconnect.png');

        parent::display($tpl);
    }

    public function addToolbar()
    {
        SCAdminHelper::addAutotuneToolbarItem();
    }

    function getDashIcon($link, $title)
    {
        if(JVERSION < 4.0)
        {
            $html = '<div class="icon-wrapper">
                        <div class="icon">
                            <a href="'.$link.'">
                                <i class="fa fa-object-ungroup fa-2x" aria-hidden="true">&nbsp;</i>
                                <span>'.$title . '</span>
                            </a>
                        </div>
                    </div>';
        }
        else
        {
            $html = '<li class="quickicon">
                        <a href="'.$link .'">
                            <div class="quickicon-info">
                                <div class="quickicon-icon">
                                    <div class="icon-cube" aria-hidden="true"></div>
                                </div>
                            </div>
                            <div class="quickicon-name d-flex align-items-end">
                                '.$title.'</div>
                        </a>
                    </li>
                    <li class="quickicon-linkadd j-links-link d-flex">
                        <a class="d-flex" href="'.$link.'" title="Add '.$title.'">
                            <span class="icon-plus" aria-hidden="true"></span>
                        </a>
                    </li>';
        }
        return $html;
    }
}