<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\Registry\Registry;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\Form\Form;

jimport('joomla.application.component.view');
jimport('sourcecoast.adminHelper');

require_once(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/includes/views.php');

class JFBConnectViewProfiles extends JFBConnectAdminView
{
    var $profilePlugins;

    function display($tpl = null)
    {
        $app = Factory::getApplication();
        PluginHelper::importPlugin('socialprofiles');
        $profilePlugins = $app->triggerEvent('onSocialProfilesGetPlugins');
        $this->profilePlugins = $profilePlugins;

        foreach ($profilePlugins as $p)
        {
            $options = array();
            $options['control'] = "profiles_" . $p->getName();
            JFBConnectUtilities::loadLanguage('plg_socialprofiles_' . $p->getName(), JPATH_ADMINISTRATOR);
            $path = JPATH_SITE . '/plugins/socialprofiles/' . $p->getName();
            Form::addFieldPath($path . '/fields/');
            $this->formLoad($p->getName(), $path . '/forms/config.xml', $options);

            $data = JFBCFactory::config()->get('profile_' . $p->getName());
            $reg = new Registry();
            $reg->loadString($data);
            $this->formBind($p->getName(), $reg);
        }
        parent::display($tpl);
    }

    function addToolbar()
    {
        ToolBarHelper::title(Text::_('COM_JFBCONNECT_PROFILES_TITLE'), 'jfbconnect.png');
        /** @var JFBConnectProvider $p */
        $settingsFetched = true;
        foreach (JFBCFactory::getAllProviders() as $p)
        {
            if (count($p->profile->getProviderFields()) <= 1)
            {
                $settingsFetched = false;
                break;
            }
        }

        if (!$settingsFetched)
        {
            Factory::getApplication()->enqueueMessage(Text::_('COM_JFBCONNECT_AUTOTUNE_NETWORKSETTINGS_NOT_LOADED'), 'error');
        }
        else
            ToolBarHelper::apply('apply', Text::_('COM_JFBCONNECT_BUTTON_APPLY_CHANGES'));
        SCAdminHelper::addAutotuneToolbarItem();
    }
}
