<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\Form\Form;

jimport('joomla.application.component.view');
jimport('sourcecoast.adminHelper');

require_once(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/includes/views.php');
require_once(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/controllers/providers.php');

class JFBConnectViewProviders extends JFBConnectAdminView
{
    var $layout = 'providers';

    var $allProviders;
    var $tabs;

    var $orderedProviders;

    function display($tpl = null)
    {
        $this->allProviders = JFBConnectControllerProviders::providers();
        $this->buildProviderDisplay();

        Form::addFieldPath(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/models/fields');
        $this->formLoad('config', JPATH_ADMINISTRATOR . '/components/com_jfbconnect/models/forms/config.xml');
        $this->formBind('config', JFBCFactory::config()->getSettings());

        foreach (JFBCFactory::getAllProviders() as $p)
        {
            $configPath = JPATH_SITE . '/plugins/jfbconnect/provider_'. strtolower($p->systemName).'/provider/'. strtolower($p->systemName) . '/config/';
            Form::addFieldPath(JPATH_PLUGINS . '/jfbconnect/provider_' . strtolower($p->systemName) . '/provider/'. strtolower($p->systemName) . '/config/fields');

            Form::addFieldPath($configPath);
            $this->formLoad($p->systemName, $configPath . 'config.xml');
            $loginField = '<form>' .
                '<fieldset name="login_button" label="' . Text::_('COM_JFBCONNECT_PROVIDER_MENU_LOGIN_BUTTON') . '">' .
                '<field type="providerloginbutton"
                    label="' . Text::_('COM_JFBCONNECT_CONFIG_LOGIN_BUTTON_DEFAULT_LABEL') . '"
                    description="' . Text::_('COM_JFBCONNECT_CONFIG_LOGIN_BUTTON_DEFAULT_DESC') . '"
                    provider="' . $p->systemName . '"
                    name="' . $p->systemName . '_login_button"
                    required="true"
                    default="icon_label.png"
                    class="login-button"
                    />' .
                '</fieldset>' .
                '</form>';
            $this->forms[$p->systemName]->load($loginField);
            $this->formBind($p->systemName, JFBCFactory::config()->getSettings());

            $files = JFBCFactory::getFilenamesFromDirectory($configPath, 'xml', array('config'));
            foreach($files as $file)
            {
                $tabName = $p->systemName . '_' . $file;
                $this->formLoad($tabName, $configPath.'/'. $file . '.xml');
                $this->formBind($tabName, JFBCFactory::config()->getSettingsByPrefix($file));
                $this->tabs[$p->systemName][] = $file;
            }
        }

        parent::display($tpl);
    }

    function addToolbar()
    {
        ToolBarHelper::title(Text::_('COM_JFBCONNECT_CONFIG_TITLE'), 'jfbconnect.png');
        ToolBarHelper::apply('apply', Text::_('COM_JFBCONNECT_BUTTON_APPLY_CHANGES'));
        SCAdminHelper::addAutotuneToolbarItem();
    }

    function buildProviderDisplay()
    {
        $installed = array();
        $uninstalled = array();

        foreach($this->allProviders as $systemName => $p)
        {
            if($p['installed'])
                $installed[$systemName] = $p;
            else
                $uninstalled[$systemName] = $p;
        }

        $this->orderedProviders = array_merge($installed, $uninstalled);
    }
}