<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\Registry\Registry;
use Joomla\CMS\Table\Table;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\HTML\HTMLHelper;

function showColumn($element)
{
    return ($element[0] != "_" && $element != "id" && $element != "provider_user_id");
}

$items = $this->_models[strtolower('UserMap')]->getList();
$row = Table::getInstance('UserMap', 'Table');
$columns = array_filter(array_keys(get_object_vars($row)), "showColumn");

include_once(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/models/usermap.php');
?>

<script>
    function addImage(containerId, url) {
        let avatarContainer = document.getElementById(containerId);

        UrlExists(url, avatarContainer, function(element, status){

            if( status === 200) // file found
            {
                element.innerHTML = '<img width="50" height="50" src="' + url +  '" />';
            }
        });
    }

    function UrlExists(url, element, cb){
        jfbcJQuery.ajax({
            url:      url,
            dataType: 'text',
            type:     'GET',
            complete:  function(xhr){
                if(typeof cb === 'function')
                    cb.apply(this, [element, xhr.status]);
            }
        });
    }
</script>

<div class="sourcecoast ns-bs5">
    <form action="index.php" method="post" id="adminForm" name="adminForm">
        <div id="filter-bar">
            <div class="filter-search mb-2">
                <div class="btn-group ">
                    <input type="text" name="search" id="search" value="<?php echo $this->lists['search']; ?>"
                           class="filter-search btn-group float-left"
                           title="<?php echo Text::_('COM_JFBCONNECT_FILTER'); ?>"
                           placeholder="<?php echo Text::_('COM_JFBCONNECT_FILTER'); ?>" />

                    <div class="btn-group">
                        <button class="btn btn-primary" id="jfbcSubmitButton" title="<?php echo Text::_('COM_JFBCONNECT_BUTTON_GO'); ?>" onclick="this.form.submit();">
                            <span class="icon-search"></span>
                        </button>
                        <?php
                        $resetJavascript = "document.getElementById('search').value='';";
                        foreach (array_keys($this->lists) as $key)
                        {
                            if ($key != 'search')
                            {
                                $resetJavascript .= "this.form.getElementById('" . $key . "').value='-1';";
                            }
                        }
                        $resetJavascript .= "this.form.submit();";
                        ?>
                        <button class="btn btn-secondary" id="jfbcResetButton" title="<?php echo Text::_('COM_JFBCONNECT_BUTTON_RESET'); ?>"
                                onclick="<?php echo $resetJavascript; ?>">
                            <span class="icon-remove"></span>
                        </button>
                    </div>
                </div>
                <div class="filter-select ml-2">
                    <?php
                    foreach (array_keys($this->lists) as $key)
                    {
                        if ($key != 'search' && $key != 'order' && $key != 'order_Dir')
                        {
                            echo $this->lists[$key];
                        }
                    }
                    ?>
                </div>
            </div>
        </div>
        <table class="adminlist table table-striped">
            <thead>
            <tr>
                <th width="5"><?php echo HTMLHelper::_('grid.sort', Text::_('COM_JFBCONNECT_USERMAP_TITLE_ID'), 'id', @$this->lists['order_Dir'], @$this->lists['order']); ?></th>
                <th width="20">
                    <?php
                    echo HTMLHelper::_('grid.checkall');
                    ?>
                </th>
                <th><?php echo HTMLHelper::_('grid.sort', Text::_('COM_JFBCONNECT_USERMAP_TITLE_JOOMLA_USER'), 'j_user_id', @$this->lists['order_Dir'], @$this->lists['order']); ?></th>
                <th><?php echo HTMLHelper::_('grid.sort', Text::_('COM_JFBCONNECT_USERMAP_TITLE_PROVIDER'), 'provider', @$this->lists['order_Dir'], @$this->lists['order']); ?></th>
                <th><?php echo HTMLHelper::_('grid.sort', Text::_('COM_JFBCONNECT_USERMAP_TITLE_FB_USER'), 'provider_user_id', @$this->lists['order_Dir'], @$this->lists['order']); ?></th>
                <th><?php echo HTMLHelper::_('grid.sort', Text::_('COM_JFBCONNECT_USERMAP_TITLE_FB_APP_AUTH'), 'authorized', @$this->lists['order_Dir'], @$this->lists['order']); ?></th>
                <th><?php echo HTMLHelper::_('grid.sort', Text::_('JGLOBAL_CREATED'), 'created_at', @$this->lists['order_Dir'], @$this->lists['order']); ?></th>
                <th><?php echo HTMLHelper::_('grid.sort', Text::_('JGLOBAL_MODIFIED'), 'updated_at', @$this->lists['order_Dir'], @$this->lists['order']); ?></th>
            </tr>
            </thead>
            <tfoot>
            <tr>
                <td colspan="10">
                    <?php echo $this->pagination->getListFooter(); ?>
                </td>
            </tr>
            </tfoot>
            <?php
            $k = 0;
            $deletedMappings = false;
            for ($i = 0, $n = count($items); $i < $n; $i++)
            {
                $row = & $items[$i];
                $checked = HTMLHelper::_('grid.id', $i, $row->id);

                $user = Factory::getUser($row->j_user_id);
                $params = new Registry();
                $params->loadString($row->params);
                ?>
                <tr class="<?php echo "row$k"; ?>">
                    <td>
                        <?php echo $row->id; ?>
                    </td>
                    <td>
                        <?php echo $checked; ?>
                    </td>
                    <td>
                        <a target="_blank" href="<?php print Route::_("index.php?option=com_users&view=user&task=user.edit&id=" . $row->j_user_id); ?>">
                            <?php print $user->name; ?>
                        </a>
                    </td>
                    <td>
                        <?php
                        if($params->get('profile_url') != '')
                            echo '<a target="_blank" href="' . $params->get('profile_url') . '">';
                        echo '<img src="' . Uri::root() . '/media/sourcecoast/images/provider/' . $row->provider . '/icon.png" title="'.ucfirst($row->provider).'"/>';
                        if($params->get('profile_url') != '')
                            echo '</a>';
                        ?>
                    </td>
                    <td align="center">
                        <?php
                        if($params->get('avatar_thumb') != '')
                        {
                            echo '<div id="avatarImageContainer'.$row->provider_user_id.'">';
                            echo '<script>addImage("avatarImageContainer'.$row->provider_user_id.'", "'.$params->get('avatar_thumb').'")</script>';
                            echo '</div>';
                        }
                        ?>
                    </td>
                    <td align="center">
                        <?php if ($row->provider == 'facebook')
                        {
                            if ($row->authorized)
                                echo '<img src="components/com_jfbconnect/assets/images/icon-16-allow.png" />';
                            else
                                echo '<img src="components/com_jfbconnect/assets/images/icon-16-deny.png" />';
                        }
                        else
                            echo ' - ';
                        ?>
                    </td>
                    <td><?php print $row->created_at; ?></td>
                    <td><?php print $row->updated_at; ?></td>
                </tr>
                <?php
                $k = 1 - $k;
            }
            ?>
        </table>


        <input type="hidden" name="option" value="com_jfbconnect" />
        <input type="hidden" name="task" value="" />
        <input type="hidden" name="permtype" value="" />
        <input type="hidden" name="boxchecked" value="0" />
        <input type="hidden" name="filter_order" value="<?php echo $this->lists['order']; ?>" />
        <input type="hidden" name="filter_order_Dir" value="<?php echo $this->lists['order_Dir']; ?>" />
        <input type="hidden" name="view" value="usermap" />
        <?php echo HTMLHelper::_('form.token'); ?>
    </form>
</div>