<?php
/**
 * @package         SourceCoast Extensions
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Factory;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;

define('SC_TYPE_ALL', '0');
define('SC_TYPE_INCLUDE', '1');
define('SC_TYPE_EXCLUDE', '2');
define('SC_VIEW_NONE', "0");
define('SC_VIEW_TOP', "1");
define('SC_VIEW_BOTTOM', "2");
define('SC_VIEW_BOTH', "3");
define('SC_VIEW_TITLE', "4");
define('SC_INTRO_CHARS', '1');
define('SC_INTRO_WORDS', '2');

class SCArticleContent
{
    static function getCurrentURL($article, $isJoomla)
    {
        if ($isJoomla)
            return SCArticleContent::_getCurrentArticleURL($article);
        else
            return SCArticleContent::_getCurrentItemURL($article);
    }

    private static function _getCurrentArticleURL($article)
    {
        if (JVERSION < 4.0)
        {
            require_once(JPATH_SITE . '/components/com_content/helpers/route.php');
        }
        $url = ContentHelperRoute::getArticleRoute($article->id, $article->catid);
        $url = SCArticleContent::_getCompleteURL($url);
        return $url;
    }

    private static function _getCurrentItemURL($article)
    {
        require_once(JPATH_SITE . '/components/com_k2/helpers/route.php');
        $url = K2HelperRoute::getItemRoute($article->id . ":" . urlencode($article->alias), $article->catid);
        $url = SCArticleContent::_getCompleteURL($url);
        return $url;
    }

    private static function _getCompleteURL($url)
    {
        $path = Route::_($url, true);
        $jUri = Uri::getInstance();

        $url = rtrim($jUri->toString(array('scheme', 'host', 'port')), '/');

        $config = Factory::getConfig();
        if($config->get('sef'))
        {
            // URL Encode the path only, but not the slashes within the path
            $url .= implode('/', array_map('urlencode', explode('/', $path)));;
        }
        else
        {
            $url .= $path;
        }

        $url = JFBConnectUtilities::forceScheme($url);
        return $url;
    }
}