<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Factory;
use Joomla\CMS\Table\Table;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Filesystem\File;

jimport('sourcecoast.articleContent');
jimport('sourcecoast.openGraphPlugin');

define('EB_POST_BLANK'      , 9);
define('EB_POST_DRAFT'      , 3);
define('EB_POST_PENDING'    , 4);
define('EB_POST_PUBLISHED'  , 1);
define('EB_POST_SCHEDULED'  , 2);
define('EB_POST_UNPUBLISHED', 0);
define('EB_POST_NORMAL', 0);
define('EB_POST_TRASHED', 1);
define('EB_POST_ARCHIVED', 2);

class plgOpenGraphEasyBlog extends OpenGraphPlugin
{
    protected function init()
    {
        $this->extensionName = "EasyBlog";
        $this->supportedComponents[] = 'com_easyblog';
        $this->supportedAutopostLabel = "Post";
        $this->supportedAutopostTypes[] = 'post';
        $this->setsDefaultTags = true;

        // Define all types of pages this component can create and would be 'objects'
        $this->addSupportedObject("Blog Post", "post");
//        $this->addSupportedObject("Category", "category");
    }

    protected function findObjectType($queryVars)
    {
        // Setup Object type for page
        $view = array_key_exists('view', $queryVars) ? $queryVars['view'] : '';

        // To determine if we are viewing normal entry layout or preview layout
        $layout = array_key_exists('layout', $queryVars) ? $queryVars['layout'] : '';
        $object = null;

        if ($view == 'entry')
        {
            $this->loadEB();
            $objectTypes = $this->getObjects('post');

            // If layout is preview, we need to get the proper blog id.
            // For preview layout, the id will be uid.
            $blogId = $layout && $layout == 'preview' ? $queryVars['uid'] : $queryVars['id'];

            //$post = EB::post($blogId); /* Do not use EB::post, not available in older versions EB */
            $post = $this->getEBTable('Blog');
            $post->load($blogId);

            if ($post)
            {
                $catId = $post->category_id;
                $object = $this->getBestCategory($objectTypes, $catId);
            }
        }

        return $object;
    }

    private function getBestCategory($objectTypes, $catId)
    {
        $object = null;
        if ($objectTypes)
        {
            $bestDistance = 99999;
            $this->db->setQuery("SELECT lft, rgt FROM #__easyblog_category WHERE id = " . $catId);
            $catLoc = $this->db->loadObject();
            foreach ($objectTypes as $type)
            {
                if($bestDistance == 99999 && $type->params->get('category') == 0)
                {
                    $object = $type;
                }
                else
                {
                    $this->db->setQuery("SELECT lft, rgt FROM #__easyblog_category WHERE id = " . $type->params->get('category'));
                    $result = $this->db->loadObject();
                    if ($result->lft <= $catLoc->lft && $result->rgt >= $catLoc->rgt)
                    {
                        $distance = $result->rgt - $result->lft;
                        if ($distance < $bestDistance)
                            $object = $type;
                        if ($distance == 1)
                            break;
                    }
                }
            }
        }
        return $object;
    }

    protected function setOpenGraphTags()
    {
        $desc = ''; //Note: meta is same as blank value, since system plugin attempts to generate from metadescription if no value is found
        $image = '';

        $view = $this->app->input->getCmd('view');

        if($this->object)
        {
            $desc_type = $this->object->params->get('custom_desc_type');
            $desc_length = $this->object->params->get('custom_desc_length');
            $image_type = $this->object->params->get('custom_image_type');
            $image_path = $this->object->params->get('custom_image_path');
            $title_type = $this->object->params->get('custom_title_type');
        }
        else
        {
            $desc_type = 'custom_desc_introwords';
            $desc_length = '50';
            $image_type = 'custom_image_item';
            $image_path = '';
            $title_type = 'custom_title_entry';
        }

        if ($view == 'entry')
        {
            $oldEasyBlogVersion = $this->loadEB();
            $item = $this->getEBTable('Blog');
            $item->load($this->app->input->getInt('id'));

            if($title_type == 'custom_title_entry')
            {
                $title = $item->title;
            }
            else //if($title_type == 'custom_title_page')
            {
                $document = Factory::getApplication()->getDocument();
                $title = $document->getTitle();
            }
            $this->addOpenGraphTag('title', $title, false);

            $desc = $this->getArticleTextFromParam($item, $desc_type, $desc_length);
            $this->addOpenGraphTag('description', $desc, false);

            //if ($image_type == 'custom_image_item')
            //{
            $image = $this->getEasyBlogMainImage($item, $oldEasyBlogVersion);
            //}
            if ($image_type == 'custom_image_first' || $image == '')
            {
                $tmpImage = $this->getFirstImageFromText($item->intro);
                if($tmpImage != '')
                    $image = $tmpImage;
                else
                {
                    $tmpImage = $this->getFirstImageFromText($item->content);
                    if($tmpImage != '')
                        $image = $tmpImage;
                }
            }
            if ($image_type == 'custom_image_category' || $image == '')
            {
                $tmpImage = $this->getEasyBlogCategoryImage($item->category_id);
                if($tmpImage != '')
                    $image = $tmpImage;
            }
            if (($image_type == 'custom_image_custom' || $image == '') && $image_path != '')
            {
                $image = $image_path;
            }
            $this->addOpenGraphTag('image', $image, false);
        }
    }

    /* Images and Descriptions */
    protected function getArticleTextFromParam($item, $desc_type, $desc_length)
    {
        $desc = '';
        $itemText = trim(strip_tags($item->intro)) . ' ' . trim(strip_tags($item->content));
        if ($desc_type == 'custom_desc_introwords')
            $desc = $this->getSelectedText($itemText, SC_INTRO_WORDS, $desc_length);
        else if ($desc_type == 'custom_desc_introchars')
            $desc = $this->getSelectedText($itemText, SC_INTRO_CHARS, $desc_length);
        return $desc;
    }

    protected function getEasyBlogMainImage($article, $oldEasyBlogVersion)
    {
        $url = '';

        if($oldEasyBlogVersion)
        {
            if (isset($article->image))
            {
                $image = json_decode($article->image);
                if (isset($image->url))
                {
                    $filePath = str_replace(Uri::root(), '', $image->url);

                    jimport('joomla.filesystem.file');
                    if (File::exists($filePath))
                        $url = $image->url;
                }
            }
        }
        else
        {
            $post = EB::post($article->id);
            $juri = Uri::getInstance();
            $scheme = $juri->getScheme();
            $image = $post->getImage('original');

            if($image && strpos($image, 'placeholder-image.png') === false)
            {
                //check if we add the scheme since the default image already have the scheme added
                if(strpos($image, $scheme) !== false)
                    $url = $image;
                else
                    $url = $scheme . ':'. $image;
            }
        }

        return $url;
    }

    protected function getEasyBlogCategoryImage($catid)
    {
        $url = '';
        $category = $this->getEasyBlogCategory($catid);

        if (isset($category->avatar))
        {
            $image = $category->avatar;
            $imageName = 'images/easyblog_cavatar/' . $image;

            jimport('joomla.filesystem.file');
            if (File::exists($imageName))
                $url = Uri::base() . $imageName;
        }
        return $url;
    }

    protected function getCurrentEasyBlogCategoryId()
    {
        $catid = $this->app->input->getInt('id');
        return $catid;
    }

    protected function getEasyBlogCategory($catid)
    {
        Table::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_easyblog/tables');
        $category = Table::getInstance('Category', 'EasyBlogTable');
        if ($category) // Some users have reported this not coming back. Haven't determined why, but this check should fix.
            $category->load($catid);
        return $category;
    }

    protected function getBestText($item)
    {
        $queryVars['view'] = 'entry';
        $queryVars['id'] = $item->id;
        $object = $this->findObjectType($queryVars);

        if($object)
        {
            $desc_type = $object->params->get('custom_desc_type');
            $desc_length = $object->params->get('custom_desc_length');
            return $this->getArticleTextFromParam($item, $desc_type, $desc_length);
        }
        else
        {
            $itemText = trim(strip_tags($item->intro)) . ' ' . trim(strip_tags($item->content));
            return $this->getSelectedText($itemText, SC_INTRO_WORDS, 20);
        }
    }

    /* Determine version of EasyBlog */
    protected function loadEB()
    {
        if(File::exists(JPATH_ADMINISTRATOR . '/components/com_easyblog/includes/easyblog.php'))
        {
            require_once(JPATH_ADMINISTRATOR . '/components/com_easyblog/includes/easyblog.php');
            return false;
        }
        else
        {
            require_once(JPATH_SITE . '/components/com_easyblog/helpers/helper.php');
            return true;
        }
    }

    protected function getEBTable($tableName)
    {
        if(File::exists(JPATH_ADMINISTRATOR . '/components/com_easyblog/includes/easyblog.php'))
            $rating = EB::table($tableName);
        else
            $rating = EasyBlogHelper::getTable($tableName, 'Table');
        return $rating;
    }

    protected function getEBConfig()
    {
        if(File::exists(JPATH_ADMINISTRATOR . '/components/com_easyblog/includes/easyblog.php'))
            $config = EB::table('Configs');
        else
            $config = EasyBlogHelper::getConfig();
        return $config;
    }

    protected function getEBRuleSet()
    {
        if(File::exists(JPATH_ADMINISTRATOR . '/components/com_easyblog/includes/easyblog.php'))
            $acl = EB::acl();
        else
            $acl = EasyBlogACLHelper::getRuleSet();

        return $acl;
    }

    /* Auto post */
    static $autopostArticle = null;
    public function onContentAfterSave($context, $article, $isNew)
    {
        if($context == 'easyblog.blog' && get_class($article) == 'EasyBlogPost')
        {
            $this->tryAutoPublish($article, $article->published);
        }
    }

    public function getAutopostMessage($ogObject, $id)
    {
        $message = $this->getBestText(self::$autopostArticle);
        return $message;
    }

    private function tryAutoPublish($article, $value)
    {
        self::$autopostArticle = $article;

        // Get object that $articleId belongs to
        $objectTypes = $this->getObjects('post');
        $ogObjects = $this->getObjectsForArticle($objectTypes, $article->category_id);

        if($article->state == EB_POST_NORMAL)
        {
            $isPending = $this->isArticlePublishPending($article->publish_up) || ($article->published == EB_POST_PENDING);

            if($value == EB_POST_PUBLISHED || $value == EB_POST_PENDING || $value == EB_POST_SCHEDULED)
            {
                $link = 'index.php?option=com_easyblog&view=entry&id='.$article->id;
                $this->autopublish($ogObjects, $article->id, $link, $isPending);
            }
            else if($isPending && $value == EB_POST_UNPUBLISHED)
            {
                $this->removePublish($ogObjects, $article->id);
            }
        }
        else
        {
            $this->removePublish($ogObjects, $article->id);
        }

        self::$autopostArticle = null;
    }

    public function openGraphUpdatePending($articleId, $link, $ext)
    {
        if($ext == $this->supportedComponents[0])
        {
            $this->loadEB();
            $blog = $this->getEBTable('Blog');
            $blog->load($articleId);
            self::$autopostArticle = $blog;

            $isPending = $this->isArticlePublishPending($blog->publish_up) || ($blog->published == EB_POST_PENDING);
            if(!$isPending)
            {
                // Get object that $articleId belongs to
                $objectTypes = $this->getObjects('post');
                $ogObjects = $this->getObjectsForArticle($objectTypes, $blog->category_id);

                $this->updatePending($ogObjects, $articleId, $link);
            }

            self::$autopostArticle = null;
        }
    }

    private function getObjectsForArticle($objectTypes, $articleCatId)
    {
        $validObjects = array();
        if ($objectTypes)
        {
            foreach ($objectTypes as $type)
            {
                if($type->params->get('category') == 0 || $type->params->get('category') == $articleCatId)
                {
                    $validObjects[] = $type;
                }
                else //is parent category
                {
                    $this->db->setQuery("SELECT lft, rgt FROM #__easyblog_category WHERE id = " . $type->params->get('category'));
                    $result = $this->db->loadObject();

                    if ($result->lft <= $articleCatId && $result->rgt >= $articleCatId)
                    {
                        $validObjects[] = $type;
                    }
                }
            }
        }
        return $validObjects;
    }

}