<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\Registry\Registry;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Form\Form;
use Joomla\CMS\Form\FormHelper;

jimport('joomla.form.helper');
FormHelper::loadFieldClass('list');

class JFormFieldWidgetSettings extends JFormField
{
    public $type = 'WidgetSettings';

    protected function getInput()
    {
        $html = '';
        if($this->form->getValue('params.provider_type') && $this->form->getValue('params.widget_type') != 'widget')
        {
            $provider = $this->form->getValue('params.provider_type');

            JFBConnectUtilities::loadLanguage('com_jfbconnect', JPATH_ADMINISTRATOR);
            JFBConnectUtilities::loadLanguage('plg_jfbconnect_provider_' . strtolower($provider), JPATH_ADMINISTRATOR);

            $xmlPath = JPATH_ROOT . '/plugins/jfbconnect/provider_'. $provider.'/provider/'. $provider . '/widget';
            Form::addFieldPath($xmlPath);
            $xmlFile = $xmlPath.'/'.$this->form->getValue('params.widget_type').'.xml';

            if(File::exists($xmlFile))
            {
                $options = array('control' => 'jform');
                $form = Form::getInstance('com_jfbconnect_' . $this->form->getValue('params.widget_type'), $xmlFile, $options);

                $registry = $this->form->getValue('params');

                $settings = new Registry();
                if(isset($registry->widget_settings))
                    $settings->set('params.widget_settings', $registry->widget_settings);
                $form->bind($settings);
                ob_start();
                foreach ($form->getFieldsets() as $fieldsets => $fieldset)
                {
                    foreach ($form->getFieldset($fieldset->name) as $field)
                        $this->formShowField($field);
                }
                $html = ob_get_clean();
            }
        }

        return '<div id="widget_settings">' . $html . '</div>';
    }

    protected function getLabel()
    {
        return '';
    }

    public function formShowField($field)
    {
        echo "  <div class=\"control-group\">\n";
        echo "    <div class=\"control-label\">\n";
        echo "   " . $field->getLabel() . "\n";
        echo "   </div>\n";
        echo "     <div class=\"controls\">\n";
        echo "       " . $field->getInput() . "\n";
        echo "     </div>\n";
        echo "  </div>\n";
    }

    public function renderField($options = array())
    {
        return $this->getInput();
    }
}
