<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\Session\Session;

jimport('joomla.application.component.controller');

class JFBConnectController extends BaseController
{
    public function __construct($config = array(), MVCFactoryInterface $factory = null, $app = null, $input = null)
    {
        if (!class_exists('JFBCFactory'))
        {
            $app = Factory::getApplication();
            $app->redirect('index.php', Text::_('COM_JFBCONNECT_ERROR_JFBCSYSTEM'));
        }
        parent::__construct($config, $factory, $app, $input);
    }

    function display($cachable = false, $urlparams = false)
    {
        $app = Factory::getApplication();

        // If the request comes here, it means the URL was index.php?option=com_jfbconnect (with no other query params)
        // At this point, we need to check if it's a possible Twitter authentication attempt and set things up properly.
        $oauth_token = $app->input->getString('oauth_token');
        $oauth_verifier = $app->input->getString('oauth_verifier');
        if ($oauth_token != "" && $oauth_verifier != "")
        {
            // Assume it's a twitter login and redirect appropriately
            $app->redirect('index.php?option=com_jfbconnect&task=authenticate.callback&provider=twitter&state=' . Session::getFormToken() . '&oauth_verifier=' . $oauth_verifier . '&oauth_token=' . $oauth_token );
        }

        parent::display();
    }

    function deauthorizeUser()
    {
        $fbProvider = JFBCFactory::provider('facebook');
        if($fbProvider)
        {
            $app = Factory::getApplication();
            $signedRequest = $app->input->post->get('signed_request', null);
            if ($signedRequest)
            {
                $parsed = $fbProvider->client->parseSignedRequest($signedRequest);
                $fbUserId = $parsed['user_id'];
                if ($fbUserId)
                {
                    BaseDatabaseModel::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/' . 'models');
                    $userModel = BaseDatabaseModel::getInstance('UserMap', 'JFBConnectModel');
                    $userModel->setAuthorized($fbUserId, '0');
                }
            }
        }
        exit;
    }

    function deleteData()
    {
        $fbProvider = JFBCFactory::provider('facebook');
        if($fbProvider)
        {
            $app = Factory::getApplication();
            $signedRequest = $app->input->get('signed_request', null);
            if ($signedRequest)
            {
                $parsed = $fbProvider->client->parseSignedRequest($signedRequest);
                $fbUserId = $parsed['user_id'];
                if ($fbUserId)
                {
                    BaseDatabaseModel::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/' . 'models');
                    $userModel = BaseDatabaseModel::getInstance('UserMap', 'JFBConnectModel');
                    $userModel->deleteMapping($fbUserId);
                }
            }
        }
        exit;
    }
}
