<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Session\Session;

jimport('joomla.user.helper');

class JFBConnectControllerAuthenticate extends JFBConnectController
{

    public function display($cachable = false, $urlparams = false)
    {
    }

    public function login()
    {
        // Removing this session check in v5.2.2 due to complaints of session token failures
        // Can investigate adding later. Still doing a check on the callback.
        //if (!Session::checkToken('get'))
        //    $this->redirect('index.php', Text::_('JLIB_ENVIRONMENT_SESSION_EXPIRED'));

        $provider = $this->getProvider();

        $this->saveReturnParameter();

        // Some providers will redirect here to go through the authentication flow, others will continue on.
        $provider->client->authenticate();

        $this->doLogin($provider);
    }

    public function callback()
    {
        $input = Factory::getApplication()->input;

        // Have to do our own token checking here. Redirect shouldn't happen to normal users since token was just inserted
        // when they tried to authenticate
        $token = Session::getFormToken();
        $returnToken = $input->get('state', '', 'alnum');
        if ($token != $returnToken)
        {
            $this->setRedirect('index.php', Text::_('JLIB_ENVIRONMENT_SESSION_EXPIRED'));
            $this->redirect();
        }

        if (Factory::getApplication()->getUserState('com_jfbconnect.login.return') == null)
            $this->saveReturnParameter();

        $provider = $this->getProvider();
        try
        {
            $provider->client->authenticate();
        }
        catch (Exception $e)
        {
            JFBCFactory::log(Text::sprintf('COM_JFBCONNECT_UNABLE_TO_RETRIEVE_USER', ucwords($provider->name)), 'warning');
            if (JFBCFactory::config()->get('facebook_display_errors'))
                JFBCFactory::log($e->getMessage(), 'error');

            $this->redirectOnFailedAuth();
        }
        $this->doLogin($provider);

    }

    private function doLogin($provider)
    {
        if ($provider->client->isAuthenticated())
        {
            $provider->setSessionToken();
            require_once(JPATH_COMPONENT . '/controllers/login.php');
            $loginController = new JFBConnectControllerLogin();

            // Set the users.login.form.redirect for the LanguageFilter plugin or anything else that expects it
            // Doesn't really matter where it goes as we redirect separately later, but this helps ensure compatibility with extensions
            // that are expected it to be set
            $lang = substr(Factory::getApplication()->getLanguage()->getDefault(), 0, 2);
            $url = rtrim(Uri::base(), '/') . Route::_('index.php?lang=' . $lang);
            Factory::getApplication()->setUserState('users.login.form.return', $url);

            // This will redirect the user on successful login, or return false if not
            $loginController->login($provider);
        }

        // If we get here, something failed
        $this->redirectOnFailedAuth();
    }

    private function getProvider()
    {
        $input = Factory::getApplication()->input;
        $provider = $input->getCmd('provider', null);
        if ($provider)
            return JFBCFactory::provider($provider);

        // No provider given, not a real call. Redirect to home
        $this->setRedirect('index.php');
    }

    private function saveReturnParameter()
    {
        $app = Factory::getApplication();
        //Save the current return parameter
        $returnParam = $app->input->getString('return', '');
        if ($returnParam != "")
        {
            $return = urlencode($returnParam); // Required for certain SEF extensions
            $return = rawurldecode($return);
            $return = base64_decode($return);
        }
        else
            $return = 'index.php';

        Factory::getApplication()->setUserState('com_jfbconnect.login.return', $return);
    }

    private function redirectOnFailedAuth()
    {
        $app = Factory::getApplication();
        $redirect = $app->getUserState('com_jfbconnect.login.return', 'index.php');
        $app->setUserState('com_jfbconnect.login.return', null);

        $this->setRedirect($redirect);
        $this->redirect();
    }

}
