<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\Registry\Registry;
use Joomla\CMS\Router\Route;
use Joomla\CMS\User\User;

class JFBConnectControllerLogin extends JFBConnectController
{

    function display($cachable = false, $urlparams = false)
    {
        Factory::getApplication()->redirect('index.php');
    }

    function login($provider = null)
    {
        $app = Factory::getApplication();

        if (!is_object($provider))
            $provider = JFBCFactory::provider($app->input->getCmd('provider'));
        // Let the provider do anything it wants before we try to login.
        $provider->onBeforeLogin();

        $loginRegisterModel = $this->getModel('LoginRegister', 'JFBConnectModel');

        // Set a cookie to prevent auto-logging in for the remainder of the session time
        $config = Factory::getConfig();
        $lifetime = $config->get('lifetime', 15);
        setcookie('jfbconnect_autologin_disable', 1, time() + ($lifetime * 60));
        // Not a perfect solution, but fixes autologin loops..

        $providerUserId = $provider->getProviderUserId();

        if (!$providerUserId)
        { # Facebook isn't returning information about this user.  Redirect them.
            JFBCFactory::log(Text::sprintf('COM_JFBCONNECT_UNABLE_TO_RETRIEVE_USER', $provider->name));
            $app->redirect('index.php');
        }

        $userMapModel = JFBCFactory::usermap();

        $jUser = Factory::getUser();
        if (!$jUser->guest) # User is already logged into Joomla. Update their mapping
        {
            //JFBConnectUtilities::clearJFBCNewMappingEnabled();
            if ($userMapModel->map($jUser->get('id'), $providerUserId, $provider->systemName, $provider->client->getToken()))
                JFBCFactory::log(Text::sprintf('COM_JFBCONNECT_MAP_USER_SUCCESS', $provider->name));
            else
                JFBCFactory::log(Text::sprintf('COM_JFBCONNECT_MAP_USER_FAIL', $provider->name));

            $return = $loginRegisterModel->getLoginRedirect($provider);
            $app->redirect($return);
        }

        // They're not logged in. Check if they have a Joomla user and log that user in. If not, create them one
        $jUserId = $userMapModel->getJoomlaUserId($providerUserId, $provider->systemName);

        if (!$jUserId)
        {
            $profile = $provider->profile->fetchProfile($providerUserId, array('email'));
            $providerEmail = $profile->get('email', null);

            # Check if automatic email mapping is allowed, and see if that email is registered
            # AND the Facebook user doesn't already have a Joomla account
            if (!$provider->initialRegistration && JFBCFactory::config()->get('facebook_auto_map_by_email'))
            {
                if ($providerEmail != null)
                {
                    $jUserEmailId = $userMapModel->getJoomlaUserIdFromEmail($providerEmail);
                    if (!empty($jUserEmailId))
                    {
                        // Found a user with the same email address
                        // do final check to make sure there isn't a FB account already mapped to it
                        $tempId = $userMapModel->getProviderUserId($jUserEmailId, $provider->systemName);
                        if (!$tempId)
                        {
                            //JFBConnectUtilities::clearJFBCNewMappingEnabled();
                            if ($userMapModel->map($jUserEmailId, $providerUserId, $provider->systemName, $provider->client->getToken()))
                            {
                                JFBCFactory::log(Text::sprintf('COM_JFBCONNECT_MAP_USER_SUCCESS', $provider->name));
                                $jUserId = $jUserEmailId; // Update the temp jId so that we login below
                            }
                            else
                                JFBCFactory::log(Text::sprintf('COM_JFBCONNECT_MAP_USER_FAIL', $provider->name));

                        }
                    }
                }
            }

            //check if user registration is turn off
            //!allowUserRegistration and !social_registration => registration not allowed
            //!allowUserRegistration and social_registration => registration allowed
            //allowUserRegistration and !social_registration => registration not allowed
            // ComponentHelper::getParams('com_users')->get('allowUserRegistration') check is not needed since
            // we prioritized the JFBConnect social registration config
            if(JFBCFactory::config()->get('social_registration') == 0 && !$jUserId)
            {
                JFBCFactory::log(Text::_('COM_JFBCONNECT_MSG_USER_REGISTRATION_DISABLED'), 'notice');
                $app->redirect(Route::_('index.php?option=com_users&view=login', false));
                return false;
            }

            // Check if no mapping, and Automatic Registration is set. If so, auto-create the new user.
            if (!$jUserId && JFBCFactory::config()->get('automatic_registration'))
            { # User is not in system, should create their account automatically
                if ($loginRegisterModel->autoCreateUser($providerUserId, $provider))
                    $jUserId = $userMapModel->getJoomlaUserId($providerUserId, $provider->systemName);
            }

            // At this point, we have nothing left to do but redirect the user to the registration page
            if (!$jUserId)
            {
                $return = $loginRegisterModel->getLoginRedirect($provider);
                $app = Factory::getApplication();
                $regComponent = JFBCFactory::config()->get('registration_component');
                if ($regComponent == 'jfbconnect')
                    $app->redirect(Route::_('index.php?option=com_jfbconnect&view=loginregister&provider=' . $provider->systemName . '&return=' . base64_encode($return), false));

                else
                {
                    $app->setUserState('com_jfbconnect.registration.alternateflow', true);
                    $app->setUserState('com_jfbconnect.registration.provider.name', $provider->systemName);
                    $app->setUserState('com_jfbconnect.registration.provider.user_id', $providerUserId);

                    $plugins = $app->triggerEvent('onSocialProfilesGetPlugins');
                    foreach ($plugins as $plugin)
                    {
                        if ($plugin->getName() == $regComponent)
                            $redirect = $plugin->registration_url;
                    }
                    if ($redirect)
                        $app->redirect(Route::_($redirect, false));
                }
            }
        }

        $jUser = User::getInstance($jUserId);

        $loginSuccess = false;
        // Try to log the user, but not if blocked and initial registration (then there will be a pretty message on how to activate)
        if (!$provider->initialRegistration || ($jUser->get('block') == 0 && $provider->initialRegistration))
        {
            $options = array('silent' => 1, 'provider' => $provider, 'provider_user_id' => $providerUserId); // Disable other authentication messages
            $password = $provider->secretKey;
            $_SERVER['REQUEST_METHOD'] = 'POST'; // Classify our authentication request as a POST to avoid a lot of SEF issues
            $loginSuccess = $app->login(array('username' => $provider->appId, 'password' => $password), $options);
        }

        if ($loginSuccess)
        {
            // lets update the user's access token with whatever we just received
            $jUser = Factory::getUser();
            $userMapModel->updateUserToken($jUser->get('id'), $provider->systemName, $provider->client->getToken());

            // Store Profile URL and Avatar URL
            $userModel = JFBConnectModelUserMap::getUser($jUser->get('id'), $provider->systemName);
            $userModel->saveParameter('profile_url', JFBCFactory::provider($provider->systemName)->profile->getProfileUrl($providerUserId));
            $avatarSettings = new Registry();
            $avatarSettings->set('width', 50);
            $avatarSettings->set('height', 50);
            $userModel->saveParameter('avatar_thumb', JFBCFactory::provider($provider->systemName)->profile->getAvatarUrl($providerUserId, false, $avatarSettings));

            if (!$provider->initialRegistration)
            {
                $args = array($provider->systemName, $jUser->get('id'), $providerUserId);
                $app->triggerEvent('onSocialProfilesOnLogin', $args);
            }
        }

        Factory::getApplication()->setUserState('com_jfbconnect.' . $provider->systemName . '.checkForNewMapping', null);
        $allProviders = JFBCFactory::getAllProviders();
        foreach ($allProviders as $p)
            $p->checkNewMapping();

        $redirect = $loginRegisterModel->getLoginRedirect($provider);

        // Clear the 'alternate flow' session bit to prevent further alterations to reg pages for this user/session
        $app->setUserState('com_jfbconnect.registration.alternateflow', false);

        $app->redirect($redirect);
    }
}
