<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Session\Session;

jimport('joomla.user.helper');

class JFBConnectControllerLoginRegister extends JFBConnectController
{
    var $_newUserPassword = "";

    function display($cachable = false, $urlparams = false)
    {
        $app = Factory::getApplication();
        $provider = $app->input->getCmd('provider');
        $app->setUserState('com_jfbconnect.' . $provider . '.checkForNewMapping', true);
        parent::display();
    }

    function createNewUser()
    {
        $loginRegisterModel = $this->getModel('LoginRegister', 'JFBConnectModel');
        $provider = Factory::getApplication()->input->getCmd('provider');
        $provider = JFBCFactory::provider($provider);
        if ($loginRegisterModel->createNewUser($provider))
        {
            require_once(JPATH_COMPONENT . '/controllers/login.php');
            $loginController = new JFBConnectControllerLogin();
            $loginController->login($provider);
        }
        else
        {
            $redirect = $loginRegisterModel->getLoginRedirect($provider);
            $returnParam = '&return=' . base64_encode($redirect);
            $this->setRedirect(Route::_('index.php?option=com_jfbconnect&view=loginregister&provider=' . $provider->systemName . $returnParam, false));
        }
    }

    public function loginMap()
    {
        Session::checkToken('post') or jexit(Text::_('JInvalid_Token'));
        $app = Factory::getApplication();

        $provider = Factory::getApplication()->input->getCmd('provider');
        $provider = JFBCFactory::provider($provider);

//        JFBConnectUtilities::setJFBCNewMappingEnabled();
        $loginRegisterModel = $this->getModel('LoginRegister', 'JFBConnectModel');
        $redirect = $loginRegisterModel->getLoginRedirect($provider);
        $returnParam = '&return=' . base64_encode($redirect);

        $providerUserId = $provider->getProviderUserId();
        if (!$providerUserId)
            $app->redirect(Route::_('index.php?option=com_jfbconnect&view=loginregister&provider=' . $provider->systemName . $returnParam, false));

        // Populate the data array:
        $data = array();
        $data['username'] = $app->input->get('username', '', 'method', 'username');
        $data['password'] = (string) $app->input->post->get('password', '', 'RAW');

        // Perform the log in.
        $error = $app->login($data);

        // Check if the log in succeeded.
        if ($error instanceof Exception || $error == false)
        {
            $app->redirect(Route::_('index.php?option=com_jfbconnect&view=loginregister&provider=' . $provider->systemName . $returnParam, false));
        }
        else //Logged in successfully
        {
            $jUser = Factory::getUser();
            if (JFBCFactory::usermap()->map($jUser->get('id'), $providerUserId, $provider->systemName, $provider->client->getToken()))
                JFBCFactory::log(Text::sprintf('COM_JFBCONNECT_MAP_USER_SUCCESS', $provider->name));

            BaseDatabaseModel::addIncludePath(JPATH_SITE . '/components/com_jfbconnect/' . 'models');
            $loginRegisterModel = BaseDatabaseModel::getInstance('LoginRegister', 'JFBConnectModel');
            $redirect = $loginRegisterModel->getLoginRedirect($provider);
            $app->redirect($redirect);
        }
    }

}
