<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Session\Session;

class JFBConnectControllerSocial extends JFBConnectController
{

    function display($cachable = false, $urlparams = false)
    {
        exit;
    }

    public function comment()
    {
        Session::checkToken('get') or die();

        $input = Factory::getApplication()->input;
        $href = $input->get('href', '', 'string');
        $href = urldecode($href);

        $type = $input->get('type');
        $title = base64_decode($input->get('title'));

        if (!$href || $href == "undefined" || !$type)
            exit;

        $this->awardPoints('facebook.comment.' . $type, $href);

        // Check if admin email should be sent
        if ($type != "create" || !JFBCFactory::config()->get('social_notification_comment_enabled'))
            exit;

        $subject = Text::_('COM_JFBCONNECT_NEW_COMMENT_SUBJECT');
        $body = Text::sprintf('COM_JFBCONNECT_NEW_COMMENT_BODY', $this->getPoster(), $title, $href);

        $this->sendEmail($subject, $body);
        exit;
    }

    public function share()
    {
        Session::checkToken('get') or die();

        $input = Factory::getApplication()->input;
        $href = $input->get('href', '', 'string');
        $id = $input->get('id', '', 'string');
        $href = urldecode($href);

        $provider = $input->get('provider', '', 'string');
        $share = $input->get('share', '', 'string');
        $type = $input->get('type');
        $title = $input->get('title');

        if ((!$id && (!$href || $href == "undefined")) || !$provider || !$share || !$type)
            exit;

        $this->awardPoints($provider . '.' . $share . '.' . $type, $href);

        exit;
    }

    private function awardPoints($name, $href)
    {
        $point = new JFBConnectPoint();
        $point->set('name', $name);
        $point->set('key', $href);
        $point->award();
    }

    private function getPoster()
    {
        $user = Factory::getUser();
        if ($user->guest)
            return "Guest";
        else
            return $user->get('username');
    }

    private function sendEmail($subject, $body)
    {
        $toname = JFBCFactory::config()->get('social_notification_email_address');
        $toname = explode(',', $toname);
        // Don't send emails to no one :)
        if ($toname[0] == "")
            return;

        $app = Factory::getApplication();
        $sitename = $app->getCfg('sitename');
        $mailfrom = $app->getCfg('mailfrom');
        $fromname = $app->getCfg('fromname');
        $subject = $subject . " - " . $sitename;

        Factory::getMailer()->sendMail($mailfrom, $fromname, $toname, $subject, $body);
    }


}
