<?php
/**
 * @package     Joomla.Platform
 * @subpackage  Google
 *
 * @copyright   Copyright (C) 2005 - 2013 Open Source Matters, Inc. All rights reserved
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\Registry\Registry;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Session\Session;
use Joomla\CMS\Input\Input;
use Joomla\CMS\Http\Http;

JLoader::register('JFBConnectAuthenticationOauth2Base', JPATH_SITE . '/components/com_jfbconnect/libraries/authentication/oauth2base.php');

/**
 * Google OAuth authentication class
 *
 * @package     Joomla.Platform
 * @subpackage  Google
 * @since       12.3
 */
class JFBConnectAuthenticationOauth2 extends JFBConnectAuthenticationOauth2Base
{
    /**
     * Constructor.
     *
     * @param   Registry $options  JGoogleAuth options object.
     * @param   application - WebApplicationInterface for 3.10+, WebApplication for 3.9-
     * @param   JOAuth2Client $client   OAuth client for Google authentication.
     *
     * @since   12.3
     */
    public $transport;
    public function __construct(Registry $options = null, Http $http = null, Input $input = null, $application = null)
    {
        // We want to use cURL, so need to build this all up. This also helps poke into each of the things when debugging
        $httpOptions = new Registry();
        $httpOptions->set('follow_location', false);
        try
        {
            $transport = new JFBConnectJoomlaHttpTransportCurl($httpOptions);
        }
        catch (Exception $e)
        {
            JFBCFactory::log('JFBConnect requires the CURL PHP extension to be installed and callable.', 'error');
            $transport = null;
        }
        $http = new Http($httpOptions, $transport);
        $this->transport = $transport;

        $options = isset($options) ? $options : new Registry;
        $options->set('state', Session::getFormToken()); // This is sent back to us to identify the provider
        if (!$options->exists('sendheaders'))
            $options->set('sendheaders', true);

        parent::__construct($options, $http, $input, $application);
    }

    protected $provider;
    public function initialize(JFBConnectProvider $p)
    {
        $this->provider = $p;
        $this->setOption('redirecturi', Uri::base() . 'index.php?option=com_jfbconnect&task=authenticate.callback&provider=' . $p->systemName);
        $this->setOption('clientid', $p->appId);
        $this->setOption('clientsecret', $p->secretKey);

        $this->setOption('authurl', $this->options->get('authurl'));
        $this->setOption('tokenurl', $this->options->get('tokenurl'));

        if (!$this->getOption('requestparams'))
        {
            $this->setOption('requestparams', Array());
        }

        $params = $this->getOption('requestparams');

        if (!array_key_exists('access_type', $params))
        {
            $params['access_type'] = 'offline';
        }
        if ($params['access_type'] == 'offline' && $this->getOption('userefresh') === null)
        {
            $this->setOption('userefresh', true);
        }
        if (!array_key_exists('approval_prompt', $params))
        {
            $params['approval_prompt'] = 'auto';
        }

        $this->setOption('requestparams', $params);
    }

    public function query($url, $data = null, $headers = array(), $method = 'get', $timeout = null)
    {
        $headers = array_merge($headers, $this->options->get('headers', array()));
        return parent::query($url, $data, $headers, $method);
    }
}

