<?php
/**
 * @package         SourceCoast Extensions
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Factory;

class JFBConnectEasytags
{
    private $tagsToReplace = null;

    public function initialize()
    {
        $this->scTags = self::getSCTags();
        $this->buildListOfTagsToReplace();
    }

    public static function getSCTags()
    {
        $scTagArray = array(
            "SCtwitch", "SCdiscord", "SCbattlenet", "SCpinterest", "SCeventbrite", "SCyahoo", "SCfacebook", "SCwindowslive", "SCamazon",
            "SCazure", "SCgithub", "SCgoogle", "SCinstagram", "SCmeetup", "SCvk", "SClinkedin", "SCtwitter");
        return implode('|', $scTagArray);
    }

    private function buildListOfTagsToReplace()
    {
        $providers = JFBCFactory::getAllProviders();

        $tagsToReplace = array();
        foreach ($providers as $p)
        {
            $tags = $p->buildListOfEasyTags();
            $tagsToReplace = array_merge($tagsToReplace, $tags);

        }
        //Manually add SCLinkedinLogin, since JLinkedLogin is the actual tag
        $tagsToReplace['sclinkedinlogin'] = array('provider' => 'linkedin', 'widget' => 'login');

        //Manually add JFBCLogin tag if Facebook provider is not enabled. Any of the login tags will be fine
        if(!array_key_exists('jfbclogin', $tagsToReplace))
        {
            foreach($tagsToReplace as $tagName => $tagValue)
            {
                if(str_ends_with($tagName, 'login'))
                {
                    $tagsToReplace['jfbclogin'] = $tagValue;
                    break;
                }
            }
        }

        //Tags like JFBCShare and JFBCRecommendations need to come up after JFBCShareDialog and JFBCRecommendationsBar
        $tagsToReplace = array_reverse($tagsToReplace);

        $this->tagsToReplace = $tagsToReplace;
    }

    public function replaceWidgetTags($contents)
    {
        //Tag like {JFBCTag} {JLinkedTag} {SCTag} {JFBCTag field=value field2=value2} {JLinkedTag field=value field2=value2} {SCTag field=value field2=value2}
        $regex = '/\{('.$this->scTags.'|JFBC|JLinked)(.*?)}/i';

        if (preg_match_all($regex, $contents, $matches, PREG_SET_ORDER))
        {
            $count = count($matches[0]);
            if ($count == 0)
                return $contents;

            $jfbcRenderKey = JFBCFactory::config()->get('social_tag_admin_key');

            foreach ($matches as $match)
            {
                $tagFields = explode(' ',$match[2]);
                $method = strtolower($match[1]) . strtolower($tagFields[0]);
                unset($tagFields[0]);
                $val = implode(' ', $tagFields);

                $params = $this->splitIntoTagParameters($val);
                $cannotRender = $this->cannotRenderEasyTag($params, $jfbcRenderKey);
                if ($cannotRender)
                    continue;

                if (array_key_exists($method, $this->tagsToReplace))
                {
                    $widgetInfo = $this->tagsToReplace[$method];
                    $fields = $this->getTagParameters($params);
                    $widget = JFBCFactory::widget($widgetInfo['provider'], $widgetInfo['widget'], $fields);
                    if($widget) //Cache only cleared in backend, prevent exception
                        $newText = $widget->render();
                    else
                        $newText = '';
                }
                else
                {
                    $newText = '';
                }

                $search = '/' . preg_quote($match[0], '/') . '/';
                $contents = preg_replace($search, $newText, $contents, 1);

            }
        }

        return $contents;
    }

    public function replaceJSPlaceholders($contents)
    {
        $javascript = '';

        $providers = JFBCFactory::getAllProviders();
        foreach ($providers as $provider)
        {
            $javascript .= $provider->getHeadData();
        }

//        $pinterestWidgets = JFBCFactory::getAllWidgets('pinterest');
//        $javascript .= $pinterestWidgets[0]->getHeadData();
//        JFBConnectProviderPinterestWidgetShare::$needsJavascript = false;

        $contents = str_replace('<SourceCoastProviderJSPlaceholder />', $javascript, $contents);
        return $contents;
    }

    public function replaceGraphTags($contents)
    {
        $app = Factory::getApplication();

        $placeholder = '<SCOpenGraphPlaceholder />';
        $regex1 = '/\{SCOpenGraph\s+(.*?)\}/i';
        $newGraphTags = array();

        $this->getGraphContents($regex1, $contents, $newGraphTags);

        //Replace Placeholder with new Head tags
        $defaultGraphFields = JFBCFactory::config()->get('social_graph_fields');
        $skipGraphFields = JFBCFactory::config()->get('social_graph_skip_fields');
        $skipComponents = JFBCFactory::config()->get('social_graph_skip_components', array());
        $locale = JFBCFactory::config()->getLocale();

        $openGraphLibrary = OpenGraphLibrary::getInstance();
        $openGraphLibrary->addBlockedTags($skipGraphFields);
        $openGraphLibrary->addOpenGraphEasyTags($newGraphTags);
        $openGraphLibrary->addDefaultSettingsTags($defaultGraphFields);
        $openGraphLibrary->addAutoGeneratedTags($locale);

        //Check to see if we need to exclude tags for this component
        $currentComponent = $app->input->getCmd('option');
        $blockComponents = array();
        if(!empty($skipComponents))
        {
            //$blockComponents = unserialize($skipComponents);//we do the unserialize in the config model
            $blockComponents = array_map('trim', $skipComponents); //Trim whitespace from each array element
        }

        if(empty($skipComponents) || !in_array($currentComponent, $blockComponents))
        {
            $graphTags = $openGraphLibrary->buildCompleteOpenGraphList();
            $contents = $openGraphLibrary->removeOverlappingTags($contents);
        }
        else
            $graphTags = '';

        $search = '/' . preg_quote($placeholder, '/') . '/';
        $graphTags = str_replace('$', '\$', $graphTags);
        $contents = preg_replace($search, $graphTags, $contents, 1);
        $contents = str_replace($placeholder, '', $contents);

        return $contents;
    }

    private function getGraphContents($regex, &$contents, &$newGraphTags)
    {
        if (preg_match_all($regex, $contents, $matches, PREG_SET_ORDER))
        {
            $count = count($matches[0]);
            if ($count == 0)
                return true;

            $jfbcRenderKey = JFBCFactory::config()->get('social_tag_admin_key');

            foreach ($matches as $match)
            {
                if (isset($match[1]))
                    $val = $match[1];
                else
                    $val = '';

                $params = $this->splitIntoTagParameters($val);
                $cannotRenderJFBC = $this->cannotRenderEasyTag($params, $jfbcRenderKey);

                if ($cannotRenderJFBC)
                    continue;

                $val = $this->removeRenderKey($val, $jfbcRenderKey);
                $newGraphTags[] = $val;
                $contents = str_replace($match[0], '', $contents);
            }
        }
    }

    private function removeRenderKey($easyTag, $renderKey)
    {
        if ($renderKey != '')
        {
            $key = 'key=' . $renderKey;
            $easyTag = str_ireplace($key . ' ', '', $easyTag); //Key with blank space
            $easyTag = str_ireplace($key, '', $easyTag);
            $easyTag = JFBConnectUtilities::trimNBSP($easyTag);
        }
        return $easyTag;
    }

    /*
     * Determines if the Easy-Tag can be rendered. If it can, then remove the render key
     */
    private function cannotRenderEasyTag(&$params, $renderKey)
    {
        $hasAKey = false;
        $hasRenderKey = false;
        $foundIndex = -1;

        //Currently there are some indices that are skipped, so count method doesn't
        //quite return what we're expecting. TODO: Fix this to not have skipped index
        end($params);
        $count = key($params) + 1;
        reset($params);

        for($i=0;$i<$count;$i++)
        {
            if(isset($params[$i]))
            {
                $p = $params[$i];
                if (stripos($p, "key=") === 0) //Key starts at position 0
                {
                    $hasAKey = true; //If render key is blank, but any key is set, we should not display it
                    $check = substr($p, 4);
                    if($check == false && $renderKey == '')
                    {
                        //Render key is blank, but a key is not really set, even though key= is present in the tag
                        $hasAKey = false;
                        $foundIndex = $i;
                    }
                    else if ($check == $renderKey)
                    {
                        $hasRenderKey = true;
                        $foundIndex = $i;
                    }
                }
            }
        }
        $canRender = ($renderKey == '' && !$hasAKey) || ($renderKey != '' && $hasRenderKey);

        //Remove the key from the parameter array before it gets rendered
        if($canRender && $foundIndex > -1)
            unset($params[$foundIndex]);

        return (!$canRender);
    }

    private function getTagParameters($params)
    {
        $newFields = array();
        foreach($params as $param)
        {
            if($param != null)
            {
                $paramValues = explode('=', $param, 2);
                if (count($paramValues) == 2) //[0] name [1] value
                {
                    $fieldName = strtolower(trim($paramValues[0]));
                    $fieldValue = trim($paramValues[1]);

                    $newFields[$fieldName] = $fieldValue;
                }
            }
        }
        return $newFields;
    }

    private function splitIntoTagParameters($paramList)
    {
        $paramList = JFBConnectUtilities::replaceNBSPWithSpace($paramList);
        $params = explode(' ', $paramList);

        $count = count($params);
        for ($i = 0; $i < $count; $i++)
        {
            $params[$i] = str_replace('"', '', $params[$i]);
            if (strpos($params[$i], '=') === false && $i > 0)
            {
                $previousIndex = $this->findPreviousParameter($params, $i - 1);
                //Combine this with previous entry and space
                $combinedParamValue = $params[$previousIndex] . ' ' . $params[$i];
                $params[$previousIndex] = $combinedParamValue;
                unset($params[$i]);
            }
        }
        return $params;
    }

    private function findPreviousParameter($params, $i)
    {
        for ($index = $i; $index >= 0; $index--)
        {
            if (isset($params[$index]))
                return $index;
        }
        return 0;
    }

    public function canExtendJoomlaForm($selView = 'login', $isConnected = false, $userFormPosition = '1')
    {
        $app = Factory::getApplication();
        $option = $app->input->getCmd('option');
        $view = $app->input->getCmd('view');
        $user = Factory::getUser();

        if ($option == 'com_users' && $view == $selView)
        {
            if (($view == 'login' && $user->guest) ||
                ($view == 'registration' && $user->guest) ||
                ($view == 'profile' && $isConnected)
            )
            {
                if($userFormPosition == SC_VIEW_TOP || $userFormPosition == SC_VIEW_BOTH)
                {
                    return true;
                }
                if($userFormPosition == SC_VIEW_BOTTOM || $userFormPosition == SC_VIEW_BOTH)
                {
                    return true;
                }
            }
        }

        return false;
    }

    /* Important: Call this method in conjunction with canExtendJoomlaForm to see if the HTML should
    actually be added to the page. Split up like this for performance improvements*/
    public function extendJoomlaUserForms($htmlTag, $userFormPosition = '1')
    {
        $document = Factory::getApplication()->getDocument();
        $output = $document->getBuffer('component');

        if($userFormPosition == SC_VIEW_TOP || $userFormPosition == SC_VIEW_BOTH)
        {
            $output = $htmlTag . $output;
        }
        if($userFormPosition == SC_VIEW_BOTTOM || $userFormPosition == SC_VIEW_BOTH)
        {
            $output = $output . $htmlTag;
        }

        $document->setBuffer($output, 'component');
    }

    /*
     * Given JFBCLike layout of box_count, button_count or standard, returns
     * the parameters to set the same layout for the other share buttons
     */
    public function getShareButtonLayout($provider, $layout, $addQuotes='', $size="")
    {
        $layoutParams = '';
        if($layout == 'box_count')
        {
            if($provider == 'facebook')
                $layoutParams = $this->quoteParam($addQuotes, 'layout', 'box_count');
            else if($provider == 'twitter')
            {
                if(!empty($size))
                    $layoutParams = $this->quoteParam($addQuotes, 'data-size', $size);
            }
            else if($provider == 'linkedin')
                $layoutParams = $this->quoteParam($addQuotes, 'data-counter', 'top');
            else if($provider == 'pinterest')
                $layoutParams = $this->quoteParam($addQuotes, 'data-pin-config','above');

        }
        else if($layout == 'button_count')
        {
            if($provider == 'facebook')
                $layoutParams = $this->quoteParam($addQuotes, 'layout', 'button_count');
            else if($provider == 'twitter')
            {
                if(!empty($size))
                    $layoutParams = $this->quoteParam($addQuotes, 'data-size', $size);
            }

            else if($provider == 'linkedin')
                $layoutParams = $this->quoteParam($addQuotes, 'data-counter', 'right');
            else if($provider == 'pinterest')
                $layoutParams = $this->quoteParam($addQuotes, 'data-pin-config','beside');
        }
        else //if($layout == 'standard' || $layout == 'button')
        {
            if($provider == 'facebook')
            {
                if($layout == 'standard')
                {
                    $layoutParams = $this->quoteParam($addQuotes, 'layout', 'standard');
                    $layoutParams .= $this->quoteParam($addQuotes, 'width', '50');
                    $layoutParams .= $this->quoteParam($addQuotes, 'show_faces', 'false');
                }
                else if($layout == 'button')
                    $layoutParams = $this->quoteParam($addQuotes, 'layout', 'button');
            }
            else if($provider == 'twitter')
            {
                if(!empty($size))
                    $layoutParams = $this->quoteParam($addQuotes, 'data-size', $size);
            }
            else if($provider == 'linkedin')
                $layoutParams = $this->quoteParam($addQuotes, 'data-counter', 'no_count');
            else if($provider == 'pinterest')
                $layoutParams = $this->quoteParam($addQuotes, 'data-pin-config','none');
        }

        return $layoutParams;
    }

    private function quoteParam($quote, $option, $value)
    {
        return ' '.$option.'='.$quote.$value.$quote;
    }
}