<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\CMS\Filesystem\Folder;

class JFBCFactory
{
    public static function initializeJoomla()
    {
        require_once JPATH_SITE . '/components/com_jfbconnect/autoloader.php';
        JFBConnectAutoloader::register('Joomla');

        JLoader::register('JFBCOAuth1Client', JPATH_SITE . '/components/com_jfbconnect/libraries/joomla/oauth1/client.php');
        JLoader::register('JFBConnectAuthenticationOauth2', JPATH_SITE . '/components/com_jfbconnect/libraries/authentication/oauth2.php');
        JLoader::register('JFBConnectAuthenticationImplicitFlow', JPATH_SITE . '/components/com_jfbconnect/libraries/authentication/implicitflow.php');

        include_once(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/models/usermap.php');

        jimport('joomla.filesystem.folder');
        PluginHelper::importPlugin('jfbconnect');
    }

    public static function initializeWordpress()
    {
        require_once ABSPATH . 'wp-content/plugins/jfbconnect/jfbconnect/component/frontend/autoloader.php';

        JFBConnectAutoloader::register('Wordpress');
        require_once ABSPATH . 'wp-content/plugins/jfbconnect/jfbconnect/wordpress/shims/registry.php';

//        JLoader::register('JFBCOAuth1Client', JPATH_SITE . '/components/com_jfbconnect/libraries/joomla/oauth1/client.php');
//        JLoader::register('JFBConnectAuthenticationOauth2', JPATH_SITE . '/components/com_jfbconnect/libraries/authentication/oauth2.php');

//        include_once(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/models/usermap.php');

//        jimport('joomla.filesystem.folder');
    }

    public static function provider($name)
    {
        $name = strtolower($name);

        static $providers = array();
        if (!isset($providers[$name]))
        {
            $className = 'JFBConnectProvider' . ucfirst($name);

            if (class_exists($className))
            {
                // Do not clean this up. It's ugly, but needs to be to prevent recursive loading
                $providers[$name] = new $className();
                $providers[$name]->setupAuthentication();
            }
        }
        if (isset($providers[$name]))
            return $providers[$name];
        else
            return null;
    }


    public static function getAllProviders()
    {
        static $allProviders;
        if (!isset($allProviders))
        {
            $allProviders = array();
            $files = array();

            //Plugin files
            $app = Factory::getApplication();
            $pluginProviders = $app->triggerEvent('onGetSocialLoginProviders');
            foreach($pluginProviders as $provider)
            {
                $directory = JPATH_SITE . '/plugins/jfbconnect/provider_'.$provider.'/provider/';
                $pluginProviderFiles = JFBCFactory::getProviderFilenames($directory);
                $files = array_merge($files, $pluginProviderFiles);
            }

            sort($files);
            foreach ($files as $f)
            {
                $p = self::provider($f);
                if ($p)
                    $allProviders[] = $p;
            }
        }
        return $allProviders;
    }

    public static function getProviderFilenames($directory)
    {
        return self::getFilenamesFromDirectory($directory, 'php');
    }

    public static function getFilenamesFromDirectory($directory, $extension, $exclude = array())
    {
        $iterator = new DirectoryIterator ($directory);
        $files = array();

        if(Folder::exists($directory))
        {
            foreach ($iterator as $info)
            {
                if ($info->isFile() && $info->getExtension() == $extension)
                {
                    $name = str_replace(".".$extension, "", $info->__toString());
                    if(!in_array($name, $exclude))
                        $files[] = $name;
                }
            }
        }
        return $files;
    }


    public static function library($path)
    {
        static $loaded;
        if (!isset($loaded[$path]))
        {
            $parts = explode('.', $path);
            $path = implode('/', $parts);
            $className = 'JFBConnect' . implode('', array_map('ucfirst', $parts));
            require_once(JPATH_SITE . '/components/com_jfbconnect/libraries/' . $path . '.php');
            $loaded[$path] = new $className();
        }
        return $loaded[$path];
    }

    public static function model($name)
    {
        BaseDatabaseModel::addIncludePath(JPATH_SITE . '/components/com_jfbconnect/models');
        $model = BaseDatabaseModel::getInstance($name, 'JFBConnectModel');
        return $model;
    }

    public static function config()
    {
        static $configModel = null;
        if (!isset($configModel))
        {
            require_once(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/models/config.php');
            $configModel = new JFBConnectModelConfig();
        }
        return $configModel;
    }

    public static function cache()
    {
        static $cache = null;
        if (!isset($cache))
        {
            require_once(__DIR__ . '/cache.php');
            $cache = new JFBConnectCache();
        }
        return $cache;
    }

    public static function easytags()
    {
        static $tags = null;
        if (!isset($tags))
        {
            require_once(__DIR__ . '/easytags.php');
            $tags = new JFBConnectEasytags();
            $tags->initialize();
        }
        return $tags;
    }

    /**
     * Method to log issues to file and display then on frontend
     * @param string $msg - The message it self to be logged or displayed.
     * @param string $type - The joomla message type (message | warning | notice | error)
     * @param boolean $debug
     * @return void
     */
    public static function log($msg = '', $type = 'message', $debug = false)
    {
        static $debugC = null;
        if (!isset($debugC))
        {
            require_once(JPATH_SITE . '/components/com_jfbconnect/libraries/debug.php');
            $debugC = new JFBConnectDebug();
        }

        $debugC->add($msg, $type, $debug);
    }

    // Return an instance of the usermap model, always creating it
    public static function usermap()
    {
        $userMapModel = new JFBConnectModelUserMap();
        return $userMapModel;
    }

    public static function widget($providerName, $name, $params = "")
    {
        $className = 'JFBConnectProvider' . ucfirst($providerName) . 'Widget' . ucfirst($name);
        $provider = self::provider($providerName);
        if (class_exists($className))
            $widget = new $className($provider, $params);
        else
            $widget = null;
        return $widget;
    }

    public static function getAllWidgets($provider)
    {
        static $allWidgets;

        if (!$provider || $provider == 'provider')
            return array();

        if (!isset($allWidgets))
            $allWidgets = array();

        if (!isset($allWidgets[$provider]))
        {
            $app = Factory::getApplication();
            $pluginProviders = $app->triggerEvent('onGetSocialLoginProviders');
            foreach($pluginProviders as $providerName)
            {
                if($provider == $providerName)
                {
                    $widgetFolder = JPATH_SITE . '/plugins/jfbconnect/provider_'.$providerName.'/provider/'.$providerName.'/widget/';
                    $allWidgets[$providerName] = JFBCFactory::getWidgets($providerName, $widgetFolder);
                }
            }
        }
        return $allWidgets[$provider];
    }

    public static function getWidgets($provider, $widgetFolder)
    {
        $widgets = array();
        if(Folder::exists($widgetFolder))
        {
            $iterator = new DirectoryIterator ($widgetFolder);
            foreach ($iterator as $info)
            {
                if ($info->isFile() && $info->getExtension() == 'xml')
                {
                    $widgets[] = self::widget($provider, str_replace(".xml", "", $info->getFilename()));
                }
            }
        }
        return $widgets;
    }

    public static function getAllAutopostExtensions()
    {
        static $allAutopostExt;

        if (!isset($allAutopostExt))
        {
            $allAutopostExt = array();

            $app = Factory::getApplication();
            PluginHelper::importPlugin('opengraph');
            $ogPlugins = $app->triggerEvent('onOpenGraphProfilesGetPlugins');

            foreach ($ogPlugins as $plugin)
            {
                if ($plugin->supportedAutopostTypes)
                    $allAutopostExt = array_merge($allAutopostExt, $plugin->supportedComponents);
            }
        }
        return $allAutopostExt;
    }

    public static function getLoginButtons($params = array())
    {
        $allProviders = self::getAllProviders();
        if(count($allProviders) == 0) //No enabled providers. Nothing to render
            return '';

        //Build up list of provider buttons to show if not passed in by SCLogin or third party extension
        if(!array_key_exists('providers', $params))
        {
            $providers = array();
            foreach($allProviders as $p)
                $providers[] = $p->systemName;
            $params['providers'] = implode(',', $providers);
        }

        // Any enabled provider can actually be used here. All roads lead to the same login widget
        $providerName = self::getEnabledProvider();

        //Login widget will return buttons for all providers in the list
        return JFBCFactory::widget($providerName, 'login', $params)->render();
    }

    public static function getReconnectButtons($params = null)
    {
        $providerName = self::getEnabledProvider();
        $params['show_reconnect'] = 'true';
        return JFBCFactory::widget($providerName, 'login', $params)->render();
    }

    public static function getEnabledProvider()
    {
        $provider = '';

        foreach (JFBCFactory::getAllProviders() as $p)
        {
            $provider = $p->systemName;
            break;
        }
        return $provider;
    }

    /***
     * Adds a stylesheet to the list of inclusions that need to be added to the page
     * Managed this way as some are added before the page is rendered, and some after
     * @var path to filename relative to /components/com_jfbconnect/
     */
    static protected $css = array();

    public static function addStylesheet($name)
    {
        if (!in_array($name, self::$css))
            self::$css[] = $name;
    }

    public static function getStylesheets()
    {
        return self::$css;
    }

}