<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Factory;
use Joomla\Registry\Registry;

class JFBConnectProfile
{
    /** @var JFBConnectProvider */
    var $provider;
    var $providerFields;

    public function __construct($provider)
    {
        $this->provider = $provider;
        $this->setProviderFields();

        JFBConnectUtilities::loadLanguage('plg_jfbconnect_provider_' . strtolower($this->provider->systemName), JPATH_ADMINISTRATOR);
    }

    protected function setProviderFields()
    {
        $this->providerFields = ['0' => 'None'];
        $this->providerFields = array_merge($this->providerFields, (array)$this->provider->networkSettings->get('profile_fields'));
    }

    public function getPermissionsForFields($fields)
    {
        $perms = array();
        $availableFields = $this->provider->networkSettings->get('profile_fields');
        if (!empty($availableFields) && !empty($fields))
        {

            foreach ($fields as $field)
            {
                foreach ($availableFields as $key => $af)
                {
                    if ($key == $field)
                    {
                        if (property_exists($af, 'scope'))
                            $perms[] = $af->scope;
                        break;
                    }
                }
            }
        }
        return $perms;
    }


    public function getProviderFields()
    {
        return $this->providerFields;
    }

    /**
     *  Get all permissions that are required by Facebook for email, status, and/or profile, regardless
     *    of whether they're set to required in JFBConnect
     * @return array - list of permissions that are required
     */
    static private $requiredScope = [];

    public function getRequiredScope()
    {
        return self::$requiredScope;
    }

    /*
     * Fetch a user's profile based on a profile plugin field-mapping
     * @return Registry with profile field values
     */
    public function fetchProfileFromFieldMap($fieldMap, $permissionGranted = true)
    {
        $fields = array();
        if (is_object($fieldMap))
        {
            foreach ($fieldMap as $field)
            {
                $fieldArray = explode('.', $field);
                if (!empty($fieldArray[0]))
                    $fields[] = $fieldArray[0]; // Get the root field to grab from FB
            }
        }
        $providerUserId = $this->provider->getProviderUserId();

        $fields = array_unique($fields);
        $profile = $this->fetchProfile($providerUserId, $fields);
        $profile->setFieldMap($fieldMap);
        return $profile;
    }

    /* Fetch a user's profile based on the passed in array of fields
    * @return Registry with profile field values
    */
    public function fetchProfile($providerUserId, $fields)
    {
        $profile = new JFBConnectProfileData();
        return $profile;
    }

    public function fetchStatus($providerUserId)
    {
        return "";
    }


    // Created for parity with JLinked/SourceCoast library
    // nullForDefault - If the avatar is the default image for the social network, return null instead
    // Prevents the default avatars from being imported
    function getAvatarUrl($fbUserId, $nullForDefault = true, $params = null)
    {
        return null;
    }

    function getProfileUrl($providerId)
    {
        return null;
    }

    function getCoverPhoto($providerId)
    {
        return null;
    }
}

class JFBConnectProfileData extends Registry
{
    var $fieldMap;

    /* bindData
     * Overridden function due to Joomla's checking of each variable to see if it's an associative array or not
     * We don't care, we want all arrays to be translated to a class
     */
    protected function bindData($parent, $data, $recursive = true, $allowNull = true)
    {
        // Ensure the input data is an array.
        if (is_object($data))
            $data = get_object_vars($data);
        else
            $data = (array)$data;

        foreach ($data as $k => $v)
        {
            if (is_array($v) || is_object($v))
            {
                $parent->$k = new stdClass;
                $this->bindData($parent->$k, $v);
            } else
                $parent->$k = $v;
        }
    }

    // All providers must support the retrieval of the following fields:
    // email, full_name, first_name, middle_name, last_name
    function get($path, $default = null)
    {
        if ($this->exists($path))
            return parent::get($path, $default);
        else
            return $default;
    }

    function setFieldMap($fieldMap)
    {
        $this->fieldMap = $fieldMap;
    }

    function getFieldWithUserState($field)
    {
        $app = Factory::getApplication();
        $val = $app->input->get($field, '', 'POST');
        // Check if there's a session variable from a previous POST, and use that
        if (empty($val))
        {
            $app = Factory::getApplication();
            $prevPost = $app->getUserState('com_jfbconnect.registration.data', array());
            if (array_key_exists($field, $prevPost))
                $val = $prevPost[$field];
        }

        if (empty($val))
            $val = $this->getFieldFromMapping($field);

        return $val;
    }

    function getFieldFromMapping($field)
    {
        if (!property_exists($this->fieldMap, $field))
            return "";

        $fieldName = $this->fieldMap->$field;
        return $this->get($fieldName, "");
    }

}