<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\Registry\Registry;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Object\CMSObject;
use Joomla\CMS\Log\Log;

require_once(__DIR__ . '/widget/login.php');

class JFBConnectProvider extends CMSObject
{
    var $name;
    var $appId;
    var $secretKey;
    var $networkSettings;

    var $auth;
    var $profile;
    var $client;
    var $usernamePrefix;
    var $loginPrefix;
    var $showLoginOnFrontEnd = true;

    protected $providerUserId;
    var $needsJavascript = false;
    var $widgetRendered = false;
    var $extraJS = array();

    // Multi-Dim array of settings => default for this specific provider
    var $customSettings = [];

    function __construct()
    {
        $this->configModel = JFBCFactory::config();
        $this->appId = $this->configModel->get($this->systemName . '_app_id');
        $this->secretKey = $this->configModel->get($this->systemName . '_secret_key');

        $networkSettings = $this->configModel->get($this->systemName . "_autotune_network_settings");
        $registry = new Registry();
        $registry->loadObject($networkSettings);
        $this->networkSettings = $registry;

        $profileClass = 'JFBConnectProfile' . ucfirst($this->systemName);
        $this->profile = new $profileClass($this);

        parent::__construct();
    }

    function __get($name)
    {
        switch ($name)
        {
            case 'systemName':
                return strtolower(str_replace(" ", "-", $this->name));
            default:
                return null;
        }
    }

    /* Override this to setup specialized parameters for the OAuth client.
        Necessary in cases where the scope is needed from the profile library.. which isn't fully initialized in the provider constructor.
    */
    public function setupAuthentication()
    {
    }

    /** Channels */
    public function channel($name, $options = null)
    {
        $channelName = 'JFBConnectProvider' . ucfirst($this->systemName) . 'Channel' . ucfirst($name);
        $options = $options == null ? new Registry() : $options;
        $channel = new $channelName($this, $options);
        return $channel;
    }

    public function getChannelsOutbound()
    {
        $channels = array();

        $pluginPath = JPATH_SITE . '/plugins/jfbconnect/provider_'.$this->systemName.'/provider/'.$this->systemName.'/channel/';
        if(Folder::exists($pluginPath))
        {
            $channels = $this->getChannelOutboundList($pluginPath);
        }
        return $channels;
    }

    private function getChannelOutboundList($path)
    {
        $channels = array();

        $names = Folder::files($path, '\.php$');
        foreach ($names as $name)
        {
            $name = str_replace(".php", "", $name);
            $channel = $this->channel($name);
            if ($channel->outbound)
                $channels[] = $channel;
        }
        return $channels;
    }

    /** End Channels */

    public function getUserAccessToken($jUserId)
    {
        return JFBCFactory::usermap()->getUserAccessToken($jUserId, $this->systemName);
    }

    var $initialRegistration = false;

    function setInitialRegistration()
    {
        $this->initialRegistration = true;
    }

    public function setSessionToken()
    {
        $token = $this->client->getToken();
        Factory::getApplication()->setUserState('com_jfbconnect.' . $this->systemName . '.token', json_encode($token));
    }

    public function onAfterInitialise()
    {

    }

    public function onGetProviderJavascriptInit()
    {

    }

    public function onBeforeRender()
    {

    }

    public function onAfterRender()
    {

    }

    /* Scope functions */
    public function hasScope($uid, $scope)
    {
        $currentScope = $this->getUserScope($uid);
        return in_array($scope, $currentScope);
    }

    public function getUserScope($uid)
    {
        return array();
    }

    private function checkRequiredScope($scope)
    {
        $neededScope = array();
        if (empty($scope)) // Don't do any checks if no scope is necessary
            return $neededScope;

        // get current scope for the user
        $token = $this->client->getToken();
        $currentScope = $this->getUserScope($this->getProviderUserId(), $token);
        foreach ($scope as $s)
        {
            if (!in_array($s, $currentScope))
                $neededScope[] = $s;
        }
        return $neededScope;
    }

    public function fetchNewScope($scope)
    {
        return true;
    }

    function logoutButton($params)
    {
        return "";
    }

    function avatar($params)
    {

    }

    // The new function of this is to iterate through the login providers and echo their buttons
    // This allows automatically adding the FB/Google buttons to apps that would normally add only FB
    public function getLoginButton($text = null)
    {
        $params = array('text' => $text);
        return JFBCFactory::getLoginButtons($params);
    }

    /*
     *  Introduced in 6.0, deprecated in 6.1.
     *  Please use JFBCFactory::loginButtons() instead.
     *  Right now, this will:
     *   - Show all login buttons if the provider isn't set
     *   - Show
     */
    function loginButton($params = null)
    {
        Log::add('loginButton method deprecated, use JFBCFactory::loginButtons() instead.', Log::WARNING, 'deprecated');
        if ($params == null && $this->systemName != 'facebook')
            return "";

        $loginWidget = JFBCFactory::widget(strtolower($this->systemName), 'login', $params);
        return $loginWidget->render();
    }


    function connectButton($params)
    {
        if (!isset($params['providers']))
            $params['providers'] = $this->systemName;
        $params['show_reconnect'] = 'true';
        return JFBCFactory::widget(strtolower($this->systemName), 'login', $params)->render();
    }

    public function getLoginButtonWithImage($params, $loginClass, $loginID)
    {
        if (is_array($params))
        {
            $reg = new Registry();
            $reg->loadArray($params);
            $params = $reg;
        }
        else if (!$params)
            $params = new Registry();

        $alignment = $this->getLoginButtonStyles($params);

        $image = $params->get('image', JFBCFactory::config()->get($this->systemName . '_login_button', 'icon_label.png'));
        JFBConnectUtilities::loadLanguage('com_jfbconnect');
        $buttonAltTitle = Text::sprintf('COM_JFBCONNECT_LOGIN_USING_SOCIAL', $this->name);
        return '<div class="social-login mr-2 ' . $this->systemName . ' ' . $loginClass . $alignment->class . '" '.$alignment->style.'>
        <a class="' . $loginID . '" href="javascript:void(0)" onclick="jfbc.login.provider(\'' . $this->systemName . '\');">
            <img src="' . Uri::root(true) . '/media/sourcecoast/images/provider/' . $this->systemName . '/' . $image . '" alt="' . $buttonAltTitle . '" title="' . $buttonAltTitle . '"/></a>
            </div>';
    }

    public function getLoginButtonStyles($params)
    {
        $alignment = $params->get('alignment', 'left');
        $alignmentClass = '';
        $alignmentStyle = '';
        if(JFBCFactory::config()->get('bootstrap_css') != '2' && JVERSION < 4.0)
        {
            if($alignment == 'center')
                $alignmentStyle = ' style="display:inline-block"';
            else
                $alignmentClass = " pull-" . $alignment;
        }

        $styles = new stdClass();
        $styles->style = $alignmentStyle;
        $styles->class = $alignmentClass;
        return $styles;
    }


    public function getLoginButtonInfo()
    {
        return new LoginButtonInfo($this->name, $this->loginPrefix);
    }

    public function buildListOfEasyTags()
    {
        $tagsToReplace = JFBCFactory::cache()->get('system.alleasytags.'.$this->systemName);
        if ($tagsToReplace === false)
        {
            $tagsToReplace = array();
            $pName = $this->systemName;
            $widgets = JFBCFactory::getAllWidgets($pName);
            foreach ($widgets as $widget)
            {
                if(is_array($widget->tagName))
                {
                    foreach($widget->tagName as $tagVariation)
                        $tagsToReplace[strtolower($tagVariation)] = array('provider' => $pName, 'widget' => $widget->systemName);
                }
                else
                    $tagsToReplace[strtolower($widget->tagName)] = array('provider' => $pName, 'widget' => $widget->systemName);
            }
            JFBCFactory::cache()->store($tagsToReplace, 'system.alleasytags.'.$this->systemName);
        }
        return $tagsToReplace;
    }

    /* getProviderUserId
    * The new way to get the userId from Facebook as of v5.1
    */
    function getProviderUserId()
    {
    }

    /* getMappedUserId
     * Return the mapped userId from Joomla
     */
    public function getMappedUserId()
    {
        $jUser = Factory::getUser();
        if ($jUser->guest)
            return null;

        $mappedUserId = Factory::getApplication()->getUserState('com_jfbconnect.' . $this->systemName . '.mapped_user_id', null);
        if (!$mappedUserId)
        {
            $mappedUserId = $this->getProviderUserId();
            if (JFBCFactory::usermap()->getJoomlaUserId($mappedUserId, $this->systemName) != $jUser->get('id'))
                $mappedUserId = null;

            Factory::getApplication()->setUserState('com_jfbconnect.' . $this->systemName . '.mapped_user_id', $mappedUserId);
        }
        return $mappedUserId;
    }

    /* onBeforeLogin
    * Any functional calls that the provider needs to make directly before the user is logged in (or automatically registered)
    */
    public function onBeforeLogin()
    {
        $userModel = JFBConnectModelUserMap::getUser(JFBCFactory::usermap()->getJoomlaUserId($this->getProviderUserId(), $this->systemName), $this->systemName);
        $requiredScope = $userModel->_data->params->get('required_scope', null);
        if ($requiredScope)
        {
            $session = Factory::getSession();
            $checked = $session->get($this->systemName . '.' . 'reqdscoperequested', false);
            if ($checked)
                return;

            // Only allow checking / re-authenticating for required scopes once to prevent redirect loops.
            $session->set($this->systemName . '.' . 'reqdscoperequested', true);

            $checkScope = array();
            foreach ($requiredScope as $key => $val) {
                $checkScope[] = $key;
            }
            // This may redirect the user back through the login flow to get the specific permissions for them.
            $neededScope = $this->checkRequiredScope($checkScope);
            if (count($neededScope))
                $this->fetchNewScope($neededScope);
        }
    }

    public function checkNewMapping()
    {
        $newMappingEnabled = Factory::getApplication()->getUserState('com_jfbconnect.' . $this->systemName . '.checkForNewMapping', false);
        if ($newMappingEnabled)
        {
            $providerId = $this->getProviderUserId();
            if ($providerId != null)
            {
                JFBCFactory::usermap()->map(Factory::getUser()->get('id'), $providerId, $this->systemName);
            }
        }

        Factory::getApplication()->setUserState('com_jfbconnect.' . $this->systemName . '.checkForNewMapping', null);
    }

    public function getHeadData()
    {
    }

    public function addJavascriptInit()
    {
        return $this->extraJS;
    }

    /**
     * Return any information about the specified scope gathered from Autotune
     * @param $scope
     */
    public function getScopeDetails($scope)
    {
        if ($this->networkSettings->exists('scopes.' . $scope))
            return $this->networkSettings->get('scopes.' . $scope);

        $response = new stdClass();
        $response->description = "No details available";
        return $response;
    }
}