<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\Folder;

class JFBConnectToolbar
{
    private $buttons = array();
    private $enabled;
    public function __construct()
    {
        $this->enabled = !Factory::getUser()->guest && JFBCFactory::config()->get('social_toolbar_enable') &&
            Factory::getUser()->authorise('jfbconnect.channels.post', 'com_jfbconnect') &&
            Factory::getApplication()->input->get('tmpl') != "component";
    }

    public function onAfterDispatch()
    {
        if ($this->enabled)
        {
            $path = JPATH_SITE . '/components/com_jfbconnect/libraries/toolbar/button/';
            $files = Folder::files($path, '\.php$');
            // Probably need to add some way to order these things..
            foreach ($files as $f)
            {
                $class = 'JFBConnectToolbarButton' . ucfirst(str_replace('.php', '', $f));
                if (class_exists($class))
                {
                    $obj = new $class();
                    $this->buttons[$obj->order] = $obj;
                }
            }
            ksort($this->buttons);

            foreach ($this->buttons as $b)
            {
                $b->afterDispatch();
            }

            $doc = Factory::getApplication()->getDocument();
            JFBCFactory::addStylesheet('jfbconnect.css');
            if (JFBCFactory::config()->get('facebook_display_errors'))
            {
                $doc->addStyleSheet('media/sourcecoast/css/jquery-ui/jquery-ui.css');
            }
            else
            {
                $doc->addStyleSheet('media/sourcecoast/css/jquery-ui/jquery-ui.min.css');
            }
        }
    }

    public function onAfterRender()
    {
        if ($this->enabled)
        {
            $buttons = array();
            $buttonHtml = array();
            $buttonJavascript = array();
            foreach ($this->buttons as $b)
            {
                $buttons[] = '<button name="' . $b->systemName . '">' . $b->displayName . '</button>';
                $buttonHtml[] = $b->html();
                $buttonJavascript[$b->systemName] = $b->javascript();
            }
            $html = '<div id="social-toolbar" class="ui-widget-header ui-corner-all" style="position:fixed; bottom: 10px">';
            $html .= implode($buttons);
            $html .= '</div>';
            $html .= implode($buttonHtml);

            $html .= $this->generateJavascript($buttonJavascript);

            $app = Factory::getApplication();
            $body = $app->getBody();
            $body = str_ireplace("</body>", $html . "</body>", $body);
            $app->setBody($body);
        }
    }

    private function generateJavascript($jsArray)
    {
        $code = '<script>jfbc.toolbar = {};';
        foreach ($jsArray as $name => $javascript)
        {
            $code .= $javascript;
            $code .= 'jfbc.toolbar.' . $name . ' = ' . $name . ';';

        }
        $code .= '</script>';
        return $code;
    }
}