<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\Registry\Registry;
use Joomla\CMS\Table\Table;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\HTML\HTMLHelper;

class JFBConnectToolbarButtonPost extends JFBConnectToolbarButton
{
    var $order = '30';
    var $displayName = "Create Post";
    var $systemName = "post";

    var $html = "";

    public function afterDispatch()
    {
        //Called in onAfterDispatch so language is not available yet
        JFBConnectUtilities::loadLanguage('com_jfbconnect');

        $this->html = HTMLHelper::_(
            'bootstrap.renderModal',
            'social-post-modal',
            array(
                'title' => Text::_('COM_JFBCONNECT_CHANNELS_POST_TITLE'),
                'animation' => false,
                'height' => '500px',
                'width' => '800px',
                'modalWidth' => '800px',
                'bodyHeight' => '500px',
                'closeButton' => true,
                'backdrop' => false,
                'footer' => '<button type="button" class="btn btn-secondary" data-dismiss="modal">Close</button>
                    <button type="button" class="btn btn-primary social-post-send-button">Post</button>'
            ),
            );
    }

    /*
     * When the submit button is pressed in the Create Post, post to social network
     * and replace the text in the modal with the success/fail response
     */
    public function html()
    {
        $query = 'option=com_jfbconnect&task=ajax.fetch&library=toolbar.button.post&subtask=getHtml&refresh=1';
        $script = '<script>' .
            "jfbcJQuery(document).on('click', '.social-post-send-button', function() {
            var form = jfbcJQuery('#social-post-form');
            var url = form . attr('action') + '&' + jfbc . token + '=1';
            var data = form . serializeArray();
            data . push({name: 'link', value: window.location.href});
            jfbcJQuery.post(url, data).done(function (ret)
                {                      
                    jfbcJQuery('#social-post-modal .modal-body').html(ret);                      
                }
            );
        });
        jfbcJQuery(document).on('shown.bs.modal', '#social-post-modal', function() {                       
            var query = '" . $query . "';
            jfbc.util.ajax(query).done(function( ret ) {
                jfbcJQuery('#social-post-modal .modal-body').html(ret);
            });
        });
        jfbcJQuery(document).on('keyup', '#social-post-message', function() {
            jfbcJQuery('.social-post-counter').each(function(i)
            {
                var el = jfbcJQuery(this);
                var max = el.data('max');
                var urlLength = el.data('urlLength');
                var remaining = max - urlLength - jfbcJQuery('#social-post-message').val().length;
                jfbcJQuery(this).text('(' + remaining + ' characters remaining)');
            });
        });                              
        </script>";

        return '<div id="social-post-container">' .
            $this->html .
            '</div>' . $script;
    }

    /*
     * Adds javascript to open and close the modal when the toolbar button is pressed
     */
    protected function generateJavascript()
    {

        $js = "display: function ()
                {
                    jfbcJQuery('#social-post-modal').modal('toggle');                              
                }";

        return $js;
    }

    public function ajaxGetHtml()
    {
        $channels = JFBCFactory::model('channel')->getChannels();
        $checkboxes = array();
        foreach ($channels as $c)
        {
            $options = new Registry();
            $options->loadObject($c);
            $data = JFBCFactory::provider($c->provider)->channel($c->type, $options);

            if ($data->postCharacterMax > 0)
                $charCounter = ' <span class="social-post-counter" data-max="' . $data->postCharacterMax . '" data-url-length="' . $data->urlLength . '"></span>';
            else
                $charCounter = "";

            if ($data->canPostToNetwork())
            {
                $checkboxes[] = '<input type="checkbox" name="cids[]" class="' . $c->provider . '-' . $c->type . '" value="' . $c->id . '" id="cid_' . $c->id . '" />
                <label class="channel-name" for="cid_' . $c->id . '">
                    <img src="' . Uri::root() . 'media/sourcecoast/images/provider/' . $c->provider . '/icon.png" />' .
                    $c->title .
                    '</label>' .
                    $charCounter;
            }
        }

        if (count($checkboxes) > 0)
        {
            ob_start();
            ?>
            <div id="social-post-popup">
                <form id="social-post-form" action="<?php echo Route::_('index.php?option=com_jfbconnect&task=ajax.fetch&library=toolbar.button.post&subtask=createPost');?>">
                    <p><?php echo Text::_('COM_JFBCONNECT_CHANNELS_POST_INSTRUCTION_LABEL');?></p>
                    <textarea id="social-post-message" name="message" style="width:90%" rows="5"></textarea>
                    <br/>
                    <input type="checkbox" id="channels_selectall" onclick="
                        jfbcJQuery('#social-post-popup :checkbox').each(function() {
                            if(this.id != 'channels_selectall')
                                this.checked = jfbcJQuery('#channels_selectall').is(':checked');
                        });"
                    />
                    <label for="channels_selectall"><?php echo Text::_('COM_JFBCONNECT_CHANNELS_POST_SELECT_ALL_LABEL');?></label>
                    <br/>
                    <?php
                    echo implode("<br/>", $checkboxes);
                    HTMLHelper::_('form.token')
                    ?>
                </form>
            </div>

            <?php
            $html = ob_get_clean();
        }
        else
        {
            $html = Text::_('COM_JFBCONNECT_CHANNELS_POST_ERROR_LABEL');
            $html .= "<style>.social-post-send-button {display:none;}</style>";
        }
        echo $html;
    }

    public function ajaxCreatePost()
    {
        if (Factory::getUser()->authorise('jfbconnect.channels.post', 'com_jfbconnect'))
        {
            $response = array();

            $input = Factory::getApplication()->input;
            $message = $input->post->getString('message');
            $link = $input->post->getString('link');

            $cids = $input->post->get('cids', array(), 'array');
            if (empty($cids))
            {
                $response[] = Text::_('COM_JFBCONNECT_CHANNELS_SELECT_CHANNEL_LABEL');
            }

            Table::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/tables/');
            $row = Table::getInstance('Channel', 'Table');
            foreach ($cids as $cid)
            {
                $row->load($cid);
                $options = new Registry();
                $options->loadObject($row->attribs);
                $channel = JFBCFactory::provider($row->provider)->channel($row->type, $options);

                $post = new Registry();
                $post->set('message', $message);
                $post->set('link', $link);

                try
                {
                    $return = $channel->post($post, AP_TYPE_MANUAL);
                } catch (Exception $e)
                {
                    $return = false;
                }

                if (!$return)
                    $response[] = Text::sprintf('COM_JFBCONNECT_CHANNELS_POST_FAILED_LABEL', $row->provider, $row->type);
                else
                    $response[] = $return;
            }
            //Hide post button
            $response[] = "<style>.social-post-send-button {display:none;}</style>";
            echo implode("<br/>", $response);
        }
        exit;
    }

}