<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\Registry\Registry;

abstract class JFBConnectWidget
{
    var $fields;
    var $provider;
    var $name;
    var $systemName;
    var $className;
    var $tagName;
    var $examples;

    function __construct($provider, $fields)
    {
        $this->provider = $provider;

        $this->fields = new Registry();

        if(is_object($fields) || is_array($fields))
        {
            $this->fields->loadObject($fields);
        }
    }

    public function getSystemName()
    {
        return $this->systemName;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getField($fieldName, $deprecatedName, $type, $defaultValue, $htmlName)
    {
        $value = $this->getParamValueEx($fieldName, $deprecatedName, $type, $defaultValue);

        if($value)
            return $this->getFieldAttribute($htmlName, $value);
        else
            return '';
    }

    public function getFieldAttribute($htmlName, $value)
    {
        return ' ' . $htmlName . '="' . $value . '"';
    }

    public function getParamValue($fieldName)
    {
        return $this->getParamValueEx($fieldName, null, null, '');
    }

    public function getParamValueEx($fieldName, $deprecatedName, $type, $defaultValue)
    {
        $value = $defaultValue;

        if($this->fields->exists($fieldName))
            $value = $this->fields->get($fieldName);
        else if($this->fields->exists($deprecatedName))
            $value = $this->fields->get($deprecatedName);

        if($value == '')
            $value = $defaultValue;

        if($type == 'boolean')
        {
            $value = strtolower($value);
            if($value == 'false' || $value == '0')
                $value = 'false';
            else if($value == 'true' || $value == '1')
                $value = 'true';
            else
                $value = $defaultValue;
        }

        return $value;
    }

    public function render()
    {
        $class[] = "sourcecoast";
        $class[] = "ns-bs5";
        $class[] = $this->systemName;

        if($this->provider)
        {
            JFBConnectUtilities::loadLanguage('plg_jfbconnect_provider_' . $this->provider->systemName, JPATH_ADMINISTRATOR);

            $class[] = $this->provider->systemName;

            if($this->provider->systemName != 'pinterest') //Don't add pinit unless there's an image
                $this->provider->needsJavascript = true;
        }

        $tag = $this->getTagHtml();

        if($tag)
        {
            if($this->provider)
                $this->provider->widgetRendered = true;

            if($this->className)
                $class[] = $this->className;

            $classString = implode(' ', $class);
            $tag = '<div class="'.$classString.'">' . $tag . '</div>';
        }

        return $tag;
    }

    public function getHeadData() {}
    protected function getTagHtml() {}
}