<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Date\Date;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Session\Session;
use Joomla\CMS\User\User;
use Joomla\CMS\User\UserHelper;
use Joomla\CMS\Form\Form;
use Joomla\CMS\Application\ApplicationHelper;

jimport('joomla.application.component.modelform');
jimport('joomla.event.dispatcher');
jimport('joomla.plugin.helper');

class JFBConnectModelLoginRegister extends BaseDatabaseModel
{
    function createNewUser($provider)
    {
        $provider->setInitialRegistration();

        $app = Factory::getApplication();

        if (!Session::checkToken())
        {
            JFBCFactory::log(Text::_('COM_JFBCONNECT_SESSION_TIMEOUT'), 'error');
            return false;
        }
        // Save the whole POST data since we may need some of the fields after a redirection (on user creation failure)
        $this->getLoginPostData();

        $providerUserId = $provider->getProviderUserId();

        // User Registration Controller code
        BaseDatabaseModel::addIncludePath(JPATH_SITE . '/components/com_users/models');
        jimport('joomla.form.form');
        if(JVERSION < 4.0)
        {
            Form::addFormPath(JPATH_SITE . '/components/com_users/models/forms');
            Form::addFieldPath(JPATH_SITE . '/components/com_users/models/fields');
        }
        else
        {
            Form::addFormPath(JPATH_SITE . '/components/com_users/forms');
        }
        $language = Factory::getApplication()->getLanguage();
        $language->load('com_users');

        $userModel = BaseDatabaseModel::getInstance('Registration', 'UsersModel');
        $requestData = $app->input->get('jform', array(), 'post', 'array');

        // Validate the posted data.
        $form = $userModel->getForm();
        if (!$form)
        {
            throw new RuntimeException($userModel->getError(), 500);
        }
        $formData = $userModel->validate($form, $requestData);

        // Check for validation errors.
        if ($formData === false)
        {
            // Get the validation messages.
            $errors = $userModel->getErrors();

            // Push up to three validation messages out to the user.
            for ($i = 0, $n = count($errors); $i < $n && $i < 3; $i++)
            {
                if($errors[$i] instanceof Exception)
                {
                    JFBCFactory::log($errors[$i]->getMessage(), 'warning');
                }
                else
                {
                    JFBCFactory::log($errors[$i], 'warning');
                }
            }

            // Save the data in the session.
            $app->setUserState('com_users.registration.data', $requestData);

            return false;
        }

        if(!$this->isValidRegistrationDomain($formData['email1']))
        {
            JFBCFactory::log(Text::_('COM_JFBCONNECT_MSG_REG_DOMAIN'));
            return false;
        }

        // Register the user and return the newly created user ID
        $useractivation = $this->getActivationMode();
        $jUser = $this->register($formData, $useractivation);

        // Check for errors.
        if ($jUser === false)
        {
            // Save the data in the session.
            $app->setUserState('com_users.registration.data', $requestData);
            // Redirect back to the edit screen.
            JFBCFactory::log(Text::sprintf('COM_USERS_REGISTRATION_SAVE_FAILED', $this->getError()), 'warning');
            return false;
        }

        $lang = $app->input->get(ApplicationHelper::getHash('language'), '', 'COOKIE');
        $jUser->setParam('language', $lang);
        $jUser->save();

        // Flush the data from the session.
        $app->setUserState('com_users.registration.data', null);
        $app->setUserState('com_jfbconnect.registration.data', null);

        #Send the new user confirmation email and admin notify emails
        $this->_newUserPassword = $formData['password1'];

        $this->onAfterRegister($provider, $jUser);

        if (JFBCFactory::usermap()->map($jUser->id, $providerUserId, $provider->systemName, $provider->client->getToken()))
        {
            JFBCFactory::log(Text::sprintf('COM_JFBCONNECT_MAP_USER_SUCCESS', $provider->name));
            return true;
        }
        else
        {
            JFBCFactory::log(Text::sprintf('COM_JFBCONNECT_MAP_USER_FAIL', $provider->name));
            return false;
        }
    }

    public function getLoginRedirect($provider = null)
    {
        if ((($provider !== null && $provider->initialRegistration) || Factory::getApplication()->getUserState('com_jfbconnect.registration.alternateflow', false))
            && JFBCFactory::config()->get('facebook_new_user_redirect') != ''
        )
        {
            $itemId = JFBCFactory::config()->get('facebook_new_user_redirect', '0');
            $redirect = JFBConnectUtilities::getLinkFromMenuItem($itemId, false);
        }
        else if (JFBCFactory::config()->get('facebook_login_redirect') != '')
        {
            $itemId = JFBCFactory::config()->get('facebook_login_redirect', '0');
            $redirect = JFBConnectUtilities::getLinkFromMenuItem($itemId, false);
        }
        else
        {
            // Get whatever has been saved in the com_jfbconnect.login.redirect state variable.
            $redirect = Factory::getApplication()->getUserState('com_jfbconnect.login.return', 'index.php');
            //Factory::getApplication()->setUserState('com_jfbconnect.login.return', null);
        }

        return $redirect;
    }

    /**
     * Method to save the form data. Primary implementation from Joomla 1.6 com_users/models/registration.php
     *
     * @param    array        The form data.
     * @return    mixed        The user id on success, false on failure.
     * @since    1.6
     */
    private function register($temp, $useractivation)
    {
        $app = Factory::getApplication();

        $tcEnabled = PluginHelper::isEnabled('user', 'terms');
        if($tcEnabled)
        {
            $requestData = $app->input->get('jform', array(), 'post', 'array');
            if (empty($requestData['terms']['terms']))
            {
                $this->setError(Text::_('PLG_USER_TERMS_FIELD_ERROR'));
                return false;
            }
        }

        $privacyEnabled = PluginHelper::isEnabled('system', 'privacyconsent');
        if($privacyEnabled)
        {
            $requestData = $app->input->get('jform', array(), 'post', 'array');
            if (empty($requestData['privacyconsent']['privacy']))
            {
                $this->setError(Text::_('PLG_SYSTEM_PRIVACYCONSENT_FIELD_ERROR'));
                return false;
            }
        }

        // Initialise the table with User.
        BaseDatabaseModel::addIncludePath(JPATH_SITE . '/components/com_users/models');
        $userModel = BaseDatabaseModel::getInstance('Registration', 'UsersModel');

        $user = new User;
        $data = (array)$userModel->getData();

        // Merge in the registration data.
        foreach ($temp as $k => $v)
        {
            $data[$k] = $v;
        }

        // Prepare the data for the user object.
        $data['email'] = $data['email1'];
        $data['password'] = $data['password1'];

        // Check if the user needs to activate their account.
        if (($useractivation == 1) || ($useractivation == 2))
        {
            jimport('joomla.user.helper');
            $password_length = JFBCFactory::config()->get('random_password_length');
            $data['activation'] = ApplicationHelper::getHash(UserHelper::genRandomPassword($password_length));
            $data['block'] = 1;
        }

        // Bind the data.
        if (!$user->bind($data))
        {
            $this->setError(Text::sprintf('COM_USERS_REGISTRATION_BIND_FAILED', $user->getError()));
            return false;
        }

        //set user groups
        $user->set('groups', JFBCFactory::config()->get('registration_usegroups'));

        // Load the users plugin group.
        PluginHelper::importPlugin('user');

        // Store the data.
        if (!$user->save())
        {
            $this->setError(Text::sprintf('COM_USERS_REGISTRATION_SAVE_FAILED', $user->getError()));
            return false;
        }

        return $user;
    }

    private function getLoginPostData()
    {
        $input = Factory::getApplication()->input;
        $postData = $input->post->getArray();

        if (isset($postData['jform']['password1']))
            $postData['jform']['password1'] = '';
        if (isset($postData['jform']['password2']))
            $postData['jform']['password2'] = '';

        $app = Factory::getApplication();
        $app->setUserState('com_jfbconnect.registration.data', $postData);
    }

    function autoCreateUser($providerUserId, $provider)
    {
        $app = Factory::getApplication();

        $provider->setInitialRegistration();

        $profile = $provider->profile->fetchProfile($providerUserId, array('first_name', 'last_name', 'email', 'full_name'));
        if ($profile == null || $profile->get('email') == null) # not enough information returned to auto-create account
            return false;

        $newEmail = $profile->get('email');
        $fullname = $profile->get('full_name');

        if(!$this->isValidRegistrationDomain($newEmail))
        {
            JFBCFactory::log(Text::_('COM_JFBCONNECT_MSG_REG_DOMAIN'));
            Factory::getApplication()->redirect('index.php');
        }

        $user['fullname'] = $fullname;
        $user['email'] = $newEmail;

        // Create random password for FB User Only, but save so we can email to the user on account creation
        if (JFBCFactory::config()->get('generate_random_password'))
        {
            $password_length = JFBCFactory::config()->get('random_password_length');
            $this->_newUserPassword = UserHelper::genRandomPassword($password_length);
            $user['password_clear'] = $this->_newUserPassword;
            $user['password'] = UserHelper::hashPassword($this->_newUserPassword);
        }
        else
        {
            $user['password_clear'] = "";
            $this->_newUserPassword = '';
        }

        $lang = $app->input->get(ApplicationHelper::getHash('language'), '', 'COOKIE');
        $user['language'] = $lang;

        $usernamePrefixFormat = JFBCFactory::config()->get('auto_username_format');
        $username = JFBConnectUtilities::getAutoUsername($profile->get('first_name'), $profile->get('last_name'), $profile->get('email'), $provider->usernamePrefix, $providerUserId, $usernamePrefixFormat);
        $user['username'] = $username;

        $useractivation = $this->getActivationMode();
        $jUser = $this->getBlankUser($user, $useractivation);

        if ($jUser && $jUser->get('id', null)) // If it's not set, there was an error
        {
            $this->onAfterRegister($provider, $jUser);
            //JFBConnectUtilities::clearJFBCNewMappingEnabled();

            if (JFBCFactory::usermap()->map($jUser->get('id'), $providerUserId, $provider->systemName, $provider->client->getToken()))
            {
                JFBCFactory::log(Text::sprintf('COM_JFBCONNECT_MAP_USER_SUCCESS', $provider->name));
                return true;
            }
            else
                JFBCFactory::log(Text::sprintf('COM_JFBCONNECT_MAP_USER_FAIL', $provider->name));
        }
        return false; // User creation failed for some reason
    }

    private function onAfterRegister($provider, $jUser)
    {
        $this->activateUser();
        $this->sendNewUserEmails($jUser, $provider->systemName);

        # New user, set their new user status and trigger the OnRegister event
        $args = array($provider->systemName, $jUser, $provider->getProviderUserId());
        Factory::getApplication()->triggerEvent('onSocialProfilesOnRegister', $args);
    }

    function &getBlankUser($user, $activationMode)
    {
        jimport('joomla.application.component.helper');
        $config = ComponentHelper::getParams('com_users');

        $instance = User::getInstance();

        // Default to Registered.
        //$defaultUserGroup = $config->get('new_usertype', 2);
        $instance->set('usertype', 'deprecated');
        //$instance->set('groups', array($defaultUserGroup));
        //set user groups
        $instance->set('groups', JFBCFactory::config()->get('registration_usegroups'));

        $instance->set('id', 0);
        $instance->set('name', $user['fullname']);
        $instance->set('username', $user['username']);
        if (array_key_exists('password', $user) && $user['password'] != "")
            $instance->set('password', $user['password']);

        if (array_key_exists('password_clear', $user))
            $instance->set('password_clear', $user['password_clear']);

        $instance->set('email', $user['email']); // Result should contain an email (check)
        $instance->setParam('language', $user['language']);

        if ($activationMode != 0)
        {
            jimport('joomla.user.helper');
            $password_length = JFBCFactory::config()->get('random_password_length');
            $instance->set('activation', ApplicationHelper::getHash(UserHelper::genRandomPassword($password_length)));
            $instance->set('block', 1);
        }
        else
            $instance->set('block', 0);

        if (!$instance->save())
        {
            JFBCFactory::log($instance->getError(), 'error');
            $instance = null;
        }

        return $instance;
    }

    /** Activation and new user email functions *****/
    private function activateUser()
    {
        $useractivation = $this->getActivationMode();
        $language = Factory::getApplication()->getLanguage();

        # Send out the new registration email
        // figure out activation
        $language->load('com_users');
        if ($useractivation == 2)
            JFBCFactory::log(Text::_('COM_USERS_REGISTRATION_COMPLETE_VERIFY'));
        else if ($useractivation == 1)
            JFBCFactory::log(Text::_('COM_USERS_REGISTRATION_COMPLETE_ACTIVATE'));

        if ($useractivation == 0)
            return true;
        else
            return false;
    }

    private function getActivationMode()
    {
        if (JFBCFactory::config()->get('joomla_skip_newuser_activation'))
        {
            return 0;
        }
        else
        {
            $params = ComponentHelper::getParams('com_users');
            $useractivation = $params->get('useractivation');
            return $useractivation;
        }
    }

    private function isValidRegistrationDomain($email)
    {
        $validDomain = JFBCFactory::config()->get('registration_domain');
        if (!empty($validDomain))
        {
            if(strpos($validDomain, '@') === false) //add @ if not found
                $validDomain = '@'.$validDomain;

            if (strpos($email, $validDomain) === false)
                return false;
        }
        return true;
    }

    private function sendNewUserEmails(&$user, $providerName)
    {
        $config = Factory::getConfig();
        if($config->get('mailonline') == false)
            return;

        $app = Factory::getApplication();
        $sendEmail = true;
        $profileEmails = $app->triggerEvent('onSocialProfilesSendsNewUserEmails');
        foreach ($profileEmails as $pe)
        {
            if ($pe)
                $sendEmail = false;
        }
        if (!$sendEmail)
            return;

        $useractivation = $this->getActivationMode();

        $newEmail = $user->get('email');
        if (JFBConnectUtilities::endswith($newEmail, "@unknown.com"))
            return;

        // Compile the notification mail values.
        $params = ComponentHelper::getParams('com_users');
        $language = Factory::getApplication()->getLanguage();
        $language->load('com_users');
        JFBConnectUtilities::loadLanguage('com_jfbconnect');

        $data = $user->getProperties();
        $data['fromname'] = $config->get('fromname');
        $data['mailfrom'] = $config->get('mailfrom');
        $data['sitename'] = $config->get('sitename');
        $data['siteurl'] = Uri::base();

        $uri = Uri::getInstance();
        $base = $uri->toString(array('scheme', 'user', 'pass', 'host', 'port'));
        $data['activate'] = $base . Route::_('index.php?option=com_users&task=registration.activate&token=' . $data['activation'], false);

        if (JFBCFactory::config()->get('registration_send_new_user_email') || $useractivation > 0)
        {
            $emailSubject = Text::sprintf(
                'COM_USERS_EMAIL_ACCOUNT_DETAILS',
                $data['name'],
                $data['sitename']
            );

            // Handle account activation/confirmation emails.
            if ($useractivation == 2)
            {
                // Set the link to confirm the user email.
                if ($this->_newUserPassword == '')
                {
                    $emailBody = Text::sprintf('COM_JFBCONNECT_EMAIL_REGISTERED_WITH_ADMIN_ACTIVATION_BODY_NOPASSWORD',
                        $data['name'],
                        $data['sitename'],
                        $data['siteurl'] . 'index.php?option=com_users&task=registration.activate&token=' . $data['activation'],
                        $data['siteurl'],
                        $providerName
                    );
                }
                else
                {
                    $emailBody = Text::sprintf('COM_JFBCONNECT_EMAIL_REGISTERED_WITH_ADMIN_ACTIVATION_BODY',
                        $data['name'],
                        $data['sitename'],
                        $data['siteurl'] . 'index.php?option=com_users&task=registration.activate&token=' . $data['activation'],
                        $data['siteurl'],
                        $providerName,
                        $data['username'],
                        $this->_newUserPassword
                    );
                }
            }
            else if ($useractivation == 1)
            {
                // Set the link to activate the user account.
                if ($this->_newUserPassword == '')
                {
                    $emailBody = Text::sprintf(
                        'COM_JFBCONNECT_EMAIL_REGISTERED_WITH_ACTIVATION_BODY_NOPASSWORD',
                        $data['name'],
                        $data['sitename'],
                        $data['siteurl'] . 'index.php?option=com_users&task=registration.activate&token=' . $data['activation'],
                        $data['siteurl'],
                        $providerName
                    );
                }
                else
                {
                    $emailBody = Text::sprintf(
                        'COM_JFBCONNECT_EMAIL_REGISTERED_WITH_ACTIVATION_BODY',
                        $data['name'],
                        $data['sitename'],
                        $data['siteurl'] . 'index.php?option=com_users&task=registration.activate&token=' . $data['activation'],
                        $data['siteurl'],
                        $providerName,
                        $data['username'],
                        $this->_newUserPassword
                    );
                }
            }
            else
            {
                $emailSubject = Text::sprintf(
                    'COM_USERS_EMAIL_ACCOUNT_DETAILS',
                    $data['name'],
                    $data['sitename']
                );

                if ($this->_newUserPassword == '')
                    $emailBody = Text::sprintf('COM_JFBCONNECT_EMAIL_REGISTERED_BODY_NOPASSWORD', $data['name'], $data['sitename'], $data['siteurl'], $providerName);
                else
                    $emailBody = Text::sprintf('COM_JFBCONNECT_EMAIL_REGISTERED_BODY', $data['name'], $data['sitename'], $data['siteurl'], $providerName, $data['username'], $this->_newUserPassword);
            }

            // Send the registration email.
            $return = Factory::getMailer()->sendMail($data['mailfrom'], $data['fromname'], $data['email'], $emailSubject, $emailBody);
        }
        else
        {
            $return = true;
        }

        if (($useractivation < 2) && ($params->get('mail_to_admin') == 1))
        {
            $emailSubject = Text::sprintf(
                'COM_USERS_EMAIL_ACCOUNT_DETAILS',
                $data['name'],
                $data['sitename']
            );

            $emailBodyAdmin = Text::sprintf(
                'COM_USERS_EMAIL_REGISTERED_NOTIFICATION_TO_ADMIN_BODY',
                $data['name'],
                $data['username'],
                $data['siteurl']
            );

            // get all admin users
            $query = 'SELECT id, name, email, sendEmail' .
                ' FROM #__users' .
                ' WHERE sendEmail=1';

            $db = Factory::getDBO();
            $db->setQuery($query);
            $rows = $db->loadObjectList();

            // Send mail to all superadministrators id
            foreach ($rows as $row)
            {
                $usercreator = Factory::getUser($row->id);
                if ($usercreator->authorise('core.create', 'com_users'))
                {
                    $return = Factory::getMailer()->sendMail($data['mailfrom'], $data['fromname'], $row->email, $emailSubject, $emailBodyAdmin);

                    // Check for an error.
                    if ($return !== true)
                    {
                        $this->setError(Text::_('COM_USERS_REGISTRATION_ACTIVATION_NOTIFY_SEND_MAIL_FAILED'));
                        return false;
                    }
                }
            }
        }

        // Check for an error.
        if ($return !== true)
        {
            $this->setError(Text::_('COM_USERS_REGISTRATION_SEND_MAIL_FAILED'));
            // Send a system message to administrators receiving system mails
            $db = Factory::getDbo();
            $query = $db->getQuery(true);
            $query->clear()
                //->select($db->quoteName(array('name', 'email', 'sendEmail', 'id'))) // possible joomla bug as it select the name instead of the id
                ->select($db->quoteName('id'))
                ->from($db->quoteName('#__users'))
                ->where($db->quoteName('block') . ' = ' . (int) 0)
                ->where($db->quoteName('sendEmail') . ' = ' . (int) 1);
            $db->setQuery($query);
            try
            {
                $sendEmail = $db->loadColumn();
            }
            catch (RuntimeException $e)
            {
                $this->setError(Text::sprintf('COM_USERS_DATABASE_ERROR', $e->getMessage()), 500);
                return false;
            }

            if (count($sendEmail) > 0)
            {
                $jdate = new Date;
                // Build the query to add the messages
                foreach ($sendEmail as $userid)
                {
                    $values = array(
                        $db->quote($userid),
                        $db->quote($userid),
                        $db->quote($jdate->toSql()),
                        $db->quote(Text::_('COM_USERS_MAIL_SEND_FAILURE_SUBJECT')),
                        $db->quote(Text::sprintf('COM_USERS_MAIL_SEND_FAILURE_BODY', $return, $data['username']))
                    );
                    $query->clear()
                        ->insert($db->quoteName('#__messages'))
                        ->columns($db->quoteName(array('user_id_from', 'user_id_to', 'date_time', 'subject', 'message')))
                        ->values(implode(',', $values));
                    $db->setQuery($query);
                    try
                    {
                        $db->execute();
                    }
                    catch (RuntimeException $e)
                    {
                        $this->setError(Text::sprintf('COM_USERS_DATABASE_ERROR', $e->getMessage()), 500);
                        return false;
                    }
                }
            }
            return false;
        }
    }
}

?>
