<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Factory;
use Joomla\Registry\Registry;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Form\Form;
use Joomla\CMS\Form\FormHelper;
use Joomla\CMS\Object\CMSObject;

jimport('joomla.application.component.model');
jimport('joomla.filesystem.file');

class JFBConnectModelOpenGraphObject extends BaseDatabaseModel
{
    var $objectId;
    var $userId;
    static $instances;

    public function __construct($config = array())
    {
        $user = Factory::getUser();
        $this->userId = $user->get('id');
        $this->definedObjects = array();

        if (self::$instances === null)
            self::$instances = array();

        parent::__construct($config);
    }

    public function getObject($id, $data = null)
    {
        if (!isset(self::$instances[$id]))
        {
            if (!$data)
            {
                $query = "SELECT * FROM #__opengraph_object WHERE " .
                        " `id` = " . $this->_db->quote($id);
                $this->_db->setQuery($query);
                $data = $this->_db->loadObject();
            }

            $object = new ogObject();
            $object->setProperties($data);
            self::$instances[$id] = $object;
        }

        return self::$instances[$id];
    }

    // When the plugin is running, get the objects of a specific type for the current page.
    // Only returns published objects
    public function getPluginObjects($plugin, $name)
    {
        $this->_db->setQuery("SELECT * FROM #__opengraph_object WHERE " .
                " plugin = " . $this->_db->quote($plugin) .
                " AND system_name = " . $this->_db->quote($name) .
                " AND published = 1");
        $rows = $this->_db->loadObjectList();
        $objects = array();
        foreach ($rows as $row)
            $objects[] = $this->getObject($row->id, $row);

        return $objects;
    }
}

// Should make these objects derive from the same class to share some of the common functionality.
// getNamespace, setProperties, etc.
class ogObject extends CMSObject
{
    var $id;
    var $plugin;
    var $system_name;
    var $display_name;
    var $type;
    var $published;
    var $params;
    var $created;
    var $modified;
    var $associatedActions;

    public function setProperties($data)
    {
        parent::setProperties($data);

        $this->params = new Registry($this->params);
    }

    public function loadDefaultObject($plugin, $name)
    {
        $this->id = 0;
        $this->plugin = $plugin;
        $this->system_name = $name;
        $this->display_name = $name;
        $this->type = "";
        $this->extension_key = "";
        $this->published = 0;
        $this->params = new Registry();
        $this->created = "--";
        $this->modified = "--";

        $xml = JPATH_SITE . '/plugins/opengraph/' . $plugin . '/objects/' . $name . '.xml';
        if (File::exists($xml))
        {
            FormHelper::addFieldPath(JPATH_SITE . '/plugins/opengraph/' . $plugin . '/objects');
            $form = Form::getInstance('opengraph.' . $plugin . '.' . $name, $xml);

            foreach ($form->getFieldset() as $field)
            {
                if ($field->value)
                {
                    $this->params->set($field->fieldname, $field->value);
                }
            }
        }
    }

    public function getObjectPath()
    {
        $objectPath = $this->type;
        return $objectPath;
    }

    private function getNamespace()
    {
        $appConfig = JFBCFactory::config()->get('autotune_app_config');
        $namespace = $appConfig['namespace'];
        return $namespace;
    }

    public function getAssociatedActions()
    {
        if (!$this->associatedActions)
        {
            $this->associatedActions = array();

            $db = Factory::getDbo();
            $db->setQuery('SELECT a.* FROM #__opengraph_action_object ao ' .
                    ' INNER JOIN #__opengraph_action a ON ao.action_id = a.id ' .
                    ' WHERE ao.object_id = ' . $this->id .
                    ' AND a.published = 1');
            $actions = $db->loadObjectList();
            if (count($actions) > 0)
            {
                $actionModel = BaseDatabaseModel::getInstance('opengraphaction', 'JFBConnectModel');
                foreach ($actions as $action)
                    $this->associatedActions[] = $actionModel->getAction($action->id, $action);
            }
        }
        return $this->associatedActions;
    }
}