<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Session\Session;
use Joomla\CMS\Factory;
use Joomla\CMS\Router\Route;

if(JVERSION < 4.0 && JFBCFactory::config()->get('use_font_awesome'))
{
    Factory::getDocument()->addStyleSheet(Uri::base(true) . '/media/sourcecoast/css/fontawesome/css/font-awesome.min.css');

    $mappedIcon = '<span class="fas fa fa-check fa-2x" aria-hidden="true">&nbsp;</span>';
    $unmappedIcon = '<span class="fas fa fa-ban fa-2x" aria-hidden="true">&nbsp;</span>';
    $unlinkIcon = '<span class="fas fa fa-unlink fa-2x" aria-hidden="true"></span>';
}
else
{
    $mappedIcon = '<img class="scicon-y" src="'.Uri::base(true).'/media/sourcecoast/images/icon-y.png" alt="'.Text::_('COM_JFBCONNECT_ACCOUNT_LINKED_ACCOUNTS_LINKED_LABEL').'" />';
    $unmappedIcon = '<img class="scicon-x" src="'.Uri::base(true).'/media/sourcecoast/images/icon-x.png" alt="'.Text::_('COM_JFBCONNECT_ACCOUNT_LINKED_ACCOUNTS_NOT_LINKED_LABEL').'"/>';
    $unlinkIcon = Text::_('COM_JFBCONNECT_ACCOUNT_LINKED_ACCOUNTS_UNLINKED_LABEL');
}
?>

<h1><?php echo Text::_('COM_JFBCONNECT_ACCOUNT_LINKED_ACCOUNTS_TITLE'); ?></h1>
<p><?php echo Text::_('COM_JFBCONNECT_ACCOUNT_LINKED_ACCOUNTS_DESC');?></p>
<div class="sourcecoast accounts ns-bs5">
    <table class="table">
        <?php

        foreach($this->providerData as $providerName=>$providerObj)
        {
            $imageName = strtolower($providerName).'_login_button';
            $providerImage = JFBCFactory::config()->get($imageName, 'icon.png');
            ?>
            <tr>
                <td>
                    <img src="<?php echo Uri::root() . '/media/sourcecoast/images/provider/' . $providerObj->systemName . '/' . $providerImage;?>" alt="<?php echo $providerName;?>"/>
                </td>
                <?php
                if($providerObj->isMapped)
                {?>
                    <td>
                        <?php echo $mappedIcon;?>
                    </td>
                    <td>
                        <?php echo Text::sprintf('COM_JFBCONNECT_ACCOUNT_LINKED_ACCOUNTS_ACCOUNT_MAPPED_LABEL', $providerName);?>
                    </td>
                    <td>
                        <?php
                        $unlink = Route::_('index.php?option=com_jfbconnect&task=account.unlink&provider='.$providerObj->provider->systemName.'&'. Session::getFormToken() .'=1');
                        ?>
                        <a href="<?php echo $unlink;?>">
                            <?php echo $unlinkIcon;?>
                        </a>
                    </td>
                <?php }
                else
                {?>
                    <td>
                        <?php echo $unmappedIcon;?>
                    </td>
                    <td>
                        <a href="javascript:void(0)" onclick="jfbc.login.provider('<?php echo $providerObj->systemName;?>')">
                            <?php echo Text::sprintf('COM_JFBCONNECT_ACCOUNT_LINKED_ACCOUNTS_CONNECT_PROFILE_LABEL', $providerName);?>
                        </a>
                    </td>
                    <td></td>
                <?php }
                ?>
            </tr>
            <?php
        }
        ?>
    </table>
</div>