<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\HTML\HTMLHelper;

HTMLHelper::_('behavior.keepalive');

if ($this->defaultValidationNeeded)
{
    HTMLHelper::_('behavior.formvalidator');
}
$colClass = (JFBCFactory::config()->get('bootstrap_css') == "1") ? 'span' : 'col-md-';
$profileFields = $this->profileFields;
?>

<div id="jfbc_loginregister" class="row-fluid jfbc_<?php echo strtolower($this->providerName); ?>">

    <h1><?php echo Text::_('COM_JFBCONNECT_WELCOME') . ' ' . $this->profile->get('first_name') ?>!</h1>

    <?php
    $existingUserSpan = $colClass.'12';
    $newUserSpan = $colClass.'12';

    if ($this->displayType == 'register-only')
        echo '<p>'.Text::sprintf('COM_JFBCONNECT_THANKS_FOR_SIGNING_IN_REGISTER_ONLY', ucfirst($this->providerName)).'</p>';
    else
    {
        echo '<p>'.Text::sprintf('COM_JFBCONNECT_THANKS_FOR_SIGNING_IN', ucfirst($this->providerName)).'</p>';

        if($this->displayType == 'horizontal')
        {
            $existingUserSpan = $colClass.'6';
            $newUserSpan = $colClass.'5';
        }
    }
    ?>

    <div id="jfbc_loginregister_userinfo" class="<?php echo $this->displayType; ?> row-fluid">
        <div id="jfbc_loginregister_existinguser" class="<?php echo $existingUserSpan;?>">
            <form action="" method="post" name="form" autocomplete="on">
                <fieldset>
                    <legend><?php echo Text::_('COM_JFBCONNECT_EXISTING_USER_REGISTRATION') ?></legend>
                    <p><?php echo Text::sprintf('COM_JFBCONNECT_EXISTING_USER_INSTRUCTIONS', ucfirst($this->providerName)); ?></p>
                    <dl>
                        <dt><label><?php echo Text::_('COM_JFBCONNECT_USERNAME') ?> </label></dt>
                        <dd><input type="text" class="input-medium" name="username" value="" size="20" autocomplete="username"/></dd>
                        <dt><label><?php echo Text::_('COM_JFBCONNECT_PASSWORD') ?> </label></dt>
                        <dd><input type="password" class="input-medium" name="password" value="" size="20"/></dd>
                        <dt></dt>
                        <dd><input type="submit" class="btn <?php echo $this->configModel->get('registration_loginbutton_class', 'btn-primary');?>" value="<?php echo Text::_('COM_JFBCONNECT_LOGIN'); ?>" />
                        </dd>
                    </dl>
                </fieldset>

                <input type="hidden" name="option" value="com_jfbconnect" />
                <input type="hidden" name="task" value="loginregister.loginMap" />
                <input type="hidden" name="provider" value="<?php echo $this->providerName; ?>" />
                <?php echo HTMLHelper::_('form.token');
                echo JFBCFactory::getLoginButtons($this->altParams);
                ?>
            </form>
            <div style="clear:both"></div>
        </div>
        <div id="jfbc_loginregister_newuser" class="<?php echo $newUserSpan;?>">
            <form action="" method="post" id="adminForm" class="form-validate" autocomplete="on">
                <?php
                $addedDesc = false;
                foreach ($this->form->getFieldsets() as $fieldset): // Iterate through the form fieldsets and display each one.?>
                    <?php $fields = $this->form->getFieldset($fieldset->name); ?>
                    <?php if (count($fields))
                    {
                        ?>
                        <fieldset>
                            <?php if (isset($fieldset->label))
                            { // If the fieldset has a label set, display it as the legend.
                                ?>
                                <legend><?php echo Text::_($fieldset->label); ?></legend>
                                <?php if(!$addedDesc) {
                                    echo '<p>'.Text::sprintf('COM_JFBCONNECT_NEW_USER_INSTRUCTIONS', ucfirst($this->providerName)).'</p>';
                                    $addedDesc = true;
                                }?>
                            <?php } ?>
                            <dl>
                                <?php foreach ($fields as $field): // Iterate through the fields in the set and display them.?>
                                    <?php if ($field->hidden)
                                    { // If the field is hidden, just display the input.
                                        ?>
                                        <?php echo $field->input; ?>
                                    <?php
                                    }
                                    else
                                    {
                                        ?>
                                        <dt>
                                            <?php echo $field->label; ?>
                                            <?php if (!$field->required && (!$field->type == "spacer"))
                                            {
                                                ?>
                                                <span class="optional"><?php echo Text::_('COM_USERS_OPTIONAL'); ?></span>
                                            <?php } ?>
                                        </dt>
                                        <dd><?php echo $field->input; ?></dd>
                                    <?php } ?>
                                <?php endforeach; ?>
                            </dl>
                        </fieldset>
                    <?php } ?>
                <?php endforeach; ?>
                <?php if (!$addedDesc) {
                    echo '<p>'.Text::sprintf('COM_JFBCONNECT_NEW_USER_INSTRUCTIONS', ucfirst($this->providerName)).'</p>';
                    
                }?>
                <div class="profile-fields">
                    <?php
                    foreach ($profileFields as $profileForm)
                        echo $profileForm;
                    ?>
                </div>

                <input type="submit" class="btn validate <?php echo $this->configModel->get('registration_registerbutton_class', 'btn-secondary');?>" value="<?php echo Text::_('COM_JFBCONNECT_REGISTER') ?>" />

                <input type="hidden" name="option" value="com_jfbconnect" />
                <input type="hidden" name="task" value="loginregister.createNewUser" />
                <input type="hidden" name="provider" value="<?php echo $this->providerName; ?>" />
                <?php echo HTMLHelper::_('form.token'); ?>
                </fieldset>
            </form>
        </div>
    </div>

    <div style="clear:both;"></div>

    <?php
    if ($this->configModel->get('show_powered_by_link'))
    {
        $link = JFBConnectUtilities::getAffiliateLink($this->configModel->get('affiliate_id'), EXT_JFBCONNECT);
        ?>
        <div id="powered-by"><?php echo Text::_('COM_JFBCONNECT_POWERED_BY'); ?> <a target="_blank"
                                                                                     href="<?php echo $link; ?>"
                                                                                     title="<?php echo HTMLHelper::_('tooltipText', 'Joomla Facebook Integration');?>">JFBConnect</a>
        </div>
    <?php } ?>

</div>