<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// no direct access
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\Registry\Registry;
use Joomla\CMS\Table\Table;
use Joomla\CMS\Installer\Installer;
use Joomla\CMS\Installer\InstallerHelper;
use Joomla\CMS\Log\Log;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Filesystem\Stream;
use Joomla\CMS\Filesystem\Folder;

jimport('joomla.filesystem.folder');
jimport('joomla.installer.helper');
jimport('joomla.installer.adapters.plugin');

class com_JFBConnectInstallerScript
{
    var $jfbcVersion;
    var $packages = array();

    var $supportedMinimumJoomla = "3.9.0";
    var $systemPluginEnabled = '0';
    var $authPluginEnabled = '0';
    var $userPluginEnabled = '0';
    var $contentPluginEnabled = '0';
    var $providerPluginsEnabled = array();

    var $installedRow = '0';
    var $installer;

    public function __construct($installer)
    {
        $this->installer = $installer;

        $this->providerPluginsEnabled = $this->getProviderPluginsEnabled("1");

        $packages = array();

        $fileStream = new Stream();
        $fileStream->open($this->installer->getParent()->getPath('source') . '/administrator/install/install.modules');
        $rawModules = $fileStream->read();
        $fileStream->close();

        $rawModules = str_replace("\r\n", "\n", $rawModules);
        $modules = explode("\n", $rawModules);

        foreach ($modules as $m)
        {
            $parts = explode('|', $m);
            if (strpos($parts[0], '[') === 0)
                $group = str_replace(array('[', ']'), '', $parts[0]);
            else
                $packages[$group][] = array('name' => $parts[1], 'file' => 'modules/' . $parts[0] . '.zip', 'install' => true);
        }

        $fileStream->open($this->installer->getParent()->getPath('source') . '/administrator/install/install.plugins');
        $rawPlugins = $fileStream->read();
        $fileStream->close();
        $rawPlugins = str_replace("\r\n", "\n", $rawPlugins);
        $plugins = explode("\n", $rawPlugins);

        foreach ($plugins as $p)
        {
            $parts = explode('|', $p);
            if (strpos($parts[0], '[') === 0)
                $group = str_replace(array('[', ']'), '', $parts[0]);
            else
            {
                $pluginFolder = explode('.', $parts[0]);
                $pluginFolder = $pluginFolder[0];
                if ($pluginFolder == 'installer' && version_compare(JVERSION, '3.2.0', '>='))
                    continue; // Don't install the installer plugin on Joomla 3.2+ as we use the extra_query param instead
                $packages[$group][] = array('name' => $parts[1], 'file' => 'plugins/' . str_replace(".", "/", $parts[0]) . '.zip', 'install' => $this->isThirdPartyInstalled($parts[2]));
            }
        }
        // Hard code the SourceCoast library here. Not optimal, but we know we'll need it.
        $packages['Core'][] = array('name' => 'SourceCoast Library', 'file' => 'libraries/sourcecoast.zip', 'install' => true);

        ksort($packages);
        $this->packages = $packages;
    }

    private function isThirdPartyInstalled($component)
    {
        if ($component == 1)
            return true;
        else
            return Folder::exists(JPATH_SITE . '/components/' . $component);
    }

    private function getProviderPluginsEnabled($enabled)
    {
        $providerPluginsEnabled['provider_facebook'] = $enabled;
        $providerPluginsEnabled['provider_facebookbusiness'] = $enabled;
        $providerPluginsEnabled['provider_google'] = $enabled;
        $providerPluginsEnabled['provider_linkedin'] = $enabled;
        $providerPluginsEnabled['provider_twitter'] = $enabled;
        return $providerPluginsEnabled;
    }

    public function preflight($type, $parent)
    {
        if($type != 'uninstall')
        {
            // Check if Joomla version is correct. Mainly, J2.5 can't be installed on J3.0
            if (version_compare(JVERSION, $this->supportedMinimumJoomla, '<'))
            {
                $installStr = 'JFBConnect requires Joomla ' . $this->supportedMinimumJoomla . ' or higher. Please upgrade Joomla to the latest stable release to use JFBConnect.';
                Factory::getApplication()->enqueueMessage($installStr, 'error');
                return false;
            }
            $this->isCombinedVersionInstalled($parent);
        }

        if($type == 'update')
        {
            //Remove old provider directories
            if(Folder::exists(JPATH_SITE . '/components/com_jfbconnect/libraries/provider/'))
                Folder::delete(JPATH_SITE . '/components/com_jfbconnect/libraries/provider/');
            if(Folder::exists(JPATH_SITE . '/components/com_jfbconnect/libraries/profile/'))
                Folder::delete(JPATH_SITE . '/components/com_jfbconnect/libraries/profile/');
        }

        return true;
    }

    public function install($parent)
    {
        $this->providerPluginsEnabled = $this->getProviderPluginsEnabled("1");
        return true;
    }

    private function isCombinedVersionInstalled($parent)
    {
        if(JVERSION < 4.0)
        {
            $manifest = new Registry();
            $extension = $parent->get('extension');
            $cache = $extension->get('manifest_cache');
            $manifest->loadString($cache);
            $prevVersion = $manifest->get('version');
            if (version_compare($prevVersion, '9.0.0', '<'))
            {
                $this->providerPluginsEnabled = $this->getProviderPluginsEnabled("1");
            }
        }
    }

    public function update($parent)
    {
        $this->savePluginState();
        $this->disablePlugins();

        return true;
    }

    public function postflight($type, $parent)
    {
        if($type != 'uninstall')
        {
            $this->loadLanguage('com_jfbconnect', JPATH_ADMINISTRATOR);

            // Run required updates that may not have happened if user uninstalls and does a fresh install
            $manifest = $parent->getParent()->getManifest();
            $this->jfbcVersion = (string)$manifest->version;
            $this->updateDatabase();
            $this->installPackages();
            $this->migrateProfilePluginSettings();
            $this->migrateProfilePluginSettings51();
            $this->migrateProfilePluginSettings612();
            $this->migrateChannelSettings();

            $this->enableInstallerPlugin();

            $this->enablePlugins($this->systemPluginEnabled, $this->authPluginEnabled, $this->userPluginEnabled, $this->contentPluginEnabled, $this->providerPluginsEnabled);
        }
        return true;
    }

    /*
         * $parent is the class calling this method
         * uninstall runs before any other action is taken (file removal or database processing).
         */
    public function uninstall($parent)
    {
        $this->disablePlugins();
    }

    // CUSTOM JFBCONNECT FUNCTIONS
    private function savePluginState()
    {
        $db = Factory::getDBO();
        $query = $db->getQuery(true);
        $query->select($db->qn('element'))
                ->select($db->qn('enabled'))
                ->from($db->qn('#__extensions'))
                ->where("(" . $db->qn('element') . '=' . $db->q('jfbconnectauth') . ' AND ' .
                $db->qn('folder') . '=' . $db->q('authentication') . ' AND ' .
                $db->qn('type') . '=' . $db->q('plugin') .
                ') OR (' .
                $db->qn('element') . '=' . $db->q('jfbcsystem') . ' AND ' .
                $db->qn('folder') . '=' . $db->q('system') . ' AND ' .
                $db->qn('type') . '=' . $db->q('plugin') .
                ') OR (' .
                $db->qn('element') . '=' . $db->q('jfbccontent') . ' AND ' .
                $db->qn('folder') . '=' . $db->q('content') . ' AND ' .
                $db->qn('type') . '=' . $db->q('plugin') .
                ') OR (' .
                $db->qn('element') . '=' . $db->q('jfbconnectuser') . ' AND ' .
                $db->qn('folder') . '=' . $db->q('user') . ' AND ' .
                $db->qn('type') . '=' . $db->q('plugin') .
                ') OR (' .
                $db->qn('element') . ' LIKE ' . $db->q('provider_%') . ' AND ' .
                $db->qn('folder') . '=' . $db->q('jfbconnect') . ' AND ' .
                $db->qn('type') . '=' . $db->q('plugin') .
                ')');

        $db->setQuery($query);
        $pluginValues = $db->loadObjectList();

        if ($pluginValues)
        {
            foreach ($pluginValues as $plugin)
            {
                $pluginName = $plugin->element;
                $pluginPublished = $plugin->enabled;

                if ($pluginName == 'jfbconnectauth')
                    $this->authPluginEnabled = $pluginPublished;
                else if ($pluginName == 'jfbconnectuser')
                    $this->userPluginEnabled = $pluginPublished;
                else if ($pluginName == 'jfbcsystem')
                    $this->systemPluginEnabled = $pluginPublished;
                else if ($pluginName == 'jfbccontent')
                    $this->contentPluginEnabled = $pluginPublished;
                else if(array_key_exists($pluginName, $this->providerPluginsEnabled) &&
                    $this->providerPluginsEnabled[$pluginName] == '0')
                    $this->providerPluginsEnabled[$pluginName] = $pluginPublished;
            }
        }
    }

    private function disablePlugins()
    {
        $providerPluginsEnabled = $this->getProviderPluginsEnabled("0");

        $this->enablePlugins("0", "0", "0", "0", $providerPluginsEnabled);
    }

    private function enablePlugins($enableSystem, $enableAuth, $enableUser, $enableContent, $enableProviders)
    {
        $db = Factory::getDBO();
        $query = $db->getQuery(true);
        $query->update($db->qn('#__extensions'))
                ->set($db->qn('enabled') . '=' . $db->q($enableSystem))
                ->where($db->qn('element') . '=' . $db->q('jfbcsystem'))
                ->where($db->qn('folder') . '=' . $db->q('system'))
                ->where($db->qn('type') . '=' . $db->q('plugin'));
        $db->setQuery($query);
        $db->execute();

        $query = $db->getQuery(true);
        $query->update($db->qn('#__extensions'))
                ->set($db->qn('enabled') . '=' . $db->q($enableAuth))
                ->where($db->qn('element') . '=' . $db->q('jfbconnectauth'))
                ->where($db->qn('folder') . '=' . $db->q('authentication'))
                ->where($db->qn('type') . '=' . $db->q('plugin'));
        $db->setQuery($query);
        $db->execute();

        $query = $db->getQuery(true);
        $query->update($db->qn('#__extensions'))
                ->set($db->qn('enabled') . '=' . $db->q($enableUser))
                ->where($db->qn('element') . '=' . $db->q('jfbconnectuser'))
                ->where($db->qn('folder') . '=' . $db->q('user'))
                ->where($db->qn('type') . '=' . $db->q('plugin'));
        $db->setQuery($query);
        $db->execute();

        $query = $db->getQuery(true);
        $query->update($db->qn('#__extensions'))
                ->set($db->qn('enabled') . '=' . $db->q($enableContent))
                ->where($db->qn('element') . '=' . $db->q('jfbccontent'))
                ->where($db->qn('folder') . '=' . $db->q('content'))
                ->where($db->qn('type') . '=' . $db->q('plugin'));
        $db->setQuery($query);
        $db->execute();

        foreach ($enableProviders as $pName => $pValue)
        {
            $query = $db->getQuery(true);
            $query->update($db->qn('#__extensions'))
                ->set($db->qn('enabled') . '=' . $db->q($pValue))
                ->where($db->qn('element') . '=' . $db->q($pName))
                ->where($db->qn('folder') . '=' . $db->q('jfbconnect'))
                ->where($db->qn('type') . '=' . $db->q('plugin'));
            $db->setQuery($query);
            $db->execute();
        }
    }

    private function updateDatabase()
    {
        // If user uninstalled JFBConnect and is re-installing, the tables will exist, but upgrade SQL files won't be called
        // Add SQL calls here as a backup
        $db = Factory::getDBO();
        $query = $db->getQuery(true);
        $query->select($db->qn('value'))
                ->from($db->qn('#__jfbconnect_config'))
                ->where($db->qn('setting') . '=' . $db->q('db_version'));
        $db->setQuery($query);
        $dbVersion = $db->loadResult();

        // Get the update files for the current database type
        $files = Folder::files(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/install/sql/updates/', '\.' . $this->getDbType() . '\.sql');
        foreach ($files as $f)
        {
            $updateVersion = str_replace('.' . $this->getDbType() . '.sql', '', $f);
            if (version_compare($dbVersion, $updateVersion, '<'))
                $this->runUpdateSQL($updateVersion);
        }
    }

    private function getDbType()
    {
        return Factory::getDBO()->name == "postgresql" ? 'postgre' : 'mysql';
    }

    private function runUpdateSQL($version)
    {
        $db = Factory::getDBO();
        $buffer = file_get_contents(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/install/sql/updates/' . $version . '.' . $this->getDbType() . '.sql');

        // Graceful exit and rollback if read not successful
        if ($buffer === false)
        {
            Log::add(Text::sprintf('JLIB_INSTALLER_ERROR_SQL_READBUFFER', $db->stderr(true)), Log::WARNING, 'jerror');
            return false;
        }

        // Create an array of queries from the sql file
        $queries = JDatabaseDriver::splitSql($buffer);

        $update_count = 0;
        if (count($queries) != 0)
        {
            // Process each query in the $queries array (split out of sql file).
            foreach ($queries as $query)
            {
                $query = trim($query);
                if ($query != '' && substr($query, 0, 1) != '#')
                {
                    $db->setQuery($query);

                    if (!$db->execute())
                    {
                        Log::add(Text::sprintf('JLIB_INSTALLER_ERROR_SQL_ERROR', $db->stderr(true)), Log::WARNING, 'jerror');
                        return false;
                    }
                    $update_count++;
                }
            }
        }
        // All done. Update the db_version row to the latest number
        $query = $db->getQuery(true);
        $query->select($db->qn('value'))
                ->from('#__jfbconnect_config')
                ->where($db->qn('setting') . '=' . $db->q('db_version'));
        $db->setQuery($query);
        $ver = $db->loadResult();

        $query->clear();
        if ($ver)
        {
            $query->update('#__jfbconnect_config')
                    ->where($db->qn('setting') . '=' . $db->q('db_version'))
                    ->set($db->qn('value') . '=' . $db->q($version))
                    ->set($db->qn('updated_at') . '= NOW()');
        }
        else
        {
            $query->insert('#__jfbconnect_config')
                    ->columns($db->qn('setting') . "," . $db->qn('value') . ',' . $db->qn('created_at') . "," . $db->qn('updated_at'))
                    ->values($db->q('db_version') . ',' . $db->q($version) . ', NOW(), NOW()');
        }

        $db->setQuery($query);
        $db->execute();
    }

    private function installPackages()
    {
        // Get current version number
        ?>

        <table>
            <tr>
                <td width="100px"><img
                            src="<?php print Uri::root(); ?>/administrator/components/com_jfbconnect/assets/images/jfbconn.png"
                            width="100px"></td>
                <td><h2>JFBConnect v<?php echo $this->jfbcVersion; ?></h2></td>
            </tr>
        </table>
        <?php echo $this->showDoneMessage(); ?>
        <h3><?php echo Text::_('COM_JFBCONNECT_INSTALL_SUCCESS');?></h3>

        <?php
        echo '<table class="adminlist table table-striped" width="100%">';
        echo '<thead><tr><th class="title">'.Text::_('COM_JFBCONNECT_INSTALL_COL_EXTENSION').'</th><th width="40%">'.Text::_('COM_JFBCONNECT_INSTALL_COL_STATUS').'</th></tr></thead>';
        echo '<tbody>';
        echo '<tr class="row0"><td>'.Text::_('COM_JFBCONNECT_INSTALL_ROW_JFBC_LABEL').'</td><td><span style="color:green; font-weight:bold">'.Text::_('COM_JFBCONNECT_INSTALL_ROW_INSTALLED').'</span></td></tr>';
        foreach ($this->packages as $group => $packages)
        {
            echo '<tr><th><strong>' . $group . '</strong></th><th><center>'.Text::_('COM_JFBCONNECT_INSTALL_COL_STATUS').'</center></th></tr>';
            echo $this->installPackageList($packages);
        }
        echo '</tbody>';
        echo '</table>';
    }

    private function installPackageList($packages)
    {
        foreach ($packages as $package)
        {
            $installer = new Installer();
            $installer->setOverwrite(true);
            $pkgName = $package['name'];
            $pkgFile = $package['file'];
            $pkgInstall = $package['install'];

            $this->installedRow++;

            if (!$pkgInstall)
                $installed = '<span style="color:blue; font-weight:bold">'.Text::_('COM_JFBCONNECT_INSTALL_ROW_NOTINSTALLED').'</span>';
            else
            {
                $pkg = InstallerHelper::unpack($this->installer->getParent()->getPath('source') . '/packages/' . $pkgFile);
                if ($installer->install($pkg['dir']))
                    $installed = '<span style="color:green; font-weight:bold">'.Text::_('COM_JFBCONNECT_INSTALL_ROW_INSTALLED').'</span>';
                else
                    $installed = '<span style="color:red; font-weight:bold">'.Text::_('COM_JFBCONNECT_INSTALL_ROW_INSTALLMANUALLY').'</span>';

                Folder::delete($pkg['extractdir']);
            }
            ?>
            <tr class="row<?php echo($this->installedRow % 2); ?>">
                <td><?php echo $pkgName; ?></td>
                <td><?php echo $installed; ?></td>
            </tr>
        <?php
        }
    }

    private function showDoneMessage()
    {
        ?>
        <p style="font-weight:bold; margin-top:20px"><?php echo Text::_('COM_JFBCONNECT_INSTALL_AUTOTUNE_DESC');?></p>
        <center><a href="index.php?option=com_jfbconnect&view=autotune"
                   style="background-color:#025A8D;color:#FFFFFF;height:35px;padding:15px 45px;font-weight:bold;font-size:18px;line-height:60px;
                   text-decoration:none;text-shadow:0 -1px 1px #565656;-webkit-border-radius:7px;border-radius:7px;-webkit-box-shadow:0 1px 3px 0 #565656;
                   box-shadow:0 1px 3px 0 #565656;"><?php echo Text::_('COM_JFBCONNECT_INSTALL_AUTOTUNE_RUNNOW');?></a></center>
    <?php
    }

    /*
     * JFBConnect v5.0 - Removed JFBCProfile plugins and moved to SocialProfiles plugins. Old plugins used a 1-row per setting in
     * #__jfbconnect_config. New plugins use a 1-row json variable for all settings that are loaded into a registry.
     * This function converts old settings to new format, if they exist.
     */
    private function migrateProfilePluginSettings()
    {
        $db = Factory::getDBO();
        $query = $db->getQuery(true);
        $query->select("setting,value")->from($db->qn('#__jfbconnect_config'))->where($db->qn('setting') . ' LIKE ' . $db->q('profiles_%'))->order('setting');
        $rows = $db->setQuery($query)->loadObjectList();

        if (!empty($rows))
        {
            require_once(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/models/config.php');
            $configModel = new JFBConnectModelConfig();
            $pluginName = "";
            $reg = new Registry();
            foreach ($rows as $row)
            {
                $values = explode("_", $row->setting);
                if ($pluginName != $values[1])
                {
                    $pluginName = $values[1];
                    // Remove the previous profiles_xyz settings from the config table
                    $query = $db->getQuery(true);
                    $query->delete($db->qn('#__jfbconnect_config'))->where($db->qn('setting') . ' LIKE ' . $db->q('profiles_' . $pluginName . '%'));
                    $db->setQuery($query)->execute();
                }
                unset($values[0]);
                unset($values[1]);
                $settingName = implode("_", $values);
                $value = $row->value;
                if ($settingName == "status_updates_pull_from_fb")
                    $settingName = "import_status";
                if ($settingName == "field_map")
                {
                    $fieldMap = unserialize($value);
                    $newMap = new stdClass();
                    foreach ($fieldMap as $key => $value)
                    {
                        if ($pluginName == "jomsocial")
                            $key = 'field' . $key;
                        $newMap->$key = $value;
                    }
                    $value = $newMap;
                }
                $reg->set($settingName, $value);
                $configModel->update('profile_' . $pluginName, $reg->toString());
            }
        }
    }

    private function migrateProfilePluginSettings51()
    {
        $db = Factory::getDBO();
        $query = $db->getQuery(true);
        // fixed PHP notice from invalid query
        $query->select("setting,value")->from($db->qn("#__jfbconnect_config"))->where($db->qn('setting') . " LIKE " . $db->q("profile_%"))->order('setting');
        $rows = $db->setQuery($query)->loadObjectList();
        if (!empty($rows))
        {
            require_once(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/models/config.php');
            $configModel = new JFBConnectModelConfig();
            foreach ($rows as $row)
            {
                $values = explode("_", $row->setting);
                $pluginName = $values[1];

                $settings = new Registry();
                $settings->loadString($row->value);
                if ($settings->exists('field_map') && !$settings->exists('field_map.facebook'))
                {
                    $fieldMap = clone($settings->get('field_map'));
                    if ($pluginName == "jomsocial")
                    {
                        $newMap = new stdClass();
                        foreach ($fieldMap as $key => $value)
                        {
                            $newKey = str_replace('field', '', $key);
                            $newMap->$newKey = $value;
                        }
                        $fieldMap = $newMap;
                    }
                    $settings->set("field_map", null);
                    $settings->set("field_map.facebook", $fieldMap);
                    $configModel->update('profile_' . $pluginName, $settings->toString());
                }
            }
        }
    }

    private function migrateChannelSettings()
    {
        $db = Factory::getDBO();

        //Get list of channel params that have to be reset to facebook business
        $query = $db->getQuery(true);
        $query->select('id, attribs')
            ->from($db->qn("#__jfbconnect_channel"))
            ->where($db->qn('provider') . '=' . $db->q('facebook'))
            ->where($db->qn('type') . '<>' . $db->q('profile'));
        $db->setQuery($query);
        $rows = $db->loadObjectList();

        //Reset provider name
        $query = $db->getQuery(true);
        $query->update($db->qn('#__jfbconnect_channel'))
            ->set($db->qn('provider') . '=' . $db->q('facebookbusiness'))
            ->set($db->qn('published') . '=' . $db->q('0'))
            ->where($db->qn('provider') . '=' . $db->q('facebook'))
            ->where($db->qn('type') . '<>' . $db->q('profile'));
        $db->setQuery($query);
        $db->execute();

        //Loop through users to remove the scope from the old channel
        include_once(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/models/usermap.php');
        Table::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/tables/');
        foreach($rows as $row)
        {
            $params = new Registry($row->attribs);
            $channelOwner = $params->get('user_id');
            $userModel = JFBConnectModelUserMap::getUser($channelOwner, 'facebook');
            $userModel->removeAllScope('channel', $row->id);
        }
    }

    private function migrateProfilePluginSettings612()
    {
        $db = Factory::getDBO();
        $query = $db->getQuery(true);
        // fixed PHP notice from invalid query
        $query->select("setting,value")->from($db->qn("#__jfbconnect_config"))->where($db->qn('setting') . " LIKE " . $db->q("profile_%"))->order('setting');
        $rows = $db->setQuery($query)->loadObjectList();
        if (!empty($rows))
        {
            require_once(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/models/config.php');
            $configModel = new JFBConnectModelConfig();
            foreach ($rows as $row)
            {
                $values = explode("_", $row->setting);
                $pluginName = $values[1];

                $settings = new Registry();
                $settings->loadString($row->value);
                if ($settings->exists('field_map') && $settings->exists('field_map.facebook'))
                {
                    $fieldMap = clone($settings->get('field_map'));
                    foreach ($fieldMap->facebook as $key => $value)
                    {
                        if ($value == "sex")
                            $fieldMap->facebook->$key = "gender";
                        if ($value == "profile_url")
                            $fieldMap->facebook->$key = "link";
                        if ($value == "about_me")
                            $fieldMap->facebook->$key = "bio";
                        if ($value == "tv")
                            $fieldMap->facebook->$key = "television";
                        if (in_array($value, array("hometown_location.city", "hometown_location.state", "hometown_location.country") ) )
                            $fieldMap->facebook->$key = "hometown.name";
                        if (in_array($value, array("current_location.city", "current_location.state", "current_location.country") ) )
                            $fieldMap->facebook->$key = "location.name";
                    }
                    $settings->set("field_map", null);
                    $settings->set("field_map", $fieldMap);
                    $configModel->update('profile_' . $pluginName, $settings->toString());
                }
            }
        }
    }

    private function enableInstallerPlugin()
    {
        $db = Factory::getDbo();
        $query = $db->getQuery(true);
        $query->update($db->qn('#__extensions'))
                ->set($db->qn('enabled') . '=1')
                ->where($db->qn('element') . '=' . $db->q('jfbconnect'))
                ->where($db->qn('folder') . '=' . $db->q('installer'))
                ->where($db->qn('type') . '=' . $db->q('plugin'));
        $db->setQuery($query);
        $db->execute();
    }

    private function loadLanguage($extension, $basePath = JPATH_BASE)
    {
        // Load translations
        $jlang = Factory::getApplication()->getLanguage();
        $jlang->load($extension, $basePath, 'en-GB', true); // Load English (British)
        // $jlang->load($extension, JPATH_BASE, $jlang->getDefault(), true); // Load the site's default language
        $jlang->load($extension, $basePath, null, true); // Load the currently selected language
    }
}