<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Filesystem\File;

/**
 * Handle commercial extension update authorization
 *
 * @since       2.5
 */
class PlgInstallerJfbconnect extends CMSPlugin
{
    public function onInstallerBeforePackageDownload(&$url, &$headers)
    {
        $uri = Uri::getInstance($url);

        $host = $uri->getHost();
        if ($host != 'www.sourcecoast.com')
        {
            return true;
        }

        $factoryFile = JPATH_ROOT . '/components/com_jfbconnect/libraries/factory.php';
        if (!File::exists($factoryFile))
        {
            return true; // Don't finish loading this plugin to prevent other errors
        }
        require_once($factoryFile);

        $subscriberId = JFBCFactory::config()->get('sc_download_id');

        // If the download ID is invalid, return without any further action
        if (!preg_match('/^([0-9]{1,}:)?[0-9a-f]{32}$/i', $subscriberId))
        {
            return true;
        }

        if (!empty($subscriberId))
        {
            $uri->setVar('dlid', $subscriberId);
            $url = $uri->toString();
        }

        return true;
    }

}
