<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\CMSPlugin;

jimport('sourcecoast.plugins.jfbconnectPlugin');

/**
 * Facebook User Plugin
 */
class plgUserJfbconnectUser extends JFBConnectPlugin
{
    function __construct(& $subject, $config)
    {
        parent::__construct($subject, $config);
    }

    function onUserAfterSave($user, $isnew, $success, $msg)
    {
        if($this->isLoaded())
        {
            if (!$isnew)
                return true;

            $app = Factory::getApplication();
            if ($app->getUserState('com_jfbconnect.registration.alternateflow', false))
            {
                $provider = $app->getUserState('com_jfbconnect.registration.provider.name', null);
                $providerUserId = $app->getUserState('com_jfbconnect.registration.provider.user_id', null);

                if ($provider && $providerUserId)
                {
                    $provider = JFBCFactory::provider($provider);
                    if ($user['id'] && $provider && $provider->getProviderUserId() == $providerUserId) // Sanity check
                    {
                        JFBCFactory::usermap()->map($user['id'], $providerUserId, $provider->systemName, $provider->client->getToken());
                        // If that worked, now call the originating plugin and tell it to finalize anything with the new user
                        $args = array($provider->systemName, $user['id'], $providerUserId);
                        $app->triggerEvent('onSocialProfilesOnNewUserSave', $args);
                    }
                }
            }
        }
    }

    function onUserLogout($user, $options = array())
    {
        if($this->isLoaded())
        {
            // Disable auto-logins for session length after a logout. Prevents auto-logins
            $config = Factory::getConfig();
            $lifetime = $config->get('lifetime', 15);
            setcookie('jfbconnect_autologin_disable', 1, time() + ($lifetime * 60));
            setcookie('jfbconnect_permissions_granted', '', time() - 10000, "/"); // clear the granted permissions cookie

            // Tell Facebook to delete session information stored for this user.
            $fbProvider = JFBCFactory::provider('facebook');
            if($fbProvider)
                $fbProvider->client->destroySession();

            return true;
        }
        return false;

    }

    function onUserBeforeDelete($user)
    {
        if($this->isLoaded())
        {
            $db = Factory::getDbo();
            $query = $db->getQuery(true);
            $query->delete($db->qn('#__jfbconnect_user_map'))
                ->where($db->qn('j_user_id') . "=" . $db->q($user['id']));
            $db->setQuery($query);
            $db->execute();

            // Remove other user data from open graph tables
            $query = $db->getQuery(true);
            $query->delete($db->qn('#__opengraph_activity'))
                ->where($db->qn('user_id') . "=" . $db->q($user['id']));
            $db->setQuery($query);
            $db->execute();
        }
    }

}
