<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Session\Session;

jimport('sourcecoast.plugins.jfbconnectPlugin');
jimport('joomla.event.plugin');
jimport('sourcecoast.openGraph');
jimport('sourcecoast.easyTags');
jimport('sourcecoast.articleContent');
jimport('joomla.filesystem.file');

class plgSystemJFBCSystem extends JFBConnectPlugin
{
    var $tagsToReplace;
    var $metadataTagsToStrip = array('JFBC', 'JLinked', 'SC');
    var $scTags;

    static $cssIncluded = false;

    function __construct(& $subject, $config)
    {
        parent::__construct($subject, $config);

        if(!$this->factoryFileFound)
        {
            Factory::getApplication()->enqueueMessage("File missing: " . $this->factoryFile . "<br/>Please re-install JFBConnect or disable the JFBCSystem Plugin", 'error');
        }
        else
        {
            require_once($this->factoryFile);
            JFBCFactory::initializeJoomla();

            $this->scTags = JFBConnectEasytags::getSCTags();
        }
    }

    public function onAfterInitialise()
    {
        //Always load openGraph plugin so that we can listen for Auto-import
        PluginHelper::importPlugin('opengraph');

        $app = Factory::getApplication();
        if ($this->isLoaded() && $app->isClient('site'))
        {
            // Need to disable Page caching so that values fetched from Facebook are not saved for the next user!
            // Do this by setting the request type to POST. In the Cache plugin, it's checked for "GET". can't be that.
            $option = $app->input->getCmd("option");
            $view = $app->input->getCmd("view");
            if ($option == 'com_jfbconnect' && $view == 'loginregister')
                $_SERVER['REQUEST_METHOD'] = 'POST';

            // Need to load our plugin group early to be able to hook into to every step after
            PluginHelper::importPlugin('socialprofiles');

            $providers = JFBCFactory::getAllProviders();
            foreach ($providers as $provider)
                $provider->onAfterInitialise();
        }
    }

    public function onAfterRoute()
    {
        $app = Factory::getApplication();
        if ($this->isLoaded() && $app->isClient('site'))
        {
            $app = Factory::getApplication();
            if ($app->getUserState('com_jfbconnect.registration.alternateflow'))
                $app->triggerEvent('onSocialProfilesPrefillRegistration');
        }
    }

    // Called after the component has executed and it's output is available in the buffer
    // Modules have *not* executed yet
    public function onAfterDispatch()
    {
        $app = Factory::getApplication();
        if ($this->isLoaded() && $app->isClient('site'))
        {
            $providers = JFBCFactory::getAllProviders();
            $this->addJavascriptInit($providers);

            foreach ($this->metadataTagsToStrip as $metadataTag)
            {
                $this->replaceTagInMetadata($metadataTag);
            }

            $doc = Factory::getApplication()->getDocument();
            $loadBsClass= JFBCFactory::config()->get('bootstrap_css');
            if($loadBsClass == '2' || JVERSION >= 4.0)
                $doc->addStyleSheet(Uri::base(true) . '/media/sourcecoast/css/sc_bootstrap5.css');
            else if($loadBsClass == '1')
                $doc->addStyleSheet(Uri::base(true) . '/media/sourcecoast/css/sc_bootstrap.css');

            $doc->addStyleSheet(Uri::base(true) . '/media/sourcecoast/css/common.css');

            if ($doc->getType() == 'html')
            {
                $doc->addCustomTag('<SourceCoastProviderJSPlaceholder />');
                $doc->addCustomTag('<SourceCoastCSSPlaceholder />');

                if (JFBCFactory::config()->get('jquery_load'))
                    HTMLHelper::_('jquery.framework');
            }

            //Add Login with FB button to com_users login view and mod_login
            $showLoginWithJoomla = JFBCFactory::config()->get('show_login_with_joomla_reg');

            if ($showLoginWithJoomla != SC_VIEW_NONE && count($providers) > 0)
            {
                $providerName = $providers[0]->systemName;
                JFBConnectUtilities::loadLanguage('com_jfbconnect');

                if (JFBCFactory::easytags()->canExtendJoomlaForm("login", false, $showLoginWithJoomla))
                {
                    $login = JFBCFactory::cache()->get('system.joomlaform.login');
                    if ($login === false)
                    {
                        $login = JFBCFactory::provider($providerName)->getLoginButton(Text::_('COM_JFBCONNECT_LOGIN_WITH'));
                        JFBCFactory::cache()->store($login, 'system.joomlaform.login');
                    }
                    JFBCFactory::easytags()->extendJoomlaUserForms($login, $showLoginWithJoomla);
                }

                if (JFBCFactory::easytags()->canExtendJoomlaForm('registration', false, $showLoginWithJoomla))
                {
                    $registration = JFBCFactory::cache()->get('system.joomlaform.registration');
                    if ($registration === false)
                    {
                        $registration = JFBCFactory::provider($providerName)->getLoginButton(Text::_('COM_JFBCONNECT_REGISTER_WITH'));
                        JFBCFactory::cache()->store($registration, 'system.joomlaform.registration');
                    }
                    JFBCFactory::easytags()->extendJoomlaUserForms($registration, $showLoginWithJoomla);
                }
            }

            $app->triggerEvent('onOpenGraphAfterDispatch');
            $this->openGraphUpdatePending();

            // Finally, load the Toolbar classes
            JFBCFactory::library('toolbar')->onAfterDispatch();
        }
    }

    private function addJavascriptInit($providers)
    {
        $doc = Factory::getApplication()->getDocument();
        if ($doc->getType() != 'html')
            return; // Only insert javascript on HTML pages, not AJAX, RSS, etc

        $app = Factory::getApplication();
        $state = $app->getUserState('users.login.form.data', null);
        $return = null;
        if ($app->input->getCmd('option') == 'com_users' && $app->input->getCmd('view') == 'login' && is_array($state) && isset($state['return']))
            $return = urldecode($state['return']);
        if (!$return)
            $return = urldecode(base64_decode($app->input->getBase64('return', '')));
        if (!$return)
        {
            $uri = Uri::getInstance();
            $return = $uri->toString(array('path', 'query'));
            if ($return == "")
                $return = 'index.php';
        }

        $debugCmd = JFBCFactory::config()->get('facebook_display_errors') ? "jfbc.debug.enable = '1';\n" : '';

        $doc->addScript(Uri::base(true) . '/components/com_jfbconnect/includes/jfbconnect.js?v=6');

        $customTag = '<script type="text/javascript">' .
            "jfbc.base = '" . Uri::base() . "';\n" .
            "jfbc.return_url = '" . base64_encode($return) . "';\n" .
            "jfbc.token = '" . Session::getFormToken() . "';\n";
        foreach ($providers as $provider)
        {
            $initJS = $provider->onGetProviderJavascriptInit();
            $customTag .= $initJS;
        }
        $customTag.= $debugCmd.
            "jfbc.init();\n" .
            "</script>";

        $doc->addCustomTag($customTag);

        if(!JFBCFactory::config()->get('social_graph_turn_off_all'))
            $doc->addCustomTag('<SCOpenGraphPlaceholder />');
    }

    // Called right before the page is rendered
    public function onBeforeRender()
    {
        if ($this->isLoaded() && Factory::getApplication()->isClient('site') && Factory::getApplication()->getDocument()->getType() == 'html')
        {
            if (Factory::getUser()->authorise('jfbconnect.opengraph.debug', 'com_jfbconnect') && JFBCFactory::config()->get('facebook_display_errors'))
                JFBCFactory::addStylesheet('jfbconnect.css');

            $providers = JFBCFactory::getAllProviders();
            foreach ($providers as $provider)
                $provider->onBeforeRender();
        }
    }

    public function onAfterRender()
    {
        if ($this->isLoaded() && Factory::getApplication()->isClient('site') && Factory::getApplication()->getDocument()->getType() == 'html')
        {
            $this->doTagReplacements();

            $providers = JFBCFactory::getAllProviders();
            foreach ($providers as $provider)
                $provider->onAfterRender();

            JFBCFactory::library('toolbar')->onAfterRender();
            $this->replaceCSSPlaceholder();
        }
        return true;
    }

    private function replaceCSSPlaceholder()
    {
        $app = Factory::getApplication();
        $cssFiles = JFBCFactory::getStylesheets();
        $contents = $app->getBody();
        $tag = '';
        foreach ($cssFiles as $f)
        {
            $tag .= '<link rel="stylesheet" href="' . Uri::base(true) . '/media/sourcecoast/css/' . $f . '" type="text/css" />';
        }
        $contents = str_replace('<SourceCoastCSSPlaceholder />', $tag, $contents);
        $app->setBody($contents);
    }

    private function replaceTagInMetadata($metadataTag)
    {
        $doc = Factory::getApplication()->getDocument();
        $description = $doc->getDescription();
        $replace = JFBConnectUtilities::stripSystemTags($description, $metadataTag);

        if ($replace)
        {
            $description = JFBConnectUtilities::trimNBSP($description);
            $doc->setDescription($description);
        }
    }

    private function doTagReplacements()
    {
        /*
         * Code to strip any {JFBCxyz} tags from head.
         */
        //Get the head
        $app = Factory::getApplication();
        $contents = $app->getBody();
        $regex = '|<head(.*)?</head>|sui';
        if (preg_match($regex, $contents, $matches))
        {
            if (count($matches) == 2) // more than one head is a problem, don't do anything
            {
                //NOTE: Searching for just SC can incorrectly remove other tags that are valid, e.g. {scrollTop:...}

                //Remove the tag if it's in the head
                $newHead = preg_replace('/\{('.$this->scTags.'|JFBC|JLinked)(.*?)}/ui', '', $matches[0], -1, $count);

                if ($count > 0)
                {
                    //Replace the head
                    $contents = preg_replace('|<head(.*)?</head>|sui', $newHead, $contents, -1, $count);
                    if ($count == 1) // Only update the body if exactly one head was found and replaced
                        $app->setBody($contents);
                }
            }
        }

        $contents = JFBCFactory::easytags()->replaceWidgetTags($contents);

        if(!JFBCFactory::config()->get('social_graph_turn_off_all'))
            $contents = JFBCFactory::easytags()->replaceGraphTags($contents);

        $contents = JFBCFactory::easytags()->replaceJSPlaceholders($contents);

        $app->setBody($contents);
    }

    private function openGraphUpdatePending()
    {
        $input = Factory::getApplication()->input;

        $option = $input->getCmd('option');

        $autopostExt = JFBCFactory::getAllAutopostExtensions();
        if(!in_array($option, $autopostExt))
            return;

        $view = $input->getCmd('view');
        $itemID = $input->getCmd('id'); //com_content, com_k2, com_easyblog

        if($option == 'com_community')
        {
            if($view == 'groups')
                $itemID = $input->getCmd('groupid');
            else if($view == 'events')
                $itemID = $input->getCmd('eventid');
        }

        if($option == 'com_easyblog' && $view == 'entry')
            $view = 'post';

        $db = Factory::getDbo();
        $query = $db->getQuery(true);
        $query->select('act.*')
            ->from('#__jfbconnect_autopost_activity act')
            ->where($db->qn('act.ext') . '=' . $db->q($option) . ' AND ' .
                $db->qn('act.layout') . '=' . $db->q($view) . ' AND ' .
                $db->qn('act.item_id') . '=' . $db->q($itemID) . ' AND ' .
                $db->qn('act.type') . '=1' . ' AND ' .
                $db->qn('act.status') . '=2'
            );
        $row = $db->setQuery($query)->loadObject();

        if($row)
        {
            $app = Factory::getApplication();
            $args = array($row->item_id, Uri::root() . $row->url, $row->ext);
            $app->triggerEvent('openGraphUpdatePending', $args);
        }
    }

}
