<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Table\Table;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;

jimport('sourcecoast.articleContent');
jimport('sourcecoast.openGraphPlugin');

class plgOpenGraphJomSocial extends OpenGraphPlugin
{
    protected function init()
    {
        $this->extensionName = "JomSocial";
        $this->supportedComponents[] = 'com_community';
        $this->supportedAutopostLabel = "Group and Event";
        $this->supportedAutopostTypes[] = 'groups';
        $this->supportedAutopostTypes[] = 'events';

        // Default Open Graph tags are set for profile, album, and group pages even if the admin hasn't defined object types for those pages
        $this->setsDefaultTags = true;

        // Define all types of pages this component can create and would be 'objects'
        $this->addSupportedObject("Group", "groups");
        $this->addSupportedObject("Photo Album", "album");
        $this->addSupportedObject("Profile", "profile");
        $this->addSupportedObject("Event", "events");

        // Slick way to hook into JomSocial activity calls to do our actions. Basically, we're faking, from this point on, that we are a community plugin
        // Joomla doesn't check the type, but JomSocial does, so this works great for us!
        $this->_type = "community";
    }

    protected function findObjectType($queryVars)
    {
        // Setup Object type for page
        $task = array_key_exists('task', $queryVars) ? $queryVars['task'] : '';
        $view = array_key_exists('view', $queryVars) ? $queryVars['view'] : '';
        $type = $task == "photo" ? 'album' : $view;
        $type = ($view == 'featured' ? $task : $type);

        $objectTypes = $this->getObjects($type);
        $object = null;

        if ($view == 'profile' ||
            ($view == 'groups' && $task == 'viewgroup') ||
            ($view == 'photos' && $task == "photo") ||
            ($view == 'events' && $task == 'viewevent') ||
            ($view == 'featured' && $task == 'groups') ||
            ($view == 'featured' && $task == 'events')

        )
        {
            // If there's an object, that's the one we want since we don't current support multiple profile types
            if (array_key_exists('0', $objectTypes))
                $object = $objectTypes[0];
        }
        return $object;
    }

    protected function setOpenGraphTags()
    {
        $view = $this->app->input->getCmd('view');
        $task = $this->app->input->getCmd('task');

        if ($view == 'profile')
        {
            // Set image from profile. Title and description are done by JFBConnect automatically
            $jspath = JPATH_ROOT . '/components/com_community';
            include_once($jspath . '/libraries/core.php');
            // Get CUser object
            $userId = $this->app->input->getInt('userid');
            $user = CFactory::getUser($userId);
            $avatarUrl = $user->getThumbAvatar();
            $this->addOpenGraphTag('image', $avatarUrl, false);

            // Add the canonical URL to the profile
            $uri = Uri::getInstance();
            $url = $uri->toString(array('scheme', 'user', 'pass', 'host', 'port'));
            $url .= CRoute::_('index.php?option=com_community&view=profile&userid=' . $userId, false);
            $this->addOpenGraphTag('url', $url, true);
            $this->addOpenGraphTag('title' , $user->getDisplayName(), true);

            if ($this->object)
            {
                $fieldcode = $this->object->params->get('custom_desc_fieldcode');
                $this->addOpenGraphTag('description' , $user->getInfo($fieldcode), true);
            }
        }
        else if ($view == 'groups' && $task == 'viewgroup')
        {
            $id = $this->app->input->getInt('groupid');
            $url = $this->getGroupUrl($id);

            $this->addOpenGraphTag('url', $url, false);
            //$this->addOpenGraphTag('image', $group->getAvatar('avatar'), false);
        }
        else if ($view == 'photos' && $task == "album")
        {
            $albumId = $this->app->input->getInt('albumid');
            $album = Table::getInstance('Album', 'CTable');
            $album->load($albumId);
            $url = CRoute::getExternalURL($album->getURI());
            $this->addOpenGraphTag('url', $url, true);

            $image = $album->getCoverThumbPath();
            $uri = Uri::getInstance();
            $base = $uri->toString(array('scheme', 'host', 'port'));
            if(strpos($image, $base) === false)
                $image = CRoute::getExternalURL($image);
            $this->addOpenGraphTag('image', $image, false);

            $this->addOpenGraphTag('description', $album->name, false);
        }
        else if ($view == 'photos' && $task == 'photo')
        {
            $photoId = $this->app->input->getInt('photoid');
            $photo = Table::getInstance('Photo', 'CTable');
            $photo->load($photoId);

            $url = 'index.php?option=com_community&view=photos&task=photo&albumid='.$photo->albumid.'&userid='.$photo->creator.'&photoid='.$photoId;
            $url = CRoute::getExternalURL($url);
            $this->addOpenGraphTag('url', $url, true);

            $image = $photo->getOriginalURI();
            $this->addOpenGraphTag('image', $image, false);

            $this->addOpenGraphTag('description', $photo->caption, false);
        }
        else if ($view == 'events' && $task == 'viewevent')
        {
            $id = $this->app->input->getInt('eventid');
            $event = $this->getEvent($id);
            $url = $this->getEventUrl($id);

            $this->addOpenGraphTag('url', $url, false);

            if($event->cover != '')
            {
                $coverImage = Route::_(Uri::root(false) . $event->cover);
                $this->addOpenGraphTag('image', $coverImage, false);
            }
        }
    }

    /* Helper functions */
    private function getGroup($id)
    {
        $group = Table::getInstance('Group', 'CTable');
        $group->load($id);
        return $group;
    }
    private function getGroupUrl($id)
    {
        //$uri = Uri::getInstance();
        //$url = $uri->toString(array('scheme', 'user', 'pass', 'host', 'port'));

        //$group = $this->getGroup($id);
        //$url .= $group->getLink();

        $itemID = $this->app->input->getInt('Itemid');
        $url = 'index.php?option=com_community&view=groups&task=viewgroup&groupid='.$id.'&Itemid='.$itemID;
        $url = CRoute::getExternalURL($url);
        return $url;
    }

    private function getEvent($id)
    {
        $event = Table::getInstance('Event', 'CTable');
        $event->load($id);
        return $event;
    }

    private function getEventUrl($id)
    {
        $itemID = $this->app->input->getInt('Itemid');
        $url = 'index.php?option=com_community&view=events&task=viewevent&eventid='.$id;
        $url = CRoute::getExternalURL($url);
        return $url;
    }

    public function onGroupCreate($params)
    {
        $this->tryAutopublish('groups', $params);
    }

    public function onEventCreate($params)
    {
        $this->tryAutopublish('events', $params);
    }

    private function tryAutopublish($objectType, $params)
    {
        $ogObjects = $this->getObjects($objectType);
        $isPending = $params->published != '1';
        //If event requires mod, object will be 1 but params will be 2.
        //If group requires mod, object will be 1 but params will be 0.
        //Want this to be pending

        if($objectType == 'events')
        {
            $url = $this->getEventUrl($params->id);
        }
        else if($objectType == 'groups')
        {
            $url = $this->getGroupUrl($params->id);
        }
        $this->autopublish($ogObjects[0], $params->id, $url, $isPending);
    }

    public function getAutopostMessage($objectType, $id)
    {
        if($objectType == 'events')
        {
            $event = $this->getEvent($id);
            $message = $event->summary;
        }
        else if($objectType == 'groups')
        {
            $group = $this->getGroup($id);

            if($group->summary != '')
                $message = $group->summary;
            else if($group->description != '')
                $message = $group->description;
            else
                $message = $group->name . " created";
        }

        return $message;
    }

    public function openGraphUpdatePending($articleId, $link, $ext)
    {
        //NOTE: If moderation is required, then the only way I've found to get
        //to autopublish is to visit on the front-end once it's published
        if($ext == $this->supportedComponents[0])
        {
            $view = $this->app->input->getCmd('view');

            if($view == 'events')
            {
                $event = $this->getEvent($articleId);
                $this->tryAutopublish($view, $event);
            }
            else if($view == 'groups')
            {
                $group = $this->getGroup($articleId);
                $this->tryAutopublish($view, $group);
            }
        }
    }

}