<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\Registry\Registry;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\CMS\Uri\Uri;

jimport('sourcecoast.articleContent');
jimport('sourcecoast.openGraphPlugin');

class plgOpenGraphJReviews extends OpenGraphPlugin
{
    protected function init()
    {
        $this->extensionName = "jReviews";
        $this->supportedComponents[] = 'com_content';
        $this->supportedComponents[] = 'com_jreviews';

        // Enable this setting if the setOpenGraph function should be called even if an object type hasn't been defined
        // Good for setting title, description, image, etc tags on pages that may not be defined as objects by the admin
        // For example, in content, those are good tags to have (for Like buttons) regardless of if Actions are setup for the page
        $this->setsDefaultTags = true;

        // Define all types of pages this component can create and would be 'objects'
        // jReviews doesn't have it's own objects, it uses Joomla content. So we just have special actions on that content.
        $this->addSupportedObject("Listing", "listing");
    }

    protected function findObjectType($queryVars)
    {
        // Setup Object type for page
        $view = array_key_exists('view', $queryVars) ? $queryVars['view'] : '';
        // We return a specific category match first, then go for a matching directory
        $bestObject = null;
        if ($view == 'article')
        {
            if (!isset($queryVars['id']) || !$queryVars['id'] || !$this->isValidArticle($queryVars['id'])) return null;

            $objectTypes = $this->getObjects('listing');
            if(JVERSION < 4.0)
            {
                BaseDatabaseModel::addIncludePath(JPATH_SITE . '/components/com_content/models');
                $contentModel = BaseDatabaseModel::getInstance('Article', 'ContentModel');
            }
            else
            {
                $mvcFactory = Factory::getApplication()->bootComponent('com_content')->getMVCFactory();
                $contentModel = $mvcFactory->createModel('Article', 'Administrator', ['ignore_request' => true]);
            }
            $item = $contentModel->getItem($queryVars['id']);
            $itemCatId = $item->catid;

            $this->db->setQuery("SELECT dirid FROM #__jreviews_categories WHERE id = " . $itemCatId);
            $itemDirId = $this->db->loadResult();

            foreach ($objectTypes as $object)
            {
                // check if directory is set, if so, check that. If not, check categories
                $dirId = $object->params->get('directory', 0);
                if ($dirId != 0)
                {
                    if ($itemDirId == $dirId)
                        $bestObject = $object;
                } else
                {
                    $objectCatId = $object->params->get('category', 0);
                    if ($itemCatId == $objectCatId)
                        return $object;
                }
            }
        }
        return $bestObject;
    }

    protected function setOpenGraphTags()
    {
        $desc = '';
        $desc_type = 'custom_desc_introwords'; //Note: meta is same as blank value, since system plugin attempts to generate from metadescription if no value is found
        $desc_length = '50';

        $view = $this->app->input->getCmd('view');

        if ($this->object)
        {
            $desc_type = $this->object->params->get('custom_desc_type');
            $desc_length = $this->object->params->get('custom_desc_length');
        }

        if ($view == 'article')
        {
            $id = $this->app->input->getInt('id');

            $article = $this->tryGetArticle($id);
            if(!$article)
                return;
            /*
            //Content Model throws error even though content is in DB
            $contentModel = BaseDatabaseModel::getInstance('Article', 'ContentModel');
            if(!$this->isValidArticle($id))
                return;
            $article = $contentModel->getItem($id);
            */

            $rawImages = $article->images;
            $rawImages = explode("\n", $rawImages);
            foreach ($rawImages as $image)
            {
                $link = explode("|", $image);
                if (count($link) > 1)
                    $this->addOpenGraphTag('image', Uri::base() . 'images/' . $link[0], false);
            }

            if ($desc_type == 'custom_desc_introwords')
                $desc = $this->getFirstArticleText($article, $desc_length, SC_INTRO_WORDS);
            else if ($desc_type == 'custom_desc_introchars')
                $desc = $this->getFirstArticleText($article, $desc_length, SC_INTRO_CHARS);
            $this->addOpenGraphTag('description', $desc, true);
        }
        else if($view == 'category')
        {
            $catid = $this->app->input->getInt('id');

            $category = $this->tryGetCategory($catid);
            if(!$category)
                return;

            $reg = new Registry();
            if (!empty($category->params))
            {
                $params = (array) json_decode($category->params);
                $reg->loadArray($params);
                $image = $reg->get('image');
                $fullImagePath = $this->getImageLink($image);
                $this->addOpenGraphTag('image', $fullImagePath, false);
            }
        }
    }

    /*
    Routing issues with JReviews / Joomla 3.6.x
    */
    protected function tryGetArticle($id)
    {
        list($articleId, $alias) = array_pad(explode(':', $id, 2), 2, '');
        $this->db->setQuery("SELECT * FROM #__content WHERE id=" . $articleId);
        $article = $this->db->loadObject();
        return $article;
    }

    protected function isValidArticle($id)
    {
        $article = $this->tryGetArticle($id);
        if(!$article)
            return false;
        else
            return true;
    }

    protected function tryGetCategory($catid)
    {
        $this->db->setQuery("SELECT * FROM #__categories WHERE id=" . $catid);
        $category = $this->db->loadObject();
        return $category;
    }

}