<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// no direct access
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Factory;
use Joomla\CMS\Helper\ModuleHelper;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;

jimport('joomla.filesystem.file');
if (!File::exists(JPATH_ROOT . '/components/com_jfbconnect/libraries/factory.php'))
{
    echo "JFBConnect not found. Please reinstall.";
    return;
}

if (!class_exists('JFBCFactory'))
{
    echo "JFBConnect not enabled. Please enable.";
    return;
}

$showType = $params->get('show_type');
$isMobileDevice = Factory::getApplication()->client->mobile;

//If Mobile and selected to show only on desktop
//If Desktop and selected to show only on mobile
if(($isMobileDevice && $showType == 'desktop') || (!$isMobileDevice && $showType == 'mobile'))
    return;

require_once(dirname(__FILE__) . '/helper.php');
$helper = new modJFBCSocialShareHelper($params);

$renderKey = JFBCFactory::config()->get('social_tag_admin_key');
$renderKeyString = " key=".$renderKey;

//General
$url = $params->get('url');
$href = '';
if(!$url)
{
    $app = Factory::getApplication();
    $option = $app->input->getCmd('option');
    $view = $app->input->getCmd('view');
    $id = $app->input->getInt('id');

    if($option == 'com_content' && $view == 'article')
    {
        if(JVERSION < 4.0)
        {
            BaseDatabaseModel::addIncludePath(JPATH_SITE . '/components/com_content/models');
            $contentModel = BaseDatabaseModel::getInstance('Article', 'ContentModel');
        }
        else
        {
            $mvcFactory = Factory::getApplication()->bootComponent('com_content')->getMVCFactory();
            $contentModel = $mvcFactory->createModel('Article', 'Administrator', ['ignore_request' => true]);
        }
        $article = $contentModel->getItem($id);
        $url = SCArticleContent::getCurrentURL($article, true);
    }
    else if($option == 'com_k2' && $view == 'item')
    {
        $itemModel = BaseDatabaseModel::getInstance('Item', 'K2Model');
        $item = $itemModel->getData();
        $url = SCArticleContent::getCurrentURL($item, false);
    }
    else
        $url = JFBConnectUtilities::getStrippedUrl();
}

if ($url)
    $href = ' href='.$url;
$layout = $params->get('layout_style');
$orientation = $params->get('orientation');
$position = $params->get('position');
$margin = $params->get('margin');
$padding = $params->get('padding');
$backgroundColor = $params->get('background_color');
$floatTop = trim($params->get('float_position_top'));
$floatLeft = trim($params->get('float_position_left'));

//Advanced
$userIntro = $params->get('user_intro');

//Facebook
$facebookEnable = $params->get('facebook_enable');
$facebookShareEnable = $params->get('facebook_share_enable');
$facebookShowFaces = $params->get('facebook_show_faces');
$facebookKidDirectedSite = $params->get('facebook_kid_directed_site');
$facebookWidth = $params->get('facebook_width');
$facebookHeight = $params->get('facebook_height');
$facebookVerb = $params->get('facebook_verb_to_display');
$facebookColorScheme = $params->get('facebook_color_scheme');
$facebookRef = $params->get('facebook_ref');
$facebookShowFacesValue = ($facebookShowFaces?'true':'false');
$facebookKidDirectedSiteValue = ($facebookKidDirectedSite?'true':'false');
$facebookShowShareButtonValue = ($facebookShareEnable?'true':'false');
$facebookHeight = $helper->addPxToString($facebookHeight);
$facebookWidth = $helper->addPxToString($facebookWidth);

//LinkedIn
$linkedinEnable = $params->get('linkedin_enable');

//Twitter
$twitterEnable = $params->get('twitter_enable');

if($position == 'fixed') //Float
{
    $groupStyles = 'position: ' . $position .";";

    if(intval($floatTop) < 3000)
    {
        $floatTop = $helper->addPxToString($floatTop == '' ? "0" : $floatTop);
        $groupStyles .= 'top:'.$floatTop . ";";
    }
    else // Float to bottom instead of offset
        $groupStyles .= 'bottom:0px;';

    if(intval($floatLeft) < 3000)
    {
        $floatLeft = $helper->addPxToString($floatLeft == '' ? "0" : $floatLeft);
        $groupStyles .= 'left:'.$floatLeft . ";";
    }
    else // Float to right instead of offset
        $groupStyles .= 'right:0px;';

    $groupStyles .= 'width: auto;';
}
else
    $groupStyles = '';

if($margin != '')
    $groupStyles .= 'margin:'.$helper->addPxToString($margin).";";
if($padding != '')
    $groupStyles .= 'padding:'.$helper->addPxToString($padding).";";
if($backgroundColor != '')
    $groupStyles .= 'background-color:'.$backgroundColor.';';

require(ModuleHelper::getLayoutPath('mod_jfbcsocialshare'));
?>
