<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\Registry\Registry;
use Joomla\CMS\Plugin\PluginHelper;

// Probably should work toward extending the JFBConnectFacebookLibrary for this class.
// Too much intertwined in the root library right now.
class JFBConnectProfileFacebookbusiness extends JFBConnectProfile
{
    /**
     *  Get all permissions that are required by Facebook for email, status, and/or profile, regardless
     *    of whether they're set to required in JFBConnect
     * @return string Comma separated list of FB permissions that are required
     */
    static private $requiredScope;

    public function getRequiredScope()
    {
        if (self::$requiredScope)
            return self::$requiredScope;

        self::$requiredScope = array();
        self::$requiredScope[] = "email";

        return self::$requiredScope;
    }

    /*
     * Fetch a user's profile based on a profile plugin field-mapping
     * @return Registry with profile field values
     */
    public function fetchProfileFromFieldMap($fieldMap, $permissionGranted = true)
    {
        $fields = array();
        if (is_object($fieldMap))
        {
            foreach ($fieldMap as $field)
            {
                $fieldArray = explode('.', $field);
                if (!empty($fieldArray[0]))
                    $fields[] = $fieldArray[0]; // Get the root field to grab from FB
            }
        }
        $fbUserId = JFBCFactory::provider('facebookbusiness')->getProviderUserId();

        $fields = array_unique($fields);
        $profile = $this->fetchProfile($fbUserId, $fields);
        $profile->setFieldMap($fieldMap);
        return $profile;

    }

    /* Fetch a user's profile based on the passed in array of fields
    * @return Registry with profile field values
    */
    public function fetchProfile($fbUserId, $fields)
    {
        if (in_array('full_name', $fields))
        {
            $fields[] = 'name';
            unset($fields[array_search('full_name', $fields)]);
        }

        $profile = new JFBConnectProfileDataFacebook();
        if (!empty($fields))
        {
            $colFields = implode(",", $fields);
            $data = JFBCFactory::provider('facebookbusiness')->api($fbUserId . '?fields=' . $colFields);
            $profile->loadObject($data);
        }
        return $profile;
    }

    // nullForDefault - If the avatar is the default image for the social network, return null instead
    // Prevents the default avatars from being imported
    function getAvatarUrl($providerUserId, $nullForDefault = false, $params = null)
    {
        if (!$params)
            $params = new Registry();

        $width = $params->get('width', 300);
        $height = $params->get('height', 300);

        $nullString = $nullForDefault ? 'null' : 'notnull';
        $avatarUrl = JFBCFactory::cache()->get('facebookbusiness.avatar.' . $nullString . '.' . $providerUserId . '.' . $width . 'x' . $height);
        if ($avatarUrl === false)
        {
            $avatarData = JFBCFactory::provider('facebookbusiness')->api('/' . $providerUserId . '/picture/?width=' . $width . "&height=" . $height . '&return_ssl_resources=1&redirect=false');
            if (is_array($avatarData) && array_key_exists('data', $avatarData))
            {
                if ($avatarData['data']['is_silhouette'] && $nullForDefault)
                    $avatarUrl = null;
                else
                    $avatarUrl = $avatarData['data']['url'];
            }
            JFBCFactory::cache()->store($avatarUrl, 'facebookbusiness.avatar.' . $nullString . '.' . $providerUserId . '.' . $width . 'x' . $height);
        }

        return $avatarUrl;
    }

}

class JFBConnectProfileDataFacebookbusiness extends JFBConnectProfileData
{
    var $fieldMap;

    function get($path, $default = null)
    {
        if ($this->exists($path))
            $data = parent::get($path, $default);
        else if ($path == "full_name") // standardized provider value for full name
            $data = parent::get('name', $default);
        else
            return $default;

        return $data;
    }
}