<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;

class JFBConnectProviderFacebookChannelProfile extends JFBConnectChannel
{
    var $name = "Profile";

    public function setup()
    {
        $this->name = "Profile";
        $this->outbound = true;
        $this->inbound = true;
        $this->requiredScope[] = 'user_posts';
        $this->postSuccessMessage = Text::sprintf('COM_JFBCONNECT_CHANNELS_POST_SUCCESS', Text::_('COM_JFBCONNECT_CHANNELS_POST_CHANNEL_FACEBOOK'));
    }

    public function canPostToNetwork()
    {
        return false;
    }

    public function canPublish($data)
    {
        return true;
    }

    public function getStream($stream)
    {
        $jid = $this->options->get('user_id');
        $uid = JFBCFactory::usermap()->getProviderUserId($jid, 'facebook');

        if (!$jid || !$uid)
            return null;

        $feed = JFBCFactory::cache()->get('facebook.profile.feed.' . $uid);

        $isAdmin = Factory::getApplication()->isClient('administrator');
        if ($feed === false || $isAdmin) // Always fetch fresh in the admin area
        {
            $params = array();
            $params['access_token'] = JFBCFactory::usermap()->getUserAccessToken($this->options->get('user_id'), 'facebook');
            $feed = $this->provider->api($uid . '/feed?fields=message,from,updated_time,name,link,caption,description,comments,picture,full_picture,privacy', $params, true, 'GET');
            JFBCFactory::cache()->store($feed, 'facebook.profile.feed.' . $uid);
        }

        if (is_array($feed) && $feed['data']) {
            //check to either show thumnail or full image of the preview
            $size_image = $this->options->get('size_image');
            $key = $size_image == 'full' ? 'full_picture' : 'picture';

            foreach ($feed['data'] as $data) {
//                if(array_key_exists('message', $data))
//                {
                $post = new JFBConnectPost();
                if ($data['privacy']['value'] != 'EVERYONE')
                {
                    if($isAdmin)
                    {
                        $post->message = Text::_('COM_JFBCONNECT_CHANNEL_FACEBOOK_POST_SKIPPED_LABEL');
                        $stream->addPost($post);
                    }
                    continue;
                }

                if (isset($data['actions'][0])) {
                    $post->link = $data['actions'][0]['link'];
                } else {
                    $ids = explode("_", $data['id']);
                    $idIndex = count($ids) - 1;
                    $post->link = 'https://www.facebook.com/' . $uid . '/posts/' . $ids[$idIndex];
                }

                $post->message = (array_key_exists('message', $data) ? $data['message'] : "");
                $post->authorID = $data['from']['id'];
                $post->authorScreenName = $data['from']['name'];
                $post->updatedTime = (array_key_exists('updated_time', $data) ? $data['updated_time'] : "");
                $post->thumbTitle = (array_key_exists('name', $data) ? $data['name'] : "");
                $post->thumbLink = (array_key_exists('link', $data) ? $data['link'] : "");
                $post->thumbPicture = (array_key_exists($key, $data) ? $data[$key] : "");
                $post->thumbCaption = (array_key_exists('caption', $data) ? $data['caption'] : "");
                $post->thumbDescription = (array_key_exists('description', $data) ? $data['description'] : "");
                $post->comments = (array_key_exists('comments', $data) ? $data['comments'] : "");

                $stream->addPost($post);
//                }
            }
        }
    }
}