<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// no direct access
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Form\FormField;
use Joomla\CMS\Factory;

jimport('joomla.version');
jimport('joomla.form.helper');

class JFormFieldScope extends FormField
{
    var $autotuneSettingsLoaded;
    var $scopes;

    protected function getLabel()
    {
        $doc = Factory::getApplication()->getDocument();
        $doc->addStyleDeclaration('#facebook_scope .control-label { display:none; }');
        $doc->addStyleDeclaration('#facebook_scope .controls { margin-left:0px; }');
    }

    protected function getInput()
    {
        $this->init();

        $html = '<div class="sourcecoast ns-bs5">'
            .'<div class="row">'
            .'<div class="col-md-12">'
            .'<div class="options-form p-2">'
            .'<p>'.Text::_('COM_JFBCONNECT_SCOPE_REVIEW_DESCRIPTION').'</p>'
            .'<p>'.Text::_('COM_JFBCONNECT_SCOPE_REVIEW_FACEBOOK_DESCRIPTION').'</p>';

        if($this->autotuneSettingsLoaded)
        {
            $html .= "<h3>" . Text::_('COM_JFBCONNECT_SCOPE_REVIEW_REQUESTED') . ":</h3>";
            $html .= "<ul>";

            if (count($this->scopes->profiles) > 0)
            {
                $html .= "<li>" . Text::_('COM_JFBCONNECT_MENU_PROFILES') . ":";
                $html .= "<ul>";
                foreach ($this->scopes->profiles as $scope => $details)
                {
                    $html .= "<li><strong>" . $scope . "</strong> - " . $details->description . "</li>";
                }
                $html .= "</ul></li>";
            }

            if (count($this->scopes->custom) > 0)
            {
                $html .= "<li>" . Text::_('COM_JFBCONNECT_SCOPE_REVIEW_CUSTOM_SCOPE') . ":";
                $html .= "<ul>";
                foreach ($this->scopes->custom as $scope => $details)
                {
                    $html .= "<li><strong>" . $scope . "</strong> - " . $details->description . "</li>";
                }
                $html .= "</ul></li>";
            }

            $html .= "</ul>";
        }
        else
        {
            $html .= "<p><strong>".Text::_('COM_JFBCONNECT_SCOPE_REVIEW_NO_SCOPES_LOADED')."</strong></p>";
        }
        $html .= "<p>".Text::_('COM_JFBCONNECT_SCOPE_REVIEW_DOCUMENTATION_LINK')."</p>";

        $html .= '</div></div></div></div>';

        return $html;
    }

    function init()
    {
        $fbProvider = JFBCFactory::provider('facebook');

        $this->autotuneSettingsLoaded = $fbProvider->networkSettings->exists('scopes');
        if (!$this->autotuneSettingsLoaded)
        {
            Factory::getApplication()->enqueueMessage(Text::_('COM_JFBCONNECT_SCOPE_REVIEW_AUTOTUNE_REQUIRED'), 'error');
        }
        else
        {
            // Only doing Facebook for now..
            $requiredScopes = new stdClass();
            $requiredScopes->profiles = $fbProvider->profile->getRequiredScope();

            $customScope = $fbProvider->getCustomScopes();
            $requiredScopes->custom = [];
            foreach ($customScope as $cs)
                $requiredScopes->custom[] = $cs;

            $scopes = new stdClass();
            foreach (['profiles', 'custom'] as $type)
            {
                $data = [];
                foreach ($requiredScopes->$type as $scope)
                {
                    $data[$scope] = $fbProvider->getScopeDetails($scope);
                }
                $scopes->$type = $data;
            }

            $this->scopes = $scopes;
        }
    }
}
?>