<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Factory;
use Joomla\Registry\Registry;

class JFBConnectProviderGoogle extends JFBConnectProvider
{
    function __construct()
    {
        $this->name = "Google";
        $this->usernamePrefix = "g_";
        $this->loginPrefix = "go";

        parent::__construct();
    }

    public function setupAuthentication()
    {
        $options = new Registry();
        $options->set('authurl', 'https://accounts.google.com/o/oauth2/v2/auth');
        $options->set('tokenurl', 'https://www.googleapis.com/oauth2/v4/token');

        $scope = implode(' ', $this->profile->getRequiredScope());
        $options->set('scope', $scope);

        $this->client = new JFBConnectAuthenticationOauth2($options);

        $token = Factory::getApplication()->getUserState('com_jfbconnect.' . $this->systemName . '.token', null);
        if ($token)
        {
            $token = (array)json_decode($token);
            $this->client->setToken($token);
        }
        $this->client->initialize($this);
    }

    /* getProviderUserId
    * Gets the provider User IdFacebook. This is regardless of whether they are mapped to an
    *  existing Joomla account.
    */
    function getProviderUserId()
    {
        if ($this->get('providerUserId', null) == null)
        {
            $profile = $this->profile->fetchProfile('me', 'id');
            $id = $profile->get('id');
            if (!empty($id))
                $this->set('providerUserId', $id);
            else
                $this->set('providerUserId', null);
        }
        return $this->get('providerUserId');
    }
}