<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\Registry\Registry;

class JFBConnectProviderLinkedin extends JFBConnectProvider
{
    var $timestampOffset;

    private $apiError;

    function __construct()
    {
        $this->name = "LinkedIn";
        $this->usernamePrefix = "li_";
        $this->loginPrefix = "li";

        parent::__construct();
    }

    function setupAuthentication()
    {
        $options = new Registry();
        $options->set('authurl', 'https://www.linkedin.com/oauth/v2/authorization');
        $options->set('tokenurl', 'https://www.linkedin.com/oauth/v2/accessToken');
        $options->set('authmethod', 'bearer');
        $options->set('getparam', 'access_token');

        $options->set('scope', implode(' ', $this->profile->getRequiredScope()));

        $headers = array();
        $headers['X-Restli-Protocol-Version'] = '2.0.0';
        $headers['Content-Type'] = "application/json";
        $options->set('headers', $headers);

        $this->client = new JFBConnectAuthenticationOauth2($options);

        $token = Factory::getApplication()->getUserState('com_jfbconnect.' . $this->systemName . '.token', null);
        if ($token) {
            $token = (array)json_decode($token);
            $this->client->setToken($token);
        }
        $this->client->initialize($this);
    }

    function fetchNewScope($scope)
    {
        $session = Factory::getSession();
        // With LinkedIn, we can't check if a scope is granted, so we need to always fetch new scope in addition to
        // previous scope. We also need to check if we're coming back from a previous scope fetch and just proceed if so
        // That prevents authentication looping.
        $alreadyFetching = $session->get('jfbconnect.linkedin.fetchingscope', false);
        if (!$alreadyFetching) {
            Factory::getSession()->set('jfbconnect.linkedin.fetchingscope', true);
            // Clear the return code that may already be set, allowing for authentication to happen again
            Factory::getApplication()->input->set('code', null);
            $this->client->setOption('scope', implode(' ', $this->profile->getRequiredScope()) . ' ' . implode(' ', $scope));
            $this->client->authenticate();
        }
    }

    /* getProviderUserId
    * Gets the provider User IdFacebook. This is regardless of whether they are mapped to an
    *  existing Joomla account.
    */
    function getProviderUserId()
    {
        if ($this->get('providerUserId', null) == null) {
            $profile = $this->profile->fetchProfile('~', 'id');
            $id = $profile->get('id');
            if (!empty($id))
                $this->set('providerUserId', $id);
            else
                $this->set('providerUserId', null);
        }
        return $this->get('providerUserId');
    }

    /*function setTimestampOffset($offset)
    {
        $this->timestampOffset = $offset;
    }*/

    function getHeadData()
    {
        $inJS = '';
        if ($this->needsJavascript && !empty($this->appId)) {
            $initJS = array_flip(array_flip($this->addJavascriptInit())); //flip to avoid the same value

            $extraJS = '';
            if ($count = count($initJS)) {
                $extraJS = ",\n";
                $i = 0;
                foreach ($initJS as $xtrajs) {
                    $extraJS .= $xtrajs;
                    $extraJS .= $i == $count ? ",\n" : "\n";
                    $i++;
                }
            }

            $inJS = '<script type="text/javascript" src="//platform.linkedin.com/in.js?async=true"></script>' . "\n";
            $inJS .= '<script type="text/javascript">' .
                "IN.init({\n" .
                "api_key: '" . $this->appId . "',\n" .
                'authorize: false' .
                $extraJS .
                '});' .
                '</script>';
        }

        return $inJS;
    }

    public function onAfterRender()
    {
        if ($this->needsJavascript && empty($this->appId)) {
            $app = Factory::getApplication();
            $body = $app->getBody();
            $javascript = '<script type="text/javascript" src="//platform.linkedin.com/in.js"></script>' . "\n";

            $newBody = str_ireplace("</body>", $javascript . "</body>", $body, $count);
            if ($count == 1)
                $app->setBody($newBody);
        }
    }

    public function query($url, $data = null, $headers = array(), $method = 'get', $timeout = null)
    {
        $this->apiError = null;
        try {
            $apiData = $this->client->query($url, $data, $headers, $method, $timeout);
        } catch (RuntimeException $e) {
            $this->apiError = $e->getMessage();
            if (JFBCFactory::config()->get('facebook_display_errors') && $this->apiError)
                JFBCFactory::log(Text::sprintf('COM_JFBCONNECT_PROVIDER_API_ERROR', $this->name) . $this->apiError, 'error');
            throw $e;
        }
        return $apiData;
    }

    public function getLastError()
    {
        $return = $this->apiError;
        $this->apiError = null;
        return $return;
    }

}