<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\Registry\Registry;
use Joomla\CMS\Language\Text;

class JFBConnectProviderLinkedinChannelCompany extends JFBConnectChannel
{
    public function setup()
    {
        $this->name = "Company";
        $this->outbound = true;
        $this->inbound = true;
        $this->requiredScope[] = 'r_basicprofile';
        $this->requiredScope[] = 'rw_organization_admin';
        $this->requiredScope[] = 'r_organization_social';
        $this->requiredScope[] = 'w_organization_social';

        $this->postCharacterMax = '700';
        $this->postSuccessMessage = Text::sprintf('COM_JFBCONNECT_CHANNELS_POST_SUCCESS', Text::_('COM_JFBCONNECT_CHANNEL_LINKEDIN_COMPANY_ID_LABEL'));
    }

    public function canPublish($data)
    {
        $canPublish = false;

        $jid = $data['attribs']['user_id'];
        if ($jid)
        {
            $uid = JFBCFactory::usermap()->getProviderUserId($jid, 'linkedin');
            if ($uid && isset($data['attribs']['company_id']) && ($data['attribs']['company_id'] != '--')
                && $this->userHasToken($jid))
            {
                $canPublish = true;
            }
        }
        return $canPublish;
    }

    public static function getCompanyName($liLibrary, $orgId)
    {
        $orgId = str_replace('urn:li:organization:', '', $orgId);

        $nameUrl = 'https://api.linkedin.com/v2/organizations/'.$orgId.'?projection=(name)';

        $name = "ID:" . $orgId;
        try
        {
            $response = $liLibrary->query($nameUrl);
            if(JFBConnectUtilities::getResponseCode($response) == '200')
            {
                $nameData = json_decode(JFBConnectUtilities::getResponseBody($response));
                $country = $nameData->name->preferredLocale->country;
                $language = $nameData->name->preferredLocale->language;
                $locale = $language .'_'.$country;
                $nameData = $nameData->name->localized->$locale;
                $name = ucfirst($nameData);
            }
        }
        catch (Exception $e)
        {
            print_r($e->getMessage());
        }

        return $name;
    }

    public static function getShareLink($liLibrary, $shareId)
    {
        /*
         $shareId = str_replace('urn:li:share:', '', $shareId);
        $shareUrl = 'https://api.linkedin.com/v2/shares/'.$shareId;

        $url = '';
        try
        {
            $response = $liLibrary->query($shareUrl);
            if(JFBConnectUtilities::getResponseCode($response) == '200')
            {
                $nameData = json_decode(JFBConnectUtilities::getResponseBody($response));
                $url = 'https://www.linkedin.com/posts/'.$nameData->id;
            }
        }
        catch (Exception $e)
        {
            print_r($e->getMessage());
        }
        return $url;
        */

    	return 'https://www.linkedin.com/feed/update/'.$shareId;
    }

    private function userHasToken($jid)
    {
        $access_token = JFBCFactory::usermap()->getUserAccessToken($jid, 'linkedin');
        $params['access_token'] = $access_token;
        $liLibrary = JFBCFactory::provider('linkedin');
        $liLibrary->client->setToken((array)$access_token);

        $url = 'https://api.linkedin.com/v2/organizationalEntityAcls?q=roleAssignee';
        try
        {
            $companies = $liLibrary->query($url);
            if (JFBConnectUtilities::getResponseCode($companies) == '200')
            {
                $body = JFBConnectUtilities::getResponseBody($companies);
                $this->companies = json_decode($body);
            }
            return true;
        }
        catch (Exception $e)
        {
            return false;
        }
        return true;
    }

    public function getStream($stream)
    {
        $user = $this->options->get('user_id');
        if (!$user)
            return;

        // Access token for channel needs to be set here for all queries to
        // successfully perform API endpoints
        $access_token = JFBCFactory::usermap()->getUserAccessToken($user, 'linkedin');
        if (!is_object($access_token) || ($access_token->created + $access_token->expires_in < time()))
                return;
                
        $this->provider->client->setToken((array)$access_token);

        // Now get API endpoints
        $companyId = $this->options->get('company_id');
        $companyName = self::getCompanyName($this->provider->client, $companyId);

        $feed = JFBCFactory::cache()->get('linkedin.stream.' . $companyId);
        if ($feed === false)
        {
            $urn = urlencode('urn:li:organization:'. $companyId);
            $url = 'https://api.linkedin.com/v2/ugcPosts?q=authors&authors=List('.$urn.')';

            try
            {
                $feedResponse = $this->provider->query($url, json_encode(array()), array(), 'get');
                if (JFBConnectUtilities::getResponseCode($feedResponse) != 200)
                    return;

                $body = JFBConnectUtilities::getResponseBody($feedResponse);
                $feed = json_decode($body);
                JFBCFactory::cache()->store($feed, 'linkedin.stream.' . $companyId);
            }
            catch (Exception $e)
            {
                return;
            }
        }

        if(isset($feed->elements) && $feed->elements)
        {
            $public = 'com.linkedin.ugc.MemberNetworkVisibility';
            $shareContent = 'com.linkedin.ugc.ShareContent';

            foreach($feed->elements as $data)
            {
                if($data->lifecycleState == 'PUBLISHED' && $data->visibility->$public == 'PUBLIC')
                {
                    $post = new JFBConnectPost();

                    $post->message = $data->specificContent->$shareContent->shareCommentary->text;

                    $picData = isset($data->specificContent->$shareContent->media[0]) ? $data->specificContent->$shareContent->media[0]: null;
                    $pic = $picData->originalUrl;
                    if (strpos($pic, "slidesharecdn"))
                        $pic = str_replace("http://", "https://", $pic);
                    $post->thumbPicture = $pic;
                    $post->thumbDescription = $picData->description->text;
                    $post->thumbLink = $picData->thumbnails[0]->url;
                    $post->link = self::getShareLink($this->provider->client, $data->id);
                    $post->authorScreenName = $companyName;

                    if (isset($data->lastModified))
                    {
                        $timestamp = intval($data->lastModified->time / 1000);
                        $post->updatedTime = gmdate($stream->options->get('datetime_format'), $timestamp);
                    }
                    else
                        $post->updatedTime = "";

                    $stream->addPost($post);
                }
            }
        }
    }

    public function performPost(Registry $data)
    {
        $user = $this->options->get('user_id');
        $access_token = JFBCFactory::usermap()->getUserAccessToken($user, 'linkedin');

        $this->provider->client->setToken((array)$access_token);

        $companyId = $this->options->get('company_id');

        $link = $data->get('link', '');
        $message = $data->get('message', '');

        $url = 'https://api.linkedin.com/v2/ugcPosts';
        $vals =
            <<<EOT
{
    "author": "urn:li:organization:{$companyId}",
    "lifecycleState": "PUBLISHED",
    "specificContent": {
        "com.linkedin.ugc.ShareContent": {
            "shareCommentary": {
                "text": "{$message}"
            },
            "shareMediaCategory": "ARTICLE",
            "media": [
                {
                    "status": "READY",
                    "originalUrl": "{$link}"
                }
            ]
        }
    },
    "visibility": {
        "com.linkedin.ugc.MemberNetworkVisibility": "PUBLIC"
    }
}
EOT;

        $return = $this->provider->query($url, $vals, array(), 'post');
        return $return;
    }
}
