<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Form\FormHelper;

jimport('joomla.form.helper');
FormHelper::loadFieldClass('list');

class JFormFieldLinkedinCompanyList extends JFormFieldList
{
    public $type = 'LinkedinCompanyList';
    private $companies;
    private $liLibrary;

    protected function getOptions()
    {
        $options = array();
        $options[] = HTMLHelper::_('select.option', "--", "-- Select a Company --");

        if(isset($this->companies->elements) && count($this->companies->elements) > 0)
        {
            foreach($this->companies->elements as $c)
            {
                if($c->role == 'ADMINISTRATOR')
                {
                    $orgId = $c->organizationalTarget;
                    $orgId = str_replace('urn:li:organization:', '', $orgId);
                    $name = JFBConnectProviderLinkedinChannelCompany::getCompanyName($this->liLibrary, $orgId);
                    $options[] = HTMLHelper::_('select.option', strtolower($orgId), $name);
                }
            }
        }

        return $options;
    }

    function getInput()
    {
        $jid = $this->form->getValue('attribs.user_id');
        if ($jid)
        {
            $uid = JFBCFactory::usermap()->getProviderUserId($jid, 'linkedin');
            if ($uid)
            {
                $access_token = JFBCFactory::usermap()->getUserAccessToken($jid, 'linkedin');
                $params['access_token'] = $access_token;
                $this->liLibrary = JFBCFactory::provider('linkedin');
                $this->liLibrary->client->setToken((array)$access_token);

                $url = 'https://api.linkedin.com/v2/organizationalEntityAcls?q=roleAssignee';
                try
                {
                    $companies = $this->liLibrary->query($url);
                    if (JFBConnectUtilities::getResponseCode($companies) == '200')
                    {
                        $this->companies = json_decode(JFBConnectUtilities::getResponseBody($companies));
                    }
                    return parent::getInput();
                }
                catch (Exception $e)
                {
                    return '<div class="jfbc-error">'.Text::_('COM_JFBCONNECT_CHANNEL_LINKEDIN_PERM_TOKEN_EXPIRED_LABEL').'</div>';
                }
            }
            else
            {
                return '<div class="jfbc-error">'.Text::_('COM_JFBCONNECT_CHANNEL_LINKEDIN_PERM_USER_AUTH_ERROR_LABEL').'</div>';
            }

        }
        else
            return '<div class="jfbc-error">'.Text::_('COM_JFBCONNECT_CHANNEL_SELECT_USER_ERROR_LABEL').'</div>';
    }
}
