<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\Registry\Registry;
use Joomla\CMS\Language\Text;

class JFBConnectProviderTwitterChannelStream extends JFBConnectChannel
{
    public function setup()
    {
        $this->name = "Stream";
        $this->outbound = true;
        $this->inbound = true;
        $this->requiredScope[] = 'write';
        $this->postCharacterMax = $this->provider->networkSettings->get('channel_char_count', 140);
        $this->urlLength = $this->provider->networkSettings->get('channel_url_length', 23);
        $this->postSuccessMessage = Text::sprintf('COM_JFBCONNECT_CHANNELS_POST_SUCCESS', Text::_('COM_JFBCONNECT_CHANNELS_POST_CHANNEL_TWITTER'));
    }

    public function canPublish($data)
    {
        $canPublish = false;

        $jid = $data['attribs']['user_id'];
        if ($jid)
        {
            $uid = JFBCFactory::usermap()->getProviderUserId($jid, 'twitter');
            if ($uid)
            {
                $canPublish = true;
            }
        }
        return $canPublish;
    }

    public function getStream($stream)
    {
        $user = $this->options->get('user_id');
        if (!$user)
            return;

        $feed = JFBCFactory::cache()->get('twitter.stream.' . $user);
        if ($feed === false)
        {
            $accessToken = JFBCFactory::usermap()->getUserAccessToken($user, 'twitter');
            if (!$accessToken)
                return;

            $params = array();
            $params['oauth_token'] = $accessToken->key;
            $this->provider->client->setToken((array)$accessToken);

            $data = array();
            $data['include_rts'] = $this->options->get('include_rts') ? 'true' : 'false';

            $path = $this->provider->options->get('api.url') . 'statuses/user_timeline.json';
            $feedResponse = $this->provider->client->oauthRequest($path, 'GET', $params, $data);
            if (!$feedResponse || JFBConnectUtilities::getResponseCode($feedResponse) != 200)
                return array();

            $body = JFBConnectUtilities::getResponseBody($feedResponse);
            $feed = json_decode($body);

            JFBCFactory::cache()->store($feed, 'twitter.stream.' . $user);
        }

        if($feed)
        {
            foreach ($feed as $data)
            {
                $post = new JFBConnectPost($this);

                $tweet = $data->retweeted ? $data->retweeted_status : $data;

                $post->message = (isset($tweet->text) ? $tweet->text : "");
                $post->authorScreenName = '@' . $tweet->user->screen_name;
                $post->authorName = $tweet->user->name;
                $post->authorImage = $tweet->user->profile_image_url_https;
                $post->updatedTime = (isset($data->created_at) ? $data->created_at : "");
                $post->thumbLink = (isset($tweet->entities->urls[0]) ? $tweet->entities->urls[0]->expanded_url : "");
                $post->thumbPicture = (isset($tweet->entities->media[0]) ? $tweet->entities->media[0]->media_url_https : "");
                $post->link = 'https://twitter.com/'.$tweet->user->screen_name.'/status/'.$data->id_str;

                $stream->addPost($post);
            }
        }
    }

    public function performPost(Registry $data)
    {
        $path = $this->provider->options->get('api.url') . 'statuses/update.json';
        $message = $data->get('message', '');
        $link = $data->get('link', '');

        // t.co length. Should update this to use the help/configuration check to verify it hasn't grown.
        $urlLength = $this->urlLength; // +1 for https
        $status = substr($message, 0, $this->postCharacterMax - $urlLength - 1) . " " . $link;

        $user = $this->options->get('user_id');
        $accessToken = JFBCFactory::usermap()->getUserAccessToken($user, 'twitter');

        if($accessToken)
        {
            $params = array();
            $params['oauth_token'] = $accessToken->key;
            $this->provider->client->setToken((array)$accessToken);

            $data = array();
            $data['status'] = $status;

            $return = $this->provider->client->oauthRequest($path, 'POST', $params, $data);
        }
        else
            $return = false;

        return $return;
    }
}